/*  Melbourne University Game-based Learning Environment
 *  This file is hereby placed into the public domain by The University of
 *  Melbourne, for use in MUGLE projects or otherwise.
 */

/* This test app is a brief demonstration of the MUGLE Developer API.
 * It contains basic UI for interacting with all of the API features.
 */

package au.edu.unimelb.csse.apitest.client;

import java.io.Serializable;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.FlexTable;

import au.edu.unimelb.csse.mugle.client.api.Services;
import au.edu.unimelb.csse.mugle.client.api.UserServiceAsync;
import au.edu.unimelb.csse.mugle.client.api.BadgeServiceAsync;
import au.edu.unimelb.csse.mugle.client.api.HighscoreServiceAsync;
import au.edu.unimelb.csse.mugle.client.api.KeyValueServiceAsync;
import au.edu.unimelb.csse.mugle.shared.api.Badge;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class APITest implements EntryPoint {
    /** The game token is a unique string which identifies the game to the
     * server, and should be kept secret.
     * This string needs to be passed to most API calls.
     * When uploading to the production MUGLE, this string needs to be
     * replaced with the one displayed in your Game Edit screen.
     */
    private static final String gameToken =
        "8b7713ae-6441-4374-a5b4-593d4a1f24a9";

    /**
     * This is the entry point method.
     */
    public void onModuleLoad() {
        RootPanel root = RootPanel.get("contents");
        VerticalPanel vp = new VerticalPanel();
        root.add(vp);

        // Test User service
        final Label lblUserName = new Label();
        lblUserName.setText("User nickname: ...");
        vp.add(lblUserName);
        Services.user.getUserNickName(new AsyncCallback<String>() {
            @Override
            public void onFailure(Throwable error)
            {
                Window.alert(error.toString());
            }
            @Override
            public void onSuccess(String nick)
            {
                lblUserName.setText("User nickname: " + nick);
            }
        });
        final Label lblUserID = new Label();
        lblUserID.setText("User ID: ...");
        vp.add(lblUserID);
        Services.user.getUserID(new AsyncCallback<String>() {
            @Override
            public void onFailure(Throwable error)
            {
                Window.alert(error.toString());
            }
            @Override
            public void onSuccess(String nick)
            {
                lblUserID.setText("User ID: " + nick);
            }
        });

        // Test KeyValue service
        Label lblKV = new Label();
        lblKV.setText("Key value service");
        vp.add(lblKV);

        HorizontalPanel hzpKVPut = new HorizontalPanel();
        vp.add(hzpKVPut);
        final TextBox txbKVPutKey = new TextBox();
        hzpKVPut.add(txbKVPutKey);
        final TextBox txbKVPutValue = new TextBox();
        hzpKVPut.add(txbKVPutValue);
        Button btnKVPut = new Button();
        btnKVPut.setText("Put");
        hzpKVPut.add(btnKVPut);
        btnKVPut.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Services.keyvalue.put(gameToken, txbKVPutKey.getText(),
                    txbKVPutValue.getText(), new AsyncCallback<Void>() {
                @Override
                public void onFailure(Throwable error)
                {
                    Window.alert(error.toString());
                }
                @Override
                public void onSuccess(Void result)
                {
                    Window.alert("Done");
                }
                });
            }
        });

        HorizontalPanel hzpKVGet = new HorizontalPanel();
        vp.add(hzpKVGet);
        final TextBox txbKVGetKey = new TextBox();
        hzpKVGet.add(txbKVGetKey);
        Button btnKVGet = new Button();
        btnKVGet.setText("Get");
        hzpKVGet.add(btnKVGet);
        btnKVGet.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Services.keyvalue.get(gameToken, txbKVGetKey.getText(),
                    new AsyncCallback<Serializable>() {
                @Override
                public void onFailure(Throwable error)
                {
                    Window.alert(error.toString());
                }
                @Override
                public void onSuccess(Serializable value)
                {
                    Window.alert(value.toString());
                }
                });
            }
        });
        Button btnKVContains = new Button();
        btnKVContains.setText("Contains Key?");
        hzpKVGet.add(btnKVContains);
        btnKVContains.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Services.keyvalue.containsKey(gameToken,txbKVGetKey.getText(),
                    new AsyncCallback<Boolean>() {
                @Override
                public void onFailure(Throwable error)
                {
                    Window.alert(error.toString());
                }
                @Override
                public void onSuccess(Boolean value)
                {
                    Window.alert(value.toString());
                }
                });
            }
        });

        // Test Badge service
        Label lblBadge = new Label();
        lblBadge.setText("Badge service");
        vp.add(lblBadge);

        final FlexTable flxBadge = new FlexTable();
        vp.add(flxBadge);
        flxBadge.setText(0, 0, "Name");
        flxBadge.setText(0, 1, "Display name");
        flxBadge.setText(0, 2, "Description");
        flxBadge.setText(0, 3, "Max progress");
        Services.badges.getBadgeNames(gameToken, new AsyncCallback<String[]>()
        {
            @Override
            public void onFailure(Throwable error)
            {
                Window.alert(error.toString());
            }
            @Override
            public void onSuccess(String[] badgeNames)
            {
                for (int i=0; i<badgeNames.length; i++)
                {
                    flxBadge.setText(i+1, 0, badgeNames[i]);
                    setupBadge(flxBadge, i+1, badgeNames[i]);
                }
            }
        });

        // Test Highscore service
        Label lblHigh = new Label();
        lblHigh.setText("Highscore service");
        vp.add(lblHigh);

        HorizontalPanel hzpHigh = new HorizontalPanel();
        vp.add(hzpHigh);
        final TextBox txbScore = new TextBox();
        hzpHigh.add(txbScore);
        Button btnSaveScore = new Button();
        btnSaveScore.setText("Save score");
        hzpHigh.add(btnSaveScore);
        btnSaveScore.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                int score;
                try
                {
                    score = Integer.parseInt(txbScore.getText());
                }
                catch (NumberFormatException e)
                {
                    Window.alert("Please enter a number");
                    return;
                }
                Services.highscore.saveScore(gameToken, score,
                    new AsyncCallback<Void>() {
                @Override
                public void onFailure(Throwable error)
                {
                    Window.alert(error.toString());
                }
                @Override
                public void onSuccess(Void result)
                {
                    Window.alert("Done");
                }
                });
            }
        });

        Button btnGetScore = new Button();
        btnGetScore.setText("Get high score");
        vp.add(btnGetScore);
        btnGetScore.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Services.highscore.getHighScore(gameToken,
                    new AsyncCallback<Integer>(){
                @Override
                public void onFailure(Throwable error)
                {
                    Window.alert(error.toString());
                }
                @Override
                public void onSuccess(Integer value)
                {
                    Window.alert(value.toString());
                }
                });
            }
        });
    }

    private static void setupBadge(final FlexTable flxBadge, final int row,
        final String badgeName)
    {
        Services.badges.getBadgeInfo(gameToken, badgeName,
                new AsyncCallback<Badge>() {
            @Override
            public void onFailure(Throwable error)
            {
                Window.alert(error.toString());
            }
            @Override
            public void onSuccess(Badge info)
            {
                flxBadge.setText(row, 1, info.getDisplayName());
                flxBadge.setText(row, 2, info.getDescription());
                flxBadge.setText(row, 3,
                    Integer.toString(info.getMaxProgress()));

                Button btnSetAchieved = new Button();
                flxBadge.setWidget(row, 4, btnSetAchieved);
                btnSetAchieved.setText("Set achieved");
                btnSetAchieved.addClickHandler(new ClickHandler() {
                    @Override
                    public void onClick(ClickEvent event) {
                        Services.badges.setAchieved(gameToken, badgeName,
                            new AsyncCallback<Void>() {
                        @Override
                        public void onFailure(Throwable error)
                        {
                            Window.alert(error.toString());
                        }
                        @Override
                        public void onSuccess(Void result)
                        {
                            Window.alert("Done");
                        }
                        });
                    }
                });

                Button btnIsAchieved = new Button();
                flxBadge.setWidget(row, 5, btnIsAchieved);
                btnIsAchieved.setText("Is achieved?");
                btnIsAchieved.addClickHandler(new ClickHandler() {
                    @Override
                    public void onClick(ClickEvent event) {
                        Services.badges.isAchieved(gameToken, badgeName,
                            new AsyncCallback<Boolean>() {
                        @Override
                        public void onFailure(Throwable error)
                        {
                            Window.alert(error.toString());
                        }
                        @Override
                        public void onSuccess(Boolean result)
                        {
                            Window.alert(result.toString());
                        }
                        });
                    }
                });

                Button btnIncProgress = new Button();
                flxBadge.setWidget(row, 6, btnIncProgress);
                btnIncProgress.setText("Inc. progress");
                btnIncProgress.addClickHandler(new ClickHandler() {
                    @Override
                    public void onClick(ClickEvent event) {
                        Services.badges.incrementProgress(gameToken,badgeName,
                            new AsyncCallback<Boolean>() {
                        @Override
                        public void onFailure(Throwable error)
                        {
                            Window.alert(error.toString());
                        }
                        @Override
                        public void onSuccess(Boolean result)
                        {
                            Window.alert(result ? "Achieved" : "Incremented");
                        }
                        });
                    }
                });

                Button btnGetProgress = new Button();
                flxBadge.setWidget(row, 7, btnGetProgress);
                btnGetProgress.setText("Get progress");
                btnGetProgress.addClickHandler(new ClickHandler() {
                    @Override
                    public void onClick(ClickEvent event) {
                        Services.badges.getProgress(gameToken, badgeName,
                            new AsyncCallback<Integer>() {
                        @Override
                        public void onFailure(Throwable error)
                        {
                            Window.alert(error.toString());
                        }
                        @Override
                        public void onSuccess(Integer result)
                        {
                            Window.alert(result.toString());
                        }
                        });
                    }
                });
            }
        });
    }
}
