/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.client;

import java.io.Serializable;

import au.edu.unimelb.csse.mugle.shared.model.User;

import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.CellPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;

@SuppressWarnings("serial")
public class AuthenticatedUser implements Serializable {

    /* Log in stuff */
    private LoginInfo loginInfo = new LoginInfo();
    private User currentUser = null;
    private Anchor signInLink = new Anchor("Sign In");
    private Anchor signOutLink = new SignOutAnchor("Sign Out");
    private VerticalPanel loginPanel = new VerticalPanel();

    /**
     * Default constructor
     */
    public AuthenticatedUser() {

    }

    public User getUser() {
        return this.currentUser;
    }

    public void setUser(User u) {
        this.currentUser = u;
    }

    public LoginInfo getLoginInfo() {
        return this.loginInfo;
    }

    public void setLoginInfo(LoginInfo loginInfo) {

        this.loginInfo = loginInfo;
        if (loginInfo == null) { return; }

        this.signInLink.setHref(loginInfo.getLoginUrl());
        this.signOutLink.setHref(loginInfo.getLogoutUrl());

    }

    public Anchor getSignInLink() {
        return this.signInLink;
    }

    public Anchor getSignOutLink() {
        return this.signOutLink;
    }

    public VerticalPanel getLoginPanel() {
        return this.loginPanel;
    }

    public boolean isLoggedIn() {
        return this.loginInfo.isLoggedIn();
    }

    public CellPanel resetLoginPanel() {

        this.loginPanel.clear();


        if (this.isLoggedIn()) {

            this.loginPanel.add(this.signOutLink);

        } else {

            Label label = new Label("Please sign in using your Google account.");

            this.loginPanel.add(label);
            this.loginPanel.add(this.signInLink);

        }

        return this.loginPanel;

    }

    private class SignOutAnchor extends Anchor {

        public SignOutAnchor(String text) {
            super(text);
            this.addStyleName("top-bar");
        }

    }

}
