/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client;

import au.edu.unimelb.csse.mugle.client.ui.MugleUiBuilder;
import au.edu.unimelb.csse.mugle.shared.model.*;

import com.google.gwt.core.client.*;
import com.google.gwt.user.client.*;
import com.google.gwt.user.client.rpc.*;
import com.google.gwt.user.client.ui.*;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class Mugle implements EntryPoint {

    private static ErrorDisplay error = new ErrorDisplay();

  private RootPanel mainPanel = null;

  /* Holds the data of the user currently logged in, to be passed around
   * for building the UI
   */
  private static AuthenticatedUser authUser = new AuthenticatedUser();

  /**
   * This is the entry point method.
   * Checks if logged in, if you are, runs the standard code for page,
   * otherwise loads the login page.
   */
  @Override
  public void onModuleLoad() {

      DataProvider.initialiseDataServices();

      this.mainPanel = RootPanel.get("main-panel");

      //Check login status using login service.
      ClientServiceAsync clientService = GWT.create(ClientService.class);
      clientService.login(GWT.getHostPageBaseURL(), new AsyncCallback<LoginInfo>() {

          public void onFailure(Throwable caught) {
                error.addToParent(mainPanel);
                showError(caught.getMessage());
          }

          public void onSuccess(LoginInfo result) {
              authUser.setLoginInfo(result);
              arrangeUI();
          }

      });

  }

  private void arrangeUI() {

      /* If the user is logged in, build the UI based on the data pulled from
       * the datastore
       */
      if (authUser.isLoggedIn()) {
          User.getLoggedInUser(new AsyncCallback<User>() {

            @Override
            public void onFailure(Throwable caught) {
                error.addToParent(mainPanel);
                showError(caught.getMessage());
            }

            @Override
            public void onSuccess(User result) {

                authUser.setUser(result);
                Grid userPanel = new Grid(1,3);
                DOM.getElementById("user-panel-holder").appendChild(userPanel.getElement());

                MugleUiBuilder.arrangeUserPanel(userPanel);
                MugleUiBuilder.arrangeMainPanel(mainPanel);

                if (authUser.getLoginInfo().isAdmin() && !(result.getRole() == Role.ADMIN)) {
                    showError("You do not have sufficient privileges to login as an Administrator!");
                }
            }
          });
      /* If the user isn't logged in construct the default page
       *
       */
      } else {

          // not logged in so take the user to the home page
          this.buildHomePage();

      }

  }

  private void buildHomePage() {

    AbsolutePanel homeInner = MugleUiBuilder.setId(new AbsolutePanel(),
                                                            "home-inner");
    this.mainPanel.add(homeInner);

    homeInner.add(authUser.resetLoginPanel());

  }

  public static ErrorDisplay getErrorDisplay() {
      return error;
  }

  public static void showError(String message) {
      error.showError(message);
  }

  public static void showMessage(String message) {
      error.showMessage(message);
  }

  public static void hideMessage() {
      error.hide();
  }

  public static AuthenticatedUser getAuthUser() {
      return authUser;
  }

  /*
   * Not used. If you need this, please consider bug LP: #785559.
  public static void redirect(String urlFragment) {
      String url = urlFragment;
      if (url.startsWith("#!/")) {
          url = url.substring(1);
      } else if (url.startsWith("/")) {
          url = "!" + url;
      } else if (!url.startsWith("!/")) {
          url = "!/" + url;
      }
      History.newItem(url);
  }
  */

}
