/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.client.model.comparators;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Comparator;

import au.edu.unimelb.csse.mugle.shared.model.DevTeam;

import com.google.gwt.user.client.rpc.IsSerializable;

/**
 * Comparator class for DevTeams.  A DevTeam is less than another DevTeam if
 * its urlName is less than the others.
 * Note: Imports the serializable interfaces at the advice of the API - because DevTeam
 * implements them, not because we're passing them to the server at any point
 */
@SuppressWarnings("serial")
public class DevTeamComparator implements Comparator<DevTeam>, Serializable, IsSerializable {

    @Override
    public int compare(DevTeam d1, DevTeam d2) {
        String[] names = {d1.getUrlName(), d2.getUrlName()};
        Arrays.sort(names);
        
        if (names[0].equals(d1.getUrlName())) {
            return -1;
        } else {
            return 1;
        }
    }


}
