/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.ScriptElement;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * JavaScript clients for the App Engine Channel API.
 */
public class Channel {

    private static final String SRC = "/_ah/channel/jsapi";
    private String channelToken = null;
    private ChannelListener listener = null;

    private final ChannelServiceAsync channelService = Services.channel;

    public Channel() {
        ScriptElement script = Document.get().createScriptElement();
        script.setSrc(SRC);
        Document.get().getElementsByTagName("head").getItem(0)
                .appendChild(script);
    }

    public void open(final ChannelListener listener) {
        getChannelToken();

        this.listener = listener;
        new Timer() {
            @Override
            public void run() {
                if (channelToken != null && scriptLoaded()) {
                    openChannel(channelToken, listener);
                    this.cancel();
                }
            }
        }.scheduleRepeating(100);
    }

    private void getChannelToken() {

        channelService.getChannelToken(new AsyncCallback<String>() {

            @Override
            public void onSuccess(String result) {
                channelToken = result;
            }

            @Override
            public void onFailure(Throwable caught) {
                // TODO Auto-generated method stub

            }
        });
    }

    public void reOpen() {
        if (Document.get().getElementById("wcs-iframe") != null) {
            Document.get().getElementById("wcs-iframe").removeFromParent();
            channelToken = null;
            open(listener);
        }
    }

    private final native void openChannel(String channelToken,
            ChannelListener listener) /*-{
		var channel = new $wnd.goog.appengine.Channel(channelToken);
		var socket = channel.open();

		socket.onopen = function(enent) {
			listener.@au.edu.unimelb.csse.mugle.client.multiplayer.api.ChannelListener::onOpen()()
		};
		socket.onmessage = function(event) {
			listener.@au.edu.unimelb.csse.mugle.client.multiplayer.api.ChannelListener::onMessage(Ljava/lang/String;)(event.data)
		};
		socket.onerror = function(event) {
			listener.@au.edu.unimelb.csse.mugle.client.multiplayer.api.ChannelListener::onError(Lau/edu/unimelb/csse/mugle/client/multiplayer/api/ChannelError;)(event.data)
		};
		socket.onclose = function(enent) {
			listener.@au.edu.unimelb.csse.mugle.client.multiplayer.api.ChannelListener::onClose()()
		};

    }-*/;

    private static native boolean scriptLoaded() /*-{
		return !!$wnd.goog;
    }-*/;
}
