/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import java.util.Set;

import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.PlayerNotFoundInGame;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.UserNotExists;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Services for sending and receiving messages between clients.
 */
@RemoteServiceRelativePath("api-channel")
public interface ChannelService extends RemoteService {
    /**
     * Send the message to every player in the game instance.
     * 
     * @param message
     *            the message as a string.
     * @param instanceToken
     *            the secret token of the current game instance.
     * @throws InstanceTokenError
     * @throws PlayerNotFoundInGame
     */
    void sendMessage(String message, String instanceToken)
            throws InstanceTokenError, PlayerNotFoundInGame;

    /**
     * Send the message to multiple receivers.
     * 
     * @param message
     *            the message as a string.
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param receivers
     *            the receivers of the message.
     * @throws InstanceTokenError
     * @throws PlayerNotFoundInGame
     * @throws UserNotExists
     */
    void sendMessage(String message, String instanceToken, Set<String> receivers)
            throws InstanceTokenError, PlayerNotFoundInGame, UserNotExists;

    /**
     * Get the token of the channel for connecting and listening to the channel.
     * 
     * @return the token of the channel.
     */
    String getChannelToken();

    /**
     * Retrieve a message with the given counter.
     * 
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param counter
     *            the counter of the message.
     * @return a message with the given counter.
     * @throws InstanceTokenError
     * @throws PlayerNotFoundInGame
     */
    String getMessage(String instanceToken, int counter)
            throws InstanceTokenError, PlayerNotFoundInGame;

    /**
     * Retrieve a list of messages with the given counters.
     * 
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param sequence
     *            the counters of the messages.
     * @return the messages.
     * @throws InstanceTokenError
     * @throws PlayerNotFoundInGame
     */
    String[] getMessages(String instanceToken, int[] sequence)
            throws InstanceTokenError, PlayerNotFoundInGame;

    /**
     * Retrieve all the messages of the game.
     * 
     * @param instanceToken
     *            the secret token of the current game instance.
     * @return all the messages of the current game instance.
     */
    String[] getAllMessages(String instanceToken) throws InstanceTokenError,
            PlayerNotFoundInGame;
}
