/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import java.util.Set;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Services for sending and receiving messages between clients.
 */
public interface ChannelServiceAsync {

    /**
     * Send the message to every player in the game instance.
     * 
     * @param message
     *            the message as a string.
     * @param instanceToken
     *            the secret token of the current game instance.
     */
    void sendMessage(String message, String instanceToken,
            AsyncCallback<Void> callback);

    /**
     * Send the message to multiple receivers.
     * 
     * @param message
     *            the message as a string.
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param receivers
     *            the receivers of the message.
     */
    void sendMessage(String message, String instanceToken,
            Set<String> receivers, AsyncCallback<Void> callback);

    /**
     * Get the token of the channel for connecting and listening to the channel.
     * 
     * @param callback
     *            return the token of the channel.
     */
    void getChannelToken(AsyncCallback<String> callback);

    /**
     * Retrieve a message with the given counter.
     * 
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param counter
     *            the counter of the message.
     * @param callback
     *            return a message with the given counter.
     */
    void getMessage(String instanceToken, int counter,
            AsyncCallback<String> callback);

    /**
     * Retrieve a list of messages with the given counters.
     * 
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param sequence
     *            the counters of the messages.
     * @param callback
     *            return the messages.
     */
    void getMessages(String instanceToken, int[] sequence,
            AsyncCallback<String[]> callback);

    /**
     * Retrieve all the messages of the game.
     * 
     * @param instanceToken
     *            the secret token of the current game instance.
     * @param callback
     *            return all the messages of the current game instance.
     */
    void getAllMessages(String instanceToken, AsyncCallback<String[]> callback);
}
