/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameInfo;

/**
 * Handler for events fired from {@link GameManager}.
 */
public interface GameHandler {
    /** Called when a new message is received. */
    void onMessage(Message message);

    /** Called when the channel is opened. */
    void onChannelOpen();

    /** Called when a player joined. */
    void onPlayerJoin(String name);

    /** Called when a player quit. */
    void onPlayerQuit(String name);

    /** Called when a player disconnects with the game. */
    void onPlayerDisconnect(String name);

    /** Called when a player is removed by the game creator. */
    void onPlayerRemove(String name);

    /** Called when a new game is created. */
    void onNewGame(String name);

    /** Called when a game is removed. */
    void onGameRemove(String name);

    /** Called when a game's information updated. */
    void onGameUpdate(GameInfo gameInfo);

    /** Called when a error occurs. */
    void onError(Throwable caught);

    /**
     * Called when a transaction is created.
     * 
     * @param transactionId
     * @param string
     *            a string represents a series of actions in the transaction.
     */
    void onTransactionCreated(String transactionId, String string);
}
