/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import java.io.Serializable;

import au.edu.unimelb.csse.mugle.shared.api.KeyError;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Services for accessing the game key-value store. This allows each game
 * instance to associate arbitrary data with string keys, independently for each
 * instance.
 */
public interface GameKeyValueServiceAsync {

    /**
     * Store an object in the game key-value store. The value can be accessed by
     * all players in the same game instance.
     * 
     * @param key
     *            The key to associate the object with.
     * @param value
     *            The object to store.
     * @param callback
     *            return true if succeed.
     */
    void getValue(String instanceToken, String key,
            AsyncCallback<Serializable> callback);

    /**
     * Retrieve an object from the game key-value store.
     * 
     * @param gameToken
     *            The secret token for the current game.
     * @param key
     *            The key the object is associated with.
     * @param callback
     *            return the object associated with the key, or throws
     *            {@link KeyError} if no object is associated with that key.
     */
    void putValue(String instanceToken, String key, Serializable value,
            AsyncCallback<Boolean> callback);

}
