/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameInfo;

import com.google.gwt.user.client.rpc.AsyncCallback;

public interface MultiPlayerGameServiceAsync {
    /**
     * Create a new game instance.
     * 
     * @param gameToken
     *            The secret token for the current game.
     * @param size
     *            The maximum number of player for the game.
     * @param isPublic
     *            True if the game can be joined public.
     * @param callback
     *            return the game instance token.
     */
    public void createGameInstance(String gameToken, int size,
            boolean isPublic, AsyncCallback<String> callback);

    /**
     * Join a game instance.
     * 
     * @param instanceToken
     *            The secret token for the game instance to join.
     * @param callback
     *            return a list of all the player names for this game.
     */
    public void joinGame(String instanceToken, AsyncCallback<String[]> callback);

    /**
     * Exit the current game instance.
     * 
     * @param instanceToken
     *            The secret token of the current game instance.
     */
    public void quitGame(String instanceToken, AsyncCallback<Void> callback);

    /**
     * Remove a player from the current if this player is the game creator.
     * 
     * @param instanceToken
     *            The secret token of the current game instance.
     * @param name
     *            the url name of the player.
     */
    public void removePlayer(String instanceToken, String name,
            AsyncCallback<Void> callback);

    /**
     * Set whether the game can be joined.
     * 
     * @param instanceToken
     *            The secret token of the current game instance.
     * @param joinable
     *            true if the game can be joined.
     */
    public void setGameJoinable(String instanceToken, boolean joinable,
            AsyncCallback<Void> callback);

    /**
     * Get the list of active game instances.
     * 
     * @param gameToken
     *            The secret token of for the current game.
     * @param callback
     *            return a list of game information.
     */
    public void getGameList(String gameToken, AsyncCallback<GameInfo[]> callback);

}
