package au.edu.unimelb.csse.mugle.client.multiplayer.api;

import au.edu.unimelb.csse.mugle.shared.api.KeyError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameInstanceNotExists;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.Transaction;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.TransactionNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.UserNotExists;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("api-transaction")
public interface TransactionService extends RemoteService {
    /**
     * Create a new transaction.
     * 
     * @param instanceToken
     *            The secret token for the game instance to join.
     * @param transaction
     *            The transaction to be created.
     * @return The transaction id.
     * @throws InstanceTokenError
     * @throws GameInstanceNotExists
     * @throws UserNotExists
     */
    public String createTransaction(String instanceToken,
            Transaction transaction) throws InstanceTokenError,
            GameInstanceNotExists, UserNotExists;

    /**
     * Confirm a transaction.
     * 
     * @param instanceToken
     *            The secret token for the game instance to join.
     * @param transactionId
     *            The id for the confirmed transaction.
     * @throws InstanceTokenError
     * @throws GameInstanceNotExists
     * @throws TransactionNotExists
     * @throws UserNotExists
     * @throws KeyError
     */
    public void confirmTransaction(String instanceToken, String transactionId)
            throws InstanceTokenError, GameInstanceNotExists,
            TransactionNotExists, UserNotExists, KeyError;
}
