package au.edu.unimelb.csse.mugle.client.ui;

import java.util.ArrayList;

import au.edu.unimelb.csse.mugle.shared.model.*;

import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.logical.shared.*;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;

public class DevTeamEditBuilder extends ViewBuilder {

    // Internal representation of the devteam information to display and edit.
    private DevTeam devteam;
    private ArrayList<User> users;
    private ArrayList<Game> games;

    // UI elements
    private FlexTable userPanel;
    private FlexTable gamePanel;


    // Prevent instantiation.
    public DevTeamEditBuilder(String urlFragment) {
        super(urlFragment);
    }



    @Override
    public Widget buildView() {

        this.clearPanel();
        try {
            loadDevTeam(this.getUrlFragment());
        } catch (Exception e) {
            this.showError("The page with URL could not be " +
                    "loaded. Reason was: " + e.getMessage());
        }
        return this.getDataPanel();

    }

    /**
     * Method in charge of creating the layout of this view.
     */
    private void buildUI(final FlowPanel panel) {

        final ViewBuilder builder = this;

        // Create second-level panels
        FlexTable flxPrims = new FlexTable();
        Button btnSave = new Button("Save changes");
        userPanel = new FlexTable();
        gamePanel = new FlexTable();
        HorizontalPanel addUserPanel = new HorizontalPanel();
        HorizontalPanel addGamePanel = new HorizontalPanel();

        // Set the window title
        this.setWindowTitle("MUGLE - Editing " + devteam.getFullName());
        this.updateWindowTitle();

        // Assemble container panel
        panel.add(flxPrims);
        panel.add(btnSave);
        panel.add(new Label("Users"));
        panel.add(userPanel);
        panel.add(addUserPanel);
        panel.add(new Label("Games"));
        panel.add(gamePanel);
        panel.add(addGamePanel);

        // Assemble name row
        flxPrims.setText(0, 0, "URL Name:");
        flxPrims.setText(0, 1, devteam.getUrlName());
        flxPrims.setText(0, 2, "The name of the team for use in URLs.");
        // Assemble full name row
        flxPrims.setText(1, 0, "Display Name:");
        final TextBox fullNameBox = new TextBox();
        fullNameBox.setVisibleLength(10);
        fullNameBox.setValue(devteam.getFullName());
        flxPrims.setWidget(1, 1, fullNameBox);
        flxPrims.setText(1, 2, "The name of the team, as displayed to "
            + "users.");

        btnSave.addClickHandler(new ClickHandler() {
          public void onClick(ClickEvent event) {
                // Store currently valid properties.
                final String tmp = devteam.getFullName();

                // Attempt to submit changes to server.
                devteam.setFullName(fullNameBox.getValue());
                devteam.write(new AsyncCallback<DevTeam>() {
                    @Override
                    public void onFailure(Throwable error) {
                        // Reset value in textbox and full name in internal
                        // devteam representation.
                        fullNameBox.setValue(tmp);
                        devteam.setFullName(tmp);
                        builder.showError(error.getMessage());
                    }

                    @Override
                    public void onSuccess(DevTeam result) {
                        builder.showMessage("Team updated successfully.");
                        builder.setWindowTitle("MUGLE - Editing "
                            + result.getFullName());
                        builder.updateWindowTitle();
                    }
                });
          }
        });

        assembleUserPanel();
        assembleGameTable();

        // Assemble panel for creating a new user
        addUserPanel.add(new Label("Add user with name: "));
        final TextBox addUserBox = new TextBox();
        addUserBox.setVisibleLength(10);
        final Button addUserButton = new Button("Add");

        addUserButton.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                //final User user = new Game();
                //game.setUrlName(addUserBox.getValue());
                //game.setDevTeamKey(devteam.getPrimaryKey());

                // Attempt to create game
                String username = addUserBox.getValue();
                User.getByName(username, new AsyncCallback<User>() {
                    @Override
                    public void onFailure(Throwable error) {
                        builder.showError(error.getMessage());
                    }

                    @Override
                    public void onSuccess(final User user) {
                        // On success add the new user to the devteam
                        devteam.addUser(user,
                            new AsyncCallback<UpdatedPair<DevTeam, User>>() {
                            @Override
                            public void onFailure(Throwable error) {
                                builder.showError(error.getMessage());
                            }

                            @Override
                            public void onSuccess(UpdatedPair<DevTeam,
                                                            User> unused) {
                                users.add(user);
                                assembleUserPanel();
                            }
                        });
                    }
                });
            }
        });

        addUserPanel.add(addUserBox);
        addUserPanel.add(addUserButton);

        // Assemble panel for creating a new game
        addGamePanel.add(new Label("Add game with name: "));
        final TextBox addGameBox = new TextBox();
        addGameBox.setVisibleLength(10);
        final Button addGameButton = new Button("Add");

        addGameButton.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                final Game game = new Game();
                game.setUrlName(addGameBox.getValue());
                game.setDevTeamKey(devteam.getPrimaryKey());

                // Attempt to create game
                game.create(new AsyncCallback<Game>() {
                    @Override
                    public void onFailure(Throwable error) {
                        builder.showError(error.getMessage());
                    }

                    @Override
                    public void onSuccess(final Game resultGame) {
                        // On success add the new game to the internal list of
                        // games and update the UI.
                        GameVersion gameVersion = new GameVersion();
                        gameVersion.setGameKey(resultGame.getPrimaryKey());
                        gameVersion.setFullName(game.getUrlName());
                        gameVersion.setDispVersion("0");
                        gameVersion.setVersion(0);

                        // Attempt to create version for game
                        gameVersion.create(new AsyncCallback<GameVersion>() {

                            @Override
                            public void onFailure(Throwable error) {
                                // If failing to add game version, remove
                                // previously created game.
                                // TODO maybe have a service in the backend
                                // allowing a game and its default version
                                // to be created at the same time.
                                resultGame.remove(new AsyncCallback<Void>() {
                                    @Override
                                    public void onFailure(Throwable error) {
                                        builder.showError(error.getMessage());
                                    }

                                    @Override
                                    public void onSuccess(Void result) {}
                                });

                                builder.showError(error.getMessage());
                            }

                            @Override
                            public void onSuccess(GameVersion result) {
                                // Set the game's activeVersion
                                resultGame.setActiveVersion(result);
                                resultGame.write(new AsyncCallback<Game>() {
                                    @Override
                                    public void onFailure(Throwable error) {
                                        builder.showError(error.getMessage());
                                    }
                                    @Override
                                    public void onSuccess(Game unused) {
                                        // Add the game to the devteam
                                        devteam.addGame(resultGame,
                                            new AsyncCallback<UpdatedPair<DevTeam, Game>>() {
                                            @Override
                                            public void onFailure(Throwable error) {
                                                builder.showError(error.getMessage());
                                            }

                                            @Override
                                            public void onSuccess(UpdatedPair<DevTeam,
                                                                            Game> unused) {
                                                games.add(resultGame);
                                                assembleGameTable();
                                            }
                                        });
                                    }
                                });
                            }

                        });
                    }
                });
            }
        });

        addGamePanel.add(addGameBox);
        addGamePanel.add(addGameButton);

    }


    /**
     * Adds all users to a panel and links their profile views.
     */
    private void assembleUserPanel() {
        userPanel.clear();
        for (int i = 0; i < users.size(); i++) {
            userPanel.setWidget(i, 0, new Anchor(users.get(i).getFullName(),
                    "#!/~" + users.get(i).getUrlName()));
        }
    }


    /**
     * Creates a table of the games associated with this dev team and allows
     * new games to be created and added to the list.
     */
    private void assembleGameTable() {
        gamePanel.clear();
        for (int i = 0; i < games.size(); i++) {
            gamePanel.setWidget(i, 0, new Anchor(games.get(i).getUrlName(),
                    "#!/" + devteam.getUrlName() + "/"
                    + games.get(i).getUrlName()));
        }
    }



    /**
     * Retrieves the dev team data for a specified dev team url name.
     * @param urlname The url name for which to retrieve the data.
     */
    private void loadDevTeam(final String urlname) {

        final ViewBuilder builder = this;

        // Try to load devteam data
        DevTeam.getByName(urlname, new AsyncCallback<DevTeam>() {
            @Override
            public void onFailure(Throwable error) {
                builder.showError("The devteam with URL " + urlname
                        + " could not be loaded. Reason was: "
                        + error.getMessage());
            }

            @Override
            public void onSuccess(DevTeam result) {
                devteam = result;
                // Check that the user has the privilege to edit this object
                if (result.userCanEdit())
                {
                    loadUsers();
                }
                else
                {
                    builder.getDataPanel().add(
                        builder.getAuthorizationError());
                }
            }
        });

    }


    /**
     * Loads the users associated with the dev team loaded
     * in {@ #loadDevTeam()}.
     */
    private void loadUsers() {

        final ViewBuilder builder = this;

        // Try to load the users of this devteam
        devteam.getUsers(new AsyncCallback<ModelCollection<User>>() {
            @Override
            public void onFailure(Throwable error) {
                builder.showError("The user list for the devteam with URL "
                        + devteam.getUrlName()
                        + " could not be loaded. Reason was: "
                        + error.getMessage());
            }

            @Override
            public void onSuccess(ModelCollection<User> result) {
                users = new ArrayList<User>();
                users.addAll(result.getCollection());
                loadGames();
            }
        });

    }


    /**
     * Loads the games associated with the dev team
     * loaded in {@ #loadDevTeam()}.
     */
    private void loadGames() {

        final ViewBuilder builder = this;

        devteam.getGames(true, new AsyncCallback<ModelCollection<Game>>() {
            @Override
            public void onFailure(Throwable error) {
                builder.showError("The game list for the devteam with URL "
                        + devteam.getUrlName()
                        + " could not be loaded. Reason was: "
                        + error.getMessage());
            }

            @Override
            public void onSuccess(ModelCollection<Game> result) {
                games = new ArrayList<Game>();
                games.addAll(result.getCollection());
                buildUI(builder.getDataPanel());
            }
        });

    }

}
