/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.client.ui;

import au.edu.unimelb.csse.mugle.client.*;
import au.edu.unimelb.csse.mugle.client.ui.admin.AdminDataTestViewBuilder;
import au.edu.unimelb.csse.mugle.client.ui.admin.AdminDevTeamsViewBuilder;
import au.edu.unimelb.csse.mugle.client.ui.admin.AdminGameLogViewBuilder;
import au.edu.unimelb.csse.mugle.client.ui.admin.AdminGameMonitorViewBuilder;
import au.edu.unimelb.csse.mugle.client.ui.admin.AdminViewBuilder;
import au.edu.unimelb.csse.mugle.shared.model.*;

import com.google.gwt.dom.client.Style.*;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.*;
import com.google.gwt.user.client.*;
import com.google.gwt.user.client.ui.*;

/**
 * Main Class for building the User Interface
 */
public class MugleUiBuilder {

    static AbsolutePanel errorDisp = null;

    static StackPanel toolsPanel = null;

    /** In very rare cases, we don't want to update the view when history
     * changes.
     * (Used when we change the urlname of an object and change the URL bar to
     * match, but want to stay on the same page.)
     */
    static boolean ignoreHistoryChanges = false;

    public static void setIgnoreHistoryChanges(boolean val)
    {
        ignoreHistoryChanges = val;
    }

    private MugleUiBuilder() { }

    /**
     * Sets the "id" of an element in the HTML page
     * @param w The Widget the id must be set in
     * @param id The id that must be used
     * @return The updated widget
     */
    public static <T extends Widget> T setId(T w, String id) {
        w.getElement().setId(id);
        return w;
    }

    /**
     * Sets the "style" of an element in the HTML page to caution type
     * @param w The Widget the id must be set in
     * @return The updated widget
     */
    public static <T extends Widget> T setWarning(T w) {
        w.setStyleName("warning-msg");
        return w;
    }

    /**
     * Builds the top user panel
     * @param userPanel
     * @param authuser
     */
    public static void arrangeUserPanel(Grid userPanel) {

      DOM.getElementById("user-panel-title").appendChild((new Label("MUGLE")).getElement());
      userPanel.setWidget(0, 0, new Label(Mugle.getAuthUser().getLoginInfo().getEmailAddress()));
      userPanel.setWidget(0, 1, new Label("|"));
      userPanel.setWidget(0, 2, Mugle.getAuthUser().getSignOutLink());

    }

    /**
     * Root method for building the user interface.
     * @param mainPanel the root panel of the page
     * @param authUser the details of the user currently logged in
     */
    public static void arrangeMainPanel(RootPanel mainPanel) {

      mainPanel.clear();
      AbsolutePanel container = setId(new AbsolutePanel(), "main-panel-container");
      mainPanel.add(container);

      AbsolutePanel inner = setId(new AbsolutePanel(), "main-panel-container-inner");
      inner.getElement().getStyle().setOverflow(Overflow.VISIBLE);
      container.add(inner);

      AbsolutePanel tp = setId(new AbsolutePanel(), "tools-panel");
      AbsolutePanel rp = setId(new AbsolutePanel(), "right-panel");
      AbsolutePanel rpi = setId(new AbsolutePanel(), "right-panel-inner");
      AbsolutePanel legal = new AbsolutePanel();
      legal.setStyleName("legal");
      legal.add(new Label("This web site, its contents and style, are"
        + " the responsibility of the developers and individual student"
        + " authors and do not necessarily represent the views, policies or"
        + " opinions of The University of Melbourne."));

      tp.add(buildToolsPanel());
      rpi.add(buildRightPanel());

      Mugle.getErrorDisplay().addToParent(rp);
      rp.add(rpi);
      rp.add(legal);

      inner.add(tp);
      inner.add(rp);

    }

    /** Rebuild the panel on the left of the page.
     */
    public static void refreshToolsPanel() {
        buildToolsPanel();
    }

    /**
     * Builds the panel on the left of the page when logged in

     * @return
     */
    private static Widget buildToolsPanel() {

      if (toolsPanel == null)
        toolsPanel = new StackPanel();
      else
        toolsPanel.clear();

      // left panel
      final StackPanel sp = toolsPanel;
      sp.setWidth("100%");
      sp.getElement().getStyle().setFontSize(1, Unit.EM);

      Widget tmp = null;
      //Build the drop down panels
      if ((tmp = buildMuglePublic(Mugle.getAuthUser().getUser())) != null) {
          sp.add(tmp, "Mugle Public", false);
      }
      if ((tmp = buildMyMugle(Mugle.getAuthUser().getUser())) != null) {
          sp.add(tmp, "My Mugle", false);
      }
      if ((tmp = buildAdmin(Mugle.getAuthUser().getUser())) != null) {
          sp.add(tmp, "Admin", false);
      }

      return sp;

    }

    /**
     * Builds the drop down panel titled "My Mugle"
     * @param user The User object from the datastore corresponding to the user currently logged in
     * @return
     */
    private static Widget buildMyMugle(User user) {

        Role role = user.getRole();
        VerticalPanel vp = new VerticalPanel();

        String profile = "#!/~" + user.getUrlName();

        vp.add(new Anchor("My Profile", profile));

        // TODO: Only show these links if the user is a member of at least one
        // devteam (LP: #786842).
        if (true)
        {
            vp.add(new Anchor("My Games", "#!/+mygames"));
            vp.add(new Anchor("My Teams", "#!/+myteams"));
        }

        return vp;

    }

    /**
     * Builds the drop down panel titled "Mugle Public"
     * @param user The User object from the datastore corresponding to the user currently logged in
     * @return
     */
    private static Widget buildMuglePublic(User user) {

        VerticalPanel vp = new VerticalPanel();

        // TODO Promoted games removed for now
        //vp.add(new Anchor("Promoted Games", "#!/+vip"));
        vp.add(new Anchor("Games Gallery", "#!/+gallery"));

        return vp;

    }

    /**
     * Builds the drop down panel titled "Admin"
     * @param user The User object from the datastore corresponding to the user currently logged in
     * @return
     */
    private static Widget buildAdmin(User user) {

        Role role = user.getRole();

        switch (role) {

            case ADMIN:

                String admin = "#!/+admin";

                VerticalPanel vp = new VerticalPanel();
                vp.add(new Anchor("Administrative Tasks", admin));
                vp.add(new Anchor("Users", admin + "/users"));
                vp.add(new Anchor("Dev Teams", admin + "/teams"));
                vp.add(new Anchor("Games", admin + "/games"));
                vp.add(new Anchor("Game Monitor", admin + "/monitor"));  // add game monitoring
                vp.add(new Anchor("Promoted Games", admin + "/vip"));
                vp.add(new Anchor("News", admin + "/news"));
                vp.add(new Anchor("MUGLE Tests", admin + "/test"));
                return vp;

        }

        return null;

    }

    public static <T extends ModelClientClass<?,T>>
          AbsolutePanel createPageNavigation(ModelCollection<T> collection,
                            ClickHandler first,ClickHandler prev,
                                  ClickHandler next, ClickHandler last) {

        AbsolutePanel panel = new AbsolutePanel();
        panel.setStylePrimaryName("page-nav");

        // add first
        Button arrow = new Button("<<");
        if (first != null) { arrow.addClickHandler(first); }
        if (collection.getStartIndex() <= 0) {
            arrow.setEnabled(false);
        }
        panel.add(arrow);

        // add prev
        arrow = new Button("<");
        if (first != null) { arrow.addClickHandler(prev); }
        if (collection.getStartIndex() <= 0) {
            arrow.setEnabled(false);
        }
        panel.add(arrow);

        // add next
        arrow = new Button(">");
        if (first != null) { arrow.addClickHandler(next); }
        if (collection.getEndIndex() >= collection.getCapacity()) {
            arrow.setEnabled(false);
        }
        panel.add(arrow);

        // add last
        arrow = new Button(">>");
        if (first != null) { arrow.addClickHandler(last); }
        if (collection.getEndIndex() >= collection.getCapacity()) {
            arrow.setEnabled(false);
        }
        panel.add(arrow);

        return panel;

    }

    /**
     * Handles most of the dynamic content - builds the centre panel on the
     * right based on the history token
     * @param authUser
     * @return
     */
    private static Widget buildRightPanel() {

        final HorizontalPanel panel = new HorizontalPanel();

        // Keeps track of the state of the page
        History.addValueChangeHandler(new ValueChangeHandler<String>() {
          public void onValueChange(ValueChangeEvent<String> event) {
              if (ignoreHistoryChanges)
                return;
              String historyToken = event.getValue();
              //Changes the module when the history token is changed
              buildByHistoryToken(historyToken, panel);
          }
        });
        //Loads the module on refresh/startup based on the history token
        History.fireCurrentHistoryState();
        return panel;
    }

    /**
     * builds the panel based on the history token provided.  All the actual arrangement
     * of the UI is handled in separate modules specifically built for each view.
     * @param historyToken the History Token in the user's browser
     * @param panel the right panel we're adding content to
     * @param authUser the details of the user currently logged in
     */
    private static void buildByHistoryToken(String historyToken, HorizontalPanel panel) {

        Mugle.hideMessage();

        ViewBuilder builder = null;

        // NOTE: All redirects should 'return' immediately.

        // Note tokens[0] will always equal "!"
        String[] tokens = historyToken.split("/");

        // Default to the gallery
        if (historyToken.isEmpty() || tokens.length == 1) {

            // redirect to gallery page
            /* Don't redirect; just serve the gallery page
             * (see bug LP: #785559)
            Mugle.redirect("+gallery");
            return;
            */
            builder = new GalleryViewBuilder();

        }
        else if (tokens[1].startsWith("~")) {

            // Viewing a User
            if (tokens.length == 2) {
                builder = new UserViewBuilder(tokens[1].substring(1));
            }
            else if (tokens[2].equals("+edit") && tokens.length == 3) {
                builder = new UserEditBuilder(tokens[1].substring(1));
            }

        }
        // reserved cases
        else if (tokens[1].startsWith("+")) {

            // admin tasks
            if (tokens[1].equals("+admin")) {

                if (tokens.length == 2) {
                    builder = new AdminViewBuilder();
                }
                else {
                    if (tokens[2].equals("users") && tokens.length == 3) {
                        // TODO implement the appropriate builder
                        builder = new PlaceHolderViewBuilder("TODO: add admin USER options.");
                    }
                    else if (tokens[2].equals("teams") && tokens.length == 3) {
                        builder = new AdminDevTeamsViewBuilder();
                    }
                    else if (tokens[2].equals("games") && tokens.length == 3) {
                        // TODO implement the appropriate builder
                        builder = new PlaceHolderViewBuilder("TODO: add admin GAMES options.");
                    }
                    else if (tokens[2].equals("monitor")) {
                        if(tokens.length == 3)
                            builder = new AdminGameMonitorViewBuilder();
                        else if(tokens.length == 5)
                            builder = new AdminGameMonitorViewBuilder(tokens[3],tokens[4]);
                    }
                    else if (tokens[2].equals("vip") && tokens.length == 3) {
                        // TODO implement the appropriate builder
                        builder = new PlaceHolderViewBuilder("TODO: add admin PROMOTED GAMES options.");
                    }
                    else if (tokens[2].equals("news") && tokens.length == 3) {
                        // TODO implement the appropriate builder
                        builder = new PlaceHolderViewBuilder("TODO: add admin NEWS for the first page (for future).");
                    }
                    else if (tokens[2].equals("test") && tokens.length == 3) {
                        builder = new AdminDataTestViewBuilder();
                    }
                }
            }
            // promoted games
            // TODO need to be implemented (removed for now)
            /*
            else if (tokens[1].equals("+vip")) {
                if (tokens.length == 2)
                {
                    builder = new PlaceHolderViewBuilder("TODO: display a list of all PROMOTED games!");
                }
                else
                {
                    // We're looking at a game
                    if (tokens.length == 3) {
                        builder = new PlaceHolderViewBuilder(
                            "TODO: promoted game view: " + tokens[2]);
                    }
                    else if (tokens[3].equals("+edit") && tokens.length == 4) {
                        builder = new PlaceHolderViewBuilder(
                            "TODO: promoted game edit: " + tokens[2]);
                    }
                }

            }*/
            // games gallery
            else if (tokens[1].equals("+gallery")) {
                builder = new GalleryViewBuilder();
            }
            else if (tokens[1].equals("+mygames")) {
                builder = new MyGamesViewBuilder("");
            }
            else if (tokens[1].equals("+myteams")) {
                builder = new MyDevTeamsViewBuilder("");
            }
        }
        // Builds the view for promoted games and DevTeams
        else {

            // check other reserved paths
            if (isInvalid(tokens)) {
                // invalid url, so Error 404
                builder = new NotFoundViewBuilder(historyToken);
            }
            // Viewing a DevTeam
            else if (tokens.length == 2) {
                builder = new DevTeamViewBuilder(tokens[1]);
            }
            // Editing a DevTeam
            else if (tokens[2].equals("+edit") && tokens.length == 3) {
                builder = new DevTeamEditBuilder(tokens[1]);
            }
            // We're looking at a game
            else {
                if (tokens.length == 3) {
                    builder = new GameViewBuilder(tokens[1], tokens[2]);
                }
                else if (tokens[3].equals("+edit") && tokens.length == 4) {
                    builder = new GameEditBuilder(tokens[1], tokens[2]);
                }
            }
        }

        panel.clear();
        // Error 404
        if (builder == null) {
            builder = new NotFoundViewBuilder(historyToken);
        }
        panel.add(builder.buildView());
        builder.updateWindowTitle();

    }

    /**
     * Used for other reserved paths
     * @param tokens
     * @return
     */
    private static boolean isInvalid(String tokens[]) {

        // token[0] == "!"

        // not a reserved case
        if (tokens.length < 2) {
            return false;
        }

        if (tokens[1].equals("img")) {
            return true;
        } else if (tokens[1].endsWith(".html")) {
            return true;
        } else if (tokens[1].endsWith(".css")) {
            return true;
        }

        return false;

    }

}
