/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui;

import java.util.Set;

import au.edu.unimelb.csse.mugle.shared.model.*;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;

public class MyDevTeamsViewBuilder extends ViewBuilder {

    public MyDevTeamsViewBuilder(String urlFragment) {
        super(urlFragment);
    }

    @Override
    public Widget buildView() {

        final FlowPanel panel = this.getDataPanel();
        final ViewBuilder builder = this;

        panel.clear();

        User.getLoggedInUser(new AsyncCallback<User>() {

            @Override
            public void onSuccess(User result) {

                result.getDevTeams(new AsyncCallback<ModelCollection<DevTeam>>() {

                    @Override
                    public void onSuccess(ModelCollection<DevTeam> result) {
                        contructView(panel, result.getCollection());
                    }

                    @Override
                    public void onFailure(Throwable caught) {
                        builder.showError("Unable to access DevTeams.");
                    }
                });

            }

            @Override
            public void onFailure(Throwable caught) {
                builder.showError("Unable to access DevTeams.");
            }

        });

        // Set the window title
        this.setWindowTitle("MUGLE - My Teams");

        return panel;

    }

    private void contructView(FlowPanel panel, Set<DevTeam> results) {

        if (!results.isEmpty()) {

            FlexTable table = new FlexTable();

            panel.add(new Label("You are a member of the following developer "
                + "teams:"));
            int i = 0;
            for (DevTeam team : results) {
                Anchor anc = new Anchor(team.getFullName(), "#!/" + team.getUrlName());
                table.setWidget(i, 0, anc);
                i++;
            }

            panel.add(table);

        } else {

            panel.add(new Label("You are not a member of any developer "
                + "teams."));

        }

    }

}
