/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui;

import au.edu.unimelb.csse.mugle.shared.model.*;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;

public class MyGamesViewBuilder extends ViewBuilder {

    public MyGamesViewBuilder(String urlFragment) {
        super(urlFragment);
    }

    @Override
    public Widget buildView() {

        final FlowPanel panel = this.getDataPanel();
        final ViewBuilder builder = this;

        panel.clear();

        Game.getMyGames(0, 20, new AsyncCallback<ModelCollection<Game>>() {

            @Override
            public void onSuccess(ModelCollection<Game> result) {
                contructView(panel, result);
            }

            @Override
            public void onFailure(Throwable caught) {
                builder.showError("Unable to access games.");
            }

        });

        // Set the window title
        this.setWindowTitle("MUGLE - My Games");

        return panel;

    }

    private void contructView(FlowPanel panel, ModelCollection<Game> results) {

        if (!results.getCollection().isEmpty()) {

            FlexTable table = new FlexTable();

            panel.add(new Label("You have published the following games:"));

            // headers
            //table.setText(0, 0, "Name");
            //table.setText(0, 1, "Active Version");

            int i = 0;
            for (Game g : results.getCollection()) {
                constructGameLink(table, i, g);
                i++;
            }

            panel.add(table);

        } else {

            panel.add(new Label("You have not published any games."));

        }

    }

    /** Get the URL for a game. */
    private void constructGameLink(final FlexTable table, final int row,
        final Game game)
    {
        // XXX Find a way to get the devteam without a server request
        //     for each individual game (LP: #787403).
        DevTeam.getByKey(game.getDevTeamKey(), new AsyncCallback<DevTeam>() {
            @Override
            public void onFailure(Throwable caught) {
                table.setText(row, 0, caught.getMessage());
            }
            @Override
            public void onSuccess(DevTeam dt)
            {
                GameVersion gv = game.getActiveVersion();
                String name;
                if (gv != null)
                    name = gv.getFullName();
                else
                    name = game.getUrlName();
                Anchor anc = new Anchor(name,
                    "#!/" + dt.getUrlName() + "/" + game.getUrlName());
                table.setWidget(row, 0, anc);
                // For now, don't show (since all games have 1 version, and we
                // don't provide a separate view for the version).
                /*
                if (gv != null) {
                    anc = new Anchor(gv.getDisplayVersion(),
                        "#!/" + dt.getUrlName() + "/"
                        + game.getUrlName() + "/" + gv.getVersion());
                    table.setWidget(row, 1, anc);
                } else {
                    table.setText(row, 1, "N/A");
                }
                */
            }
        });
    }

}
