package au.edu.unimelb.csse.mugle.client.ui;

import java.util.ArrayList;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.client.Mugle;

import com.google.gwt.event.dom.client.*;
import com.google.gwt.event.logical.shared.*;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.History;

public class UserEditBuilder extends ViewBuilder {

    // Internal representation of the user information to display and edit.
    private User user;

    // Prevent instantiation.
    public UserEditBuilder(String urlFragment) {
        super(urlFragment);
    }

    @Override
    public Widget buildView() {

        this.clearPanel();
        try {
            loadUser(this.getUrlFragment());
        } catch (Exception e) {
            this.showError("The page with URL could not be " +
                    "loaded. Reason was: " + e.getMessage());
        }
        return this.getDataPanel();

    }

    /**
     * Method in charge of creating the layout of this view.
     */
    private void buildUI(final FlowPanel panel) {

        final ViewBuilder builder = this;

        // Create second-level panels
        FlexTable flxPrims = new FlexTable();
        Button btnSave = new Button("Save changes");

        // Set the window title
        this.setWindowTitle("MUGLE - Editing " + user.getFullName());
        this.updateWindowTitle();

        // Assemble container panel
        panel.add(flxPrims);
        panel.add(btnSave);

        // Assemble name row
        flxPrims.setText(0, 0, "URL Name:");
        final TextBox urlNameBox = new TextBox();
        urlNameBox.setVisibleLength(25);
        urlNameBox.setValue(user.getUrlName());
        flxPrims.setWidget(0, 1, urlNameBox);
        flxPrims.setText(0, 2, "The name of the user for use in URLs. "
            + "Change this if you do not want your email address public.");
        // Assemble full name row
        flxPrims.setText(1, 0, "Display Name:");
        final TextBox fullNameBox = new TextBox();
        fullNameBox.setVisibleLength(25);
        fullNameBox.setValue(user.getFullName());
        flxPrims.setWidget(1, 1, fullNameBox);
        flxPrims.setText(1, 2, "The full name of the user, as displayed to "
            + "other users.");
        // Assemble email row
        flxPrims.setText(2, 0, "Email:");
        flxPrims.setText(2, 1, user.getEmail());
        flxPrims.setText(2, 2, "The user's email address (this is private).");

        btnSave.addClickHandler(new ClickHandler() {
          public void onClick(ClickEvent event) {
                // Store currently valid properties.
                final String tmpUrl = user.getUrlName();
                final String tmpFull = user.getFullName();

                // Attempt to submit changes to server.
                user.setUrlName(urlNameBox.getValue());
                user.setFullName(fullNameBox.getValue());
                user.write(new AsyncCallback<User>() {
                    @Override
                    public void onFailure(Throwable error) {
                        // Reset value in textbox and full name in internal
                        // user representation.
                        urlNameBox.setValue(tmpUrl);
                        user.setUrlName(tmpUrl);
                        fullNameBox.setValue(tmpFull);
                        user.setFullName(tmpFull);
                        builder.showError(error.getMessage());
                    }

                    @Override
                    public void onSuccess(User result) {
                        builder.showMessage("User updated successfully.");
                        builder.setWindowTitle("MUGLE - Editing "
                            + result.getFullName());
                        builder.updateWindowTitle();
                        if (!result.getUrlName().equals(tmpUrl))
                        {
                            // Need to redirect to the new URL
                            MugleUiBuilder.setIgnoreHistoryChanges(true);
                            try
                            {
                                History.newItem("!/~" + result.getUrlName() +
                                    "/+edit");
                            }
                            finally
                            {
                                MugleUiBuilder.setIgnoreHistoryChanges(false);
                            }
                            // If this is the logged in user, need to update
                            // Mugle.authUser (to fix side links)
                            if (result.getPrimaryKey().equals(
                                Mugle.getAuthUser().getUser().getPrimaryKey()))
                            {
                                Mugle.getAuthUser().getUser().setUrlName(
                                    result.getUrlName());
                                MugleUiBuilder.refreshToolsPanel();
                            }
                        }
                    }
                });
            }
        });
    }

    /**
     * Retrieves the user data for a specified user url name.
     * @param urlname The url name for which to retrieve the data.
     */
    private void loadUser(final String urlname) {

        final ViewBuilder builder = this;

        // Try to load user data
        User.getByName(urlname, new AsyncCallback<User>() {
            @Override
            public void onFailure(Throwable error) {
                builder.showError("The user with URL " + urlname
                        + " could not be loaded. Reason was: "
                        + error.getMessage());
            }

            @Override
            public void onSuccess(User result) {
                user = result;
                // Check that the user has the privilege to edit this object
                if (result.userCanEdit())
                {
                    buildUI(builder.getDataPanel());
                }
                else
                {
                    builder.getDataPanel().add(
                        builder.getAuthorizationError());
                }
            }
        });

    }
}
