/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui;

import au.edu.unimelb.csse.mugle.client.Mugle;

import au.edu.unimelb.csse.mugle.shared.model.DevTeam;
import au.edu.unimelb.csse.mugle.shared.model.ModelCollection;
import au.edu.unimelb.csse.mugle.shared.model.User;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;

/**
 * Loads the private view for the User Panel
 * @author Scott
 *
 */
public class UserViewBuilder extends ViewBuilder {

    public UserViewBuilder(String urlFragment) {
        super(urlFragment);
    }

    @Override
    public Widget buildView() {
        this.clearPanel();
        return loadProfile(this.getDataPanel(), this.getUrlFragment());
    }

    /**
     * @param userID, ID of the user you want to load
     * @return A panel to render
     */
    public FlowPanel loadProfile(final FlowPanel panel, String urlName) {

        final ViewBuilder builder = this;

        User.getByName(urlName, new AsyncCallback<User>() {

            public void onFailure(Throwable error) {
                // This will happen only if the user doesn't exist
                // so show the Error 404 page
                panel.clear();
                panel.add(builder.getError404());
            }

            public void onSuccess(User result) {
                loadUser(panel, result);
            }

        });

        return panel;

    }

    /**
     * Constructs the actual panel
     * @param result
     */
    private Widget loadUser(final FlowPanel panel, User result) {

        Panel nameEditPanel = new HTMLPanel("");
        panel.add(nameEditPanel);
        if (result.getFullName() != null) {
            nameEditPanel.add(new InlineLabel(result.getFullName()));
        }
        else
        {
            // No full name; display url name
            nameEditPanel.add(new InlineLabel(result.getUrlName()));
        }
        // Add "edit" link, if authorized
        if (result.userCanEdit())
        {
            nameEditPanel.add(new InlineLabel(" "));
            nameEditPanel.add(new InlineHyperlink("(edit)",
                "!/~" + result.getUrlName() + "/+edit"));
        }
        if (result.getEmail() != null) {
            panel.add(new Label("Email: " + result.getEmail()));
        }
        //panel.add(new Label("Role: " + result.getRole()));

        result.getDevTeams(new AsyncCallback<ModelCollection<DevTeam>>() {
            @Override
            public void onFailure(Throwable caught) {
                Mugle.showError(caught.getMessage());
            }

            @Override
            public void onSuccess(ModelCollection<DevTeam> result) {
                if (result.getCount() > 0) {
                    panel.add(new Label("This user is a member of the "
                        + "following developer teams:"));
                }
                else
                {
                    panel.add(new Label("This user has not uploaded any games."));
                }
                for (DevTeam dt : result.getCollection()) {
                    panel.add(new Anchor(dt.getFullName(), "#!/" + dt.getUrlName()));
                }
            }
        });

        // Set the window title
        this.setWindowTitle("MUGLE - " + result.getFullName());
        this.updateWindowTitle();

        return panel;

    }

}
