/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui;

import au.edu.unimelb.csse.mugle.client.*;
import au.edu.unimelb.csse.mugle.shared.model.*;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.*;

public abstract class ViewBuilder {

    private FlowPanel dataPanel = new FlowPanel();

    private String urlFragment = "";

    private String windowTitle = "MUGLE";


    /**
     * Default constructor
     */
    public ViewBuilder() {

    }

    /**
     * Constructor with urlFragment
     * @param urlFragment
     */
    public ViewBuilder(String urlFragment) {
        this.urlFragment = urlFragment;
    }

    /**
     * The main method that builds the whole view
     * @return
     */
    public abstract Widget buildView();

    /**
     * Clear the main panel and hides any messages
     */
    public void clearPanel() {
        this.hideMessage();
        this.dataPanel.clear();
    }

    public FlowPanel getDataPanel() {
        return this.dataPanel;
    }

    protected void setDataPanel(FlowPanel dataPanel) {
        this.dataPanel = dataPanel;
    }

    public String getUrlFragment() {
        return this.urlFragment;
    }

    public void setUrlFragment(String urlFragment) {
        this.urlFragment = urlFragment;
    }

    protected void showMessage(String message) {
        Mugle.showMessage(message);
    }

    protected void showError(String message) {
        Mugle.showError(message);
    }

    protected void hideMessage() {
        Mugle.hideMessage();
    }

    /**
     * Builds a widget that has an error due to incorrect user privileges.
     * @return
     */
    protected Widget getAuthorizationError() {
        Label label = new Label("You are not authorized to view this page.");
        label.addStyleName("error-auth");
        return label;
    }

    /**
     * Builds a widget that has an error due to incorrect user privileges.
     * @param role The required role to view the page.
     * @return
     */
    protected Widget getAuthorizationError(Role role) {

        VerticalPanel vp = new VerticalPanel();
        vp.addStyleName("error-auth");
        vp.add(new Label("You are not authorized to view this page."));

        switch (role) {

            case ADMIN:
                vp.add(new Label("Please login as an administrator."));
                break;

            case DEVELOPER:
                vp.add(new Label("Please login as a developer."));
                break;

        }

        return vp;

    }

    /**
     * Builds a common Error 404 page.
     * @param panel Main panel to add
     * @return The modified panel
     */
    protected Widget getError404() {

        FlowPanel panel = new FlowPanel();

        panel.add(new Label("Error 404"));
        panel.add(new Label("Page not found!"));
        //panel.add(new Label("Page at '" + this.getUrlFragment() + "' not found!"));
        panel.addStyleName("error-404");

        return panel;

    }

    public void updateWindowTitle() {
        Window.setTitle(this.windowTitle);
    }

    public String getWindowTitle() {
        return this.windowTitle;
    }

    protected void setWindowTitle(String windowTitle) {
        this.windowTitle = windowTitle;
    }

}
