/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui.admin;

import au.edu.unimelb.csse.mugle.client.*;
import au.edu.unimelb.csse.mugle.client.ui.ViewBuilder;
import au.edu.unimelb.csse.mugle.shared.model.Role;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;

public class AdminDataTestViewBuilder extends ViewBuilder {

    private Button populateButton = new Button("Populate Data");

    public AdminDataTestViewBuilder() {
        super();
    }

    public Widget buildView() {

        FlowPanel panel = this.getDataPanel();

        panel.clear();

        if (Mugle.getAuthUser().getUser().getRole() != Role.ADMIN) {
            panel.add(this.getAuthorizationError(Role.ADMIN));
            return panel;
        }

        return this.constructView();

    }

    private Widget constructView() {

        this.populateButton.addStyleName("sendButton");
        this.populateButton.addClickHandler(new ClickHandler() {

            DataTestServiceAsync dataTestService = GWT.create(DataTestService.class);

            @Override
            public void onClick(ClickEvent event) {

                dataTestService.populateDatastore(new AsyncCallback<String>() {

                    @Override
                    public void onFailure(Throwable caught) {
                        populateButton.setEnabled(true);
                        populateButton.setText("Try again! Populate Database");
                        Mugle.showError(caught.getMessage());

                    }

                    @Override
                    public void onSuccess(String result) {
                        populateButton.setText("Database successfully populated");
                        populateButton.setEnabled(true);

                        dataTestService.retrieveData(new AsyncCallback<String>() {

                            @Override
                            public void onFailure(Throwable caught) {
                                populateButton.setEnabled(true);
                                populateButton.setText("Second test failed");
                            }

                            @Override
                            public void onSuccess(String result) {
                                populateButton.setText("Data population and retrieval successful!");
                            }

                        });

                    }

                });

                populateButton.setEnabled(false);
                populateButton.setText("Processing");

            }

        });

        this.getDataPanel().add(this.populateButton);
        return this.getDataPanel();

    }

}
