/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui.admin;

import java.util.ArrayList;
import java.util.Collections;

import au.edu.unimelb.csse.mugle.client.*;
import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.model.services.DevTeamService;
import au.edu.unimelb.csse.mugle.shared.model.services.DevTeamServiceAsync;
import au.edu.unimelb.csse.mugle.client.model.comparators.DevTeamComparator;
import au.edu.unimelb.csse.mugle.client.ui.MugleUiBuilder;
import au.edu.unimelb.csse.mugle.client.ui.ViewBuilder;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.*;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.user.client.ui.HTMLTable.RowFormatter;

public class AdminDevTeamsViewBuilder extends ViewBuilder {

    private final int PAGE_SIZE = 20; //Number of results to display per page
    private int curPage = 0; //TODO: Needs to be controlled by the arrow buttons

    private FlowPanel datatable = new FlowPanel();
    private FlexTable devTeamFlexTable = new FlexTable();
    private TextBox devNameBox = new TextBox();
    private TextBox devFullNameBox = new TextBox();
    private Button addDevButton = new Button("Create");
    private ArrayList<DevTeam> devTeams = new ArrayList<DevTeam>();
    private ModelCollection<DevTeam> toDelete = new ModelCollection<DevTeam>();
    private HorizontalPanel hp = new HorizontalPanel();
    private Label navLabel = new Label();

    @Override
    public Widget buildView() {
        if (Mugle.getAuthUser().getUser().getRole() != Role.ADMIN) {
            datatable.add(this.getAuthorizationError(Role.ADMIN));
            return datatable;
        }
        constructView();
        return datatable;
    }

    private void constructView() {
        clearAll();

        datatable.add(new Label("Create a devTeam:"));

        //Add the Panel for DevTeam Creation
        FlexTable createPanel = new FlexTable();
        createPanel.setText(0, 0, "Name:");
        createPanel.setText(0, 1, "Full Name:");
        createPanel.setWidget(1, 0, devNameBox);
        createPanel.setWidget(1, 1, devFullNameBox);
        createPanel.setWidget(1, 2, addDevButton);

        datatable.add(createPanel);

        //Now Show the DevTeams on the current page
        showDevTeams();
        datatable.add(devTeamFlexTable);

        // Listen for mouse events on the Add button.
        addDevButton.addClickHandler(new ClickHandler() {
          public void onClick(ClickEvent event) {
            addDevTeam(devNameBox.getText(), devFullNameBox.getText());
          }
        });

    }

    /**
     * Shows the devTeams for the current page
     */
    private void showDevTeams() {
        DevTeam.getDevTeams(PAGE_SIZE*curPage, PAGE_SIZE,
                new AsyncCallback<ModelCollection<DevTeam>>() {

            @Override
            public void onFailure(Throwable caught) {
                  Mugle.showError(caught.getMessage());
            }

            @Override
            public void onSuccess(ModelCollection<DevTeam> result) {
                //When we get the DevTeams, create the table
                devTeamFlexTable.setStyleName("entities");

                devTeamFlexTable.setText(0, 1, "Unique Name");
                devTeamFlexTable.setText(0, 2, "Full Name");
                devTeamFlexTable.setText(0, 3, "Members");
                devTeamFlexTable.setText(0, 4, "Edit");
                //Format first row
                RowFormatter rf = devTeamFlexTable.getRowFormatter();
                rf.addStyleName(0, "even");

                for (DevTeam dt : result.getCollection()) {
                    addDevTeamToTable(dt);
                }

                constructPageNav(result);
            }
        });
    }


    /**
     * Shows an individual DevTeam in the table
     * @param dt the DevTeam to show
     */
    private void showDevTeam(DevTeam dt) {
        RowFormatter rf = devTeamFlexTable.getRowFormatter();
        final DevTeam temp = dt; // Need this for async calls
        final int row = devTeams.indexOf(dt) + 1;

        if (row-1 == -1) {
            /* Note this _should_ never happen - the row is -1 if the DevTeam isnt
             * found in the client side list.  This should be impossible due to the
             * way addDevTeamToTable() works
             */
            Mugle.showError("Error displaying DevTeam, does not exist on this page");
            return;
        }

        /* Checkbox adds/removes the devteam to the client side list of DevTeams
         * waiting to be deleted
         */
        CheckBox cb = new CheckBox();
        cb.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                boolean checked = ((CheckBox)event.getSource()).getValue();
                if (checked) {
                    toDelete.getCollection().add(temp);
                } else {
                    toDelete.getCollection().remove(temp);
                }
            }
        });
        //Sets the data in each row
        // Removed temporarily due to LP: #786418
        //devTeamFlexTable.setWidget(row, 0, cb);
        devTeamFlexTable.setWidget(row, 1, new Anchor(dt.getUrlName(), "#!/" + dt.getUrlName() + "/+edit"));

        final TextBox full = new TextBox();
        full.setText(dt.getFullName());

        devTeamFlexTable.setWidget(row, 2, full);

        //Display the number of members in the DevTeam
        dt.getUsers(new AsyncCallback<ModelCollection<User>>() {
            @Override
            public void onFailure(Throwable caught) {
                int row = devTeams.indexOf(temp) + 1;
                devTeamFlexTable.setText(row, 3, caught.getMessage());
            }

            @Override
            public void onSuccess(ModelCollection<User> result) {
                int row = devTeams.indexOf(temp) + 1;
                devTeamFlexTable.setText(row, 3, "(" + Integer.toString(result.getCount()) + ")");
            }

        });

        Button edit = new Button("Edit");
        devTeamFlexTable.setWidget(row, 4, edit);
        edit.addStyleDependentName("edit");
        edit.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                temp.setFullName(full.getText());
                temp.write(new AsyncCallback<DevTeam>() {
                    public void onFailure(Throwable caught) {
                        Mugle.showError(caught.getMessage());
                    }

                    @Override
                    public void onSuccess(DevTeam result) {
                        Mugle.showMessage(temp.getUrlName() + " successfully updated!");
                        devTeams.remove(row-1);
                        devTeamFlexTable.removeRow(row);
                        addDevTeamToTable(result);
                    }

                });
            }
        });

        // format the rows
        String style = ((row%2)!=0) ? "odd" : "even";
        rf.addStyleName(row, style);
    }

    /**
     * Adds a DevTeam to the table being rendered - It first inserts it
     * to the correct position according to the client side list of DevTeams
     * currently being rendered, if it is then outside the PAGE_SIZE, it is
     * dropped from the client side list and isnt rendered.
     * @param dt
     */
    private void addDevTeamToTable(DevTeam dt) {
        devTeams.add(dt);
        updateNavLabel();
        // Need to sort the client side list by UrlName
        Collections.sort(devTeams, new DevTeamComparator());

        //Remove any extra devTeams from our client side page list
        for (int i = PAGE_SIZE; i < devTeams.size(); i++) {
            devTeams.remove(i);
        }

        int row = devTeams.indexOf(dt);

        /* If the row == -1 then the devTeam isnt on our client side list
         * This means it was pruned from the end of our list and comes in
         * a later page
         */
        if (row != -1) {
            for (int i = row; i < devTeams.size(); i++) {
                showDevTeam(devTeams.get(i));
            }
        }
    }

    /**
     * Removes all the DevTeams in the client side toDelete list from
     * the table rendered.  This should only be called from removeDevTeams()
     * which removes the DevTeams from the datastore
     */
    private void removeDevTeamsFromTable() {
        //Remove each DevTeam from the table, and the client side list
        for (DevTeam dt : toDelete.getCollection()) {
            int row = devTeams.indexOf(dt);
            devTeams.remove(row);
            devTeamFlexTable.removeRow(row+1);
        }
        toDelete.getCollection().clear();

        //need to get the next devTeam
        DevTeam.getDevTeams(curPage*PAGE_SIZE + devTeams.size(),
                PAGE_SIZE - devTeams.size(),
                new AsyncCallback<ModelCollection<DevTeam>>() {

                    public void onFailure(Throwable caught) {
                        Mugle.showError(caught.getMessage());
                    }

                    public void onSuccess(ModelCollection<DevTeam> result) {
                        for (DevTeam dt : result.getCollection()) {
                            Mugle.showMessage(dt.getUrlName());
                            addDevTeamToTable(dt);
                        }

                        //reformat the rows
                        RowFormatter rf = devTeamFlexTable.getRowFormatter();
                        for (int row = 0; row <= devTeams.size(); row++) {
                            String style = ((row%2)!=0) ? "odd" : "even";
                            rf.setStyleName(row, style);
                        }
                        updateNavLabel();
                    }
        });
      }

    /**
     * Creates a new DevTeam and renders it in the table
     * @param text
     */
    private void addDevTeam(String name, String fullName) {
        Mugle.hideMessage();
        DevTeam devteam = new DevTeam();
        devteam.setUrlName(name);
        devteam.setFullName(fullName);
        devteam.create(new AsyncCallback<DevTeam>() {
              public void onFailure(Throwable error) {
                  Mugle.showError(error.getMessage());
              }

              public void onSuccess(DevTeam result) {
                  devNameBox.setText(""); // clears the textbox
                  devFullNameBox.setText("");
                  Mugle.showMessage("DevTeam '" + result.getUrlName() + "' successfully created!");
                  addDevTeamToTable(result);
              }
        });
    }

    /**
     * removes the devTeams in the list of DevTeams generated by the checkboxes
     */
    private void removeDevTeams() {
        Mugle.hideMessage();

        DevTeamServiceAsync dts = GWT.create(DevTeamService.class);
        dts.remove(toDelete, new AsyncCallback<Void>() {
            public void onFailure(Throwable caught) {
                Mugle.showError(caught.getMessage());
            }
            public void onSuccess(Void result) {
                removeDevTeamsFromTable();
            }
        });
    }

    /**
     * Constructs the Navigation bar down the bottom (with delete button)
     */
    private void constructPageNav(ModelCollection<DevTeam> result) {
        // add data details

        hp.setStylePrimaryName("entities-control");
        updateNavLabel();
        // Add a remove button for the DevTeams
        /* Removed temporarily due to LP: #786418
        Button removeDevTeamButton = new Button("Delete");
        removeDevTeamButton.addStyleDependentName("remove");
        removeDevTeamButton.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                Mugle.showMessage("Deleting!");
                removeDevTeams();
            }
        });

        /* The Horizontal Panel down the bottom with the delete button
         * and navigational buttons
         */
        //hp.add(removeDevTeamButton);
        hp.add(navLabel);

        hp.add(MugleUiBuilder.createPageNavigation(result,
                null, null, null, null));

        datatable.add(hp);
    }

    /**
     * Updates the Label showing "Showing results X-Y" based on the
     * current page number of results
     */
    private void updateNavLabel() {
        String text = "Showing results: " + Integer.toString(curPage*PAGE_SIZE+1)
                        + "-" + Integer.toString(curPage*PAGE_SIZE+devTeams.size());
        navLabel.setText(text);
    }

    /**
     * Clears all internal client datastructures
     */
    private void clearAll() {
        datatable.clear();
        devTeams.clear();
        hp.clear();
        toDelete.getCollection().clear();
        devTeamFlexTable.clear();
        devTeamFlexTable.removeAllRows();
        navLabel.setText("");
        devNameBox.setText("");
        devFullNameBox.setText("");
    }
}
