/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.client.ui.admin;

import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import au.edu.unimelb.csse.mugle.client.Mugle;
import au.edu.unimelb.csse.mugle.client.ui.ViewBuilder;
import au.edu.unimelb.csse.mugle.shared.model.ModelList;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.GameInstance;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.GameLog;

import com.google.gwt.cell.client.ActionCell;
import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.DateCell;
import com.google.gwt.cell.client.NumberCell;
import com.google.gwt.cell.client.TextCell;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.i18n.client.DateTimeFormat.PredefinedFormat;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.user.cellview.client.*;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.cellview.client.SimplePager.TextLocation;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;
import com.google.gwt.view.client.*;

/**
 * Loads the view for game log
 */
public class AdminGameLogViewBuilder extends ViewBuilder {

    private CellTable<GameLog> table;
    private HashMap<String, Column<GameLog, String>> customColumns;
    private HashMap<String, DataGrid<String>> customLists;
    private Panel mainPanel;
    private boolean isBuilt = false;
    private ViewBuilder parent;
    private String gameName;

    private final int PAGE_SIZE = Math.max(Window.getClientHeight() / 50, 15);

    public AdminGameLogViewBuilder(GameInstance instance, ViewBuilder parent) {
        super(instance.getToken());
        this.parent = parent;
        this.gameName = instance.getName();
        this.mainPanel = new HorizontalPanel();
        this.mainPanel.setPixelSize(-1, 35 * PAGE_SIZE);
        this.customColumns = new HashMap<String, Column<GameLog, String>>();
        this.customLists = new HashMap<String, DataGrid<String>>();
    }

    @Override
    public Widget buildView() {
        this.clearPanel();
        if (this.isBuilt) {
            this.getDataPanel().add(mainPanel);
            return this.getDataPanel();
        } else {
            isBuilt = true;
            return loadGameLogs(this.getDataPanel());
        }
    }

    private FlowPanel loadGameLogs(final FlowPanel panel) {

        String token = this.getUrlFragment();
        GameLog.getGameLogs(token, new AsyncCallback<ModelList<GameLog>>() {

            @Override
            public void onSuccess(ModelList<GameLog> result) {
                initTable(panel, result.getList());
            }

            @Override
            public void onFailure(Throwable caught) {
                Mugle.showError(caught.getMessage());
            }
        });
        setWindowTitle("MUGLE - " + gameName + " Game Logs");
        updateWindowTitle();

        return panel;
    }

    private void initTable(FlowPanel panel, List<GameLog> data) {

        this.table = new CellTable<GameLog>();
        table.setWidth("100%", false);
        table.getElement().getStyle().setProperty("textAlign", "center");
        table.setVisibleRange(0, Math.min(PAGE_SIZE, data.size()));
        table.setRowCount(data.size());

        ListDataProvider<GameLog> dataProvider = new ListDataProvider<GameLog>(
                data);
        dataProvider.addDataDisplay(table);

        ListHandler<GameLog> sortHandler = new ColumnSortEvent.ListHandler<GameLog>(
                dataProvider.getList());
        table.addColumnSortHandler(sortHandler);

        final SingleSelectionModel<GameLog> selectionModel = new SingleSelectionModel<GameLog>();
        table.setSelectionModel(selectionModel,
                DefaultSelectionEventManager.<GameLog> createDefaultManager());
        selectionModel
                .addSelectionChangeHandler(new SelectionChangeEvent.Handler() {

                    @Override
                    public void onSelectionChange(SelectionChangeEvent event) {
                        showList(selectionModel.getSelectedObject(), mainPanel);
                    }
                });

        initColumn(selectionModel, sortHandler, data);

        SimplePager.Resources pagerResources = GWT
                .create(SimplePager.Resources.class);
        SimplePager pager = new SimplePager(TextLocation.CENTER,
                pagerResources, false, 0, false);
        pager.setDisplay(table);
        pager.setPageSize(Math.min(PAGE_SIZE, data.size()));
        pager.getElement().getStyle().setProperty("float", "left");

        VerticalPanel vPanel = new VerticalPanel();
        vPanel.add(table);
        HorizontalPanel hPanel = new HorizontalPanel();
        Button button = new Button("Back to Monitor");
        button.getElement().getStyle().setProperty("marginRight", "100px");
        button.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                parent.buildView();
            }
        });
        hPanel.add(button);
        hPanel.add(pager);
        vPanel.add(hPanel);
        mainPanel.add(vPanel);
        panel.add(mainPanel);

    }

    private void initColumn(SelectionModel<GameLog> selectionModel,
            ListHandler<GameLog> sortHandler, List<GameLog> data) {

        // Game Name.
        Column<GameLog, Number> nameColumn = new Column<GameLog, Number>(
                new NumberCell()) {

            @Override
            public Number getValue(GameLog gl) {
                return null;
            }

            @Override
            public void render(Cell.Context context, GameLog gl,
                    SafeHtmlBuilder sb) {
                sb.append(SafeHtmlUtils.fromTrustedString(String
                        .valueOf(context.getIndex() + 1)));
            }
        };
        String title = gameName.length() > 20 ? gameName.substring(0, 19)
                + "..." : gameName;
        table.addColumn(nameColumn, title);

        // Created Time.
        DateCell dateCell = new DateCell(
                DateTimeFormat.getFormat(PredefinedFormat.DATE_TIME_MEDIUM));
        Column<GameLog, Date> timeColumn = new Column<GameLog, Date>(dateCell) {

            @Override
            public Date getValue(GameLog gl) {
                return gl.getCreatedTime();
            }
        };
        timeColumn.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
        timeColumn.setSortable(true);
        sortHandler.setComparator(timeColumn, new Comparator<GameLog>() {
            public int compare(GameLog o1, GameLog o2) {
                return o1.getCreatedTime().compareTo(o2.getCreatedTime());
            }
        });
        table.addColumn(timeColumn, "Created Time");

        // Message.
        final ViewBuilder thisPanel = this;
        Column<GameLog, GameLog> messageColumn = new Column<GameLog, GameLog>(
                new ActionCell<GameLog>("view",
                        new ActionCell.Delegate<GameLog>() {

                            @Override
                            public void execute(GameLog gl) {
                                getDataPanel().clear();
                                ViewBuilder logView = new AdminLogMessageViewBuilder(
                                        gameName, gl.getToken(), thisPanel);
                                getDataPanel().add(logView.buildView());
                                logView.updateWindowTitle();
                            }
                        }) {
                    @Override
                    public void render(Cell.Context context, GameLog gl,
                            SafeHtmlBuilder sb) {
                        if (gl.hasLogMessages())
                            super.render(context, gl, sb);
                    }
                }) {

            @Override
            public GameLog getValue(GameLog gl) {
                return gl;
            }
        };
        messageColumn.setSortable(true);
        sortHandler.setComparator(messageColumn, new Comparator<GameLog>() {
            public int compare(GameLog o1, GameLog o2) {
                return ((Boolean) o2.hasLogMessages()).compareTo(o1
                        .hasLogMessages());
            }
        });
        table.addColumn(messageColumn, "Messages");

        // Game custom columns.
        for (GameLog gl : data) {
            if (gl.getKeyStringPairs() == null)
                continue;
            for (String k : gl.getKeyStringPairs().keySet()) {
                if (!customColumns.containsKey(k))
                    addCustomColumn(k);
            }
        }
    }

    protected void showList(GameLog data, Panel panel) {
        if (data.getKeyListPairs() == null)
            return;
        for (DataGrid<?> list : customLists.values()) {
            list.setVisible(false);
        }
        DataGrid<String> list;
        for (String k : data.getKeyListPairs().keySet()) {
            if (customLists.containsKey(k)) {
                list = customLists.get(k);
                list.setVisible(true);
            } else {
                list = new DataGrid<String>();
                list.setPixelSize(160, 35 * PAGE_SIZE);
                list.getElement().getStyle().setProperty("marginLeft", "10px");
                list.getElement().getStyle().setProperty("textAline", "center");
                Column<String, String> column = new Column<String, String>(
                        new TextCell()) {
                    @Override
                    public String getValue(String object) {
                        return object;
                    }
                };
                list.addColumn(column, k);
                customLists.put(k, list);
                panel.add(list);
            }
            list.setRowData(data.getKeyListPairs().get(k));
        }
    }

    private void addCustomColumn(final String title) {
        Column<GameLog, String> column = new Column<GameLog, String>(
                new TextCell()) {

            @Override
            public String getValue(GameLog object) {
                return object.getKeyStringPairs().get(title);
            }
        };
        table.addColumn(column, title);
        customColumns.put(title, column);
    }

}
