/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.server;

import java.util.List;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import au.edu.unimelb.csse.mugle.client.*;
import au.edu.unimelb.csse.mugle.server.api.KeyValueServiceImpl;
import au.edu.unimelb.csse.mugle.server.model.*;
import au.edu.unimelb.csse.mugle.server.model.services.*;
import au.edu.unimelb.csse.mugle.shared.KVPtester;
import au.edu.unimelb.csse.mugle.shared.model.*;
@SuppressWarnings("serial")
public class DataTestServiceImpl extends RemoteServiceServlet implements DataTestService {

  //Throws Exception because i dont want to list ALL the shared exceptions
  public String populateDatastore() throws Exception {

      System.err.println("getting current user");

      // NOTE: Do not manually create users as they rely on GoogleID!!!
      UserData curUser = UserGetter.getCurrentUser();

      System.err.println("started");

      DevTeam[] devteams = new DevTeam[] {
                                        new DevTeam()
                                      };
      devteams[0].setUrlName("test");
      devteams[0].setFullName("Test Team");
      DevTeamServiceImpl dts = new DevTeamServiceImpl();
      devteams[0] = dts.create(devteams[0]);

      System.err.println("devteam created");

      // Adds the current user to the devTeam
      UserServiceImpl us = new UserServiceImpl();
      User cu = us.getByKey(curUser.getPrimaryKey());
      dts.addUser(devteams[0], cu);

      System.err.println("user added");

      Game[] games = new Game[] {
                                  new Game()
                                 };
      games[0].setUrlName("test");
      games[0].setDevTeamKey(devteams[0].getPrimaryKey());
      GameServiceImpl gs = new GameServiceImpl();
      gs.create(games[0]);

      System.err.println("game created");

      // need to retrieve again so we get key
      games[0] = gs.getByName("test", "test");

      GameVersion[] gameversions = new GameVersion[] {
                                        new GameVersion()
                                      };
      gameversions[0].setDescription("A sample game");
      gameversions[0].setDispVersion("1.0");
      gameversions[0].setFullName("Test Game");
      gameversions[0].setVersion(0);
      gameversions[0].setGameKey(games[0].getPrimaryKey());

      GameVersionServiceImpl gvs = new GameVersionServiceImpl();
      gameversions[0] = gvs.create(gameversions[0]);

      System.out.println("game version created!");

      //games[0].setActiveVersion(gameversions[0]);
      //gs.write(games[0]);
      // this should automatically set the active version
      gs.addGameVersion(games[0], gameversions[0]);

      System.out.println("active version set!");


      GameFile[] gamefiles = new GameFile[] {
                                        new GameFile()
                                      };

      gamefiles[0].setPath("index.html");
      gamefiles[0].setMimeType("text/html");
      gamefiles[0].setGameVersionKey(gameversions[0].getPrimaryKey());

      GameFileServiceImpl gfs = new GameFileServiceImpl();
      gfs.create(gamefiles[0]);

      // Test Key value pairs

      KeyValueServiceImpl kvps = new KeyValueServiceImpl();

      kvps.put(games[0].getGameToken(), "TestString", "Test Value ABCDE!");
      System.err.println("Using setValue(): " + kvps.get(games[0].getGameToken(), "TestString"));

      KVPtester kvp = new KVPtester("Test1", 3);
      kvp.addToArray(true);
      kvp.addToArray(false);
      kvps.put(games[0].getGameToken(), "TestSerializable", kvp);
      System.err.println("Getting a serialized object; "
               + kvps.get(games[0].getGameToken(), "TestSerializable").toString());

      Achievement[] achievements = new Achievement[] {
              new Achievement("bool", "Test Achievement",
                      "This is testing a boolean achievement", 0, games[0].getPrimaryKey()),
              new Achievement("prog", "Test Achievement",
                      "This is testing a progress achievement", 7, games[0].getPrimaryKey())
      };

      AchievementServiceImpl as = new AchievementServiceImpl();
      as.create(achievements[0]);
      as.create(achievements[1]);

      System.err.println("Data test completed");

      return "Data addition succeful!";

  }

  @SuppressWarnings("unchecked")
  @Override
  public String retrieveData() {

    PersistenceManager pm = PMF.getManager();
    KeyValuePairData kvpn = null;

    try {
        Query q = pm.newQuery(KeyValuePairData.class, "key == k");
        q.declareParameters("String k");
        System.err.println("Setup query");
        List<KeyValuePairData> results = (List<KeyValuePairData>) q.execute("TestString");
        if (!results.isEmpty()) {
            kvpn = results.get(0);
            System.err.println("Result found");
        }
    } finally {
        pm.close();
    }
    System.err.println("Query complete and returned");
    if (kvpn != null) {
        System.err.println("Returned getValue(): " + (String)kvpn.getValue());
    }

    return null;

  }

}
