/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.server;

import java.io.OutputStream;
import java.io.IOException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.server.model.GameData;
import au.edu.unimelb.csse.mugle.server.model.GameGetter;
import au.edu.unimelb.csse.mugle.server.model.GameVersionData;
import au.edu.unimelb.csse.mugle.server.model.GameVersionGetter;
import au.edu.unimelb.csse.mugle.server.model.GameFileData;
import au.edu.unimelb.csse.mugle.server.model.GameFileGetter;
import au.edu.unimelb.csse.mugle.server.model.GameFileContents;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.DevTeamNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.GameFileNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.GameFileContentsNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.GameNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.GameVersionNotExists;

@SuppressWarnings("serial")
public class GameFileServer extends HttpServlet {
    
    public void doGet(HttpServletRequest req, HttpServletResponse res) 
        throws IOException {
        String path = req.getPathInfo();
        if (path.length() > 0 && path.charAt(0) == '/')
            path = path.substring(1);
        String[] pathSegments = path.split("/", 3);
        // XXX Assumes a devteam/game, not a promoted game
        if (pathSegments.length == 2)
        {
            // Redirect to the same path with a / on the end
            // (Otherwise relative paths will be incorrect)
            res.sendRedirect(req.getRequestURI() + "/");
            return;
        }
        if (pathSegments.length != 3)
        {
            res.sendError(404, "Not found");
            return;
        }
        String devteamName = pathSegments[0];
        String gameName = pathSegments[1];
        String filePath = pathSegments[2];

        GameFileData file;
        GameFileContents contents;
        try
        {
            GameData game = GameGetter.getGameByDevTeam(devteamName,gameName);
            Key versionKey = game.getActiveVersion();
            GameVersionData version =
                GameVersionGetter.getGameVersion(versionKey);
            if (filePath.equals(""))
            {
                // Special case: Alias to whichever file is named by the
                // GameVersion.entryPoint field
                filePath = version.getEntryPoint();
            }
            file = GameFileGetter.getGameFile(game, version, filePath);
            contents = GameFileGetter.getGameFileContents(file);
        }
        catch (DevTeamNotExists e)
        {
            res.sendError(404, e.toString());
            return;
        }
        catch (GameNotExists e)
        {
            res.sendError(404, e.toString());
            return;
        }
        catch (GameVersionNotExists e)
        {
            res.sendError(404, e.toString());
            return;
        }
        catch (GameFileNotExists e)
        {
            res.sendError(404, e.toString());
            return;
        }
        catch (GameFileContentsNotExists e)
        {
            res.sendError(404, e.toString());
            return;
        }

        // Serve the file on the HTTP response
        res.setContentType(file.getMimeType());
        byte[] bytes = contents.getContents();
        res.setContentLength(bytes.length);
        res.getOutputStream().write(bytes);
    }

    /*
    public void doPost(HttpServletRequest req, HttpServletResponse res) 
        throws ServletException, IOException{
        BlobstoreService blobstoreService = BlobstoreServiceFactory.getBlobstoreService();
        
        Map<String, BlobKey> blobs = blobstoreService.getUploadedBlobs(req);
        /* We want to iterate over each of the blobs, and create the 
         * appropriate datastore classes
         *
        for(Entry<String, BlobKey> blobEntry : blobs.entrySet()){
            
        }
        
    } */
}
