/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.api;

import au.edu.unimelb.csse.mugle.client.api.HighscoreService;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.UserGameProfileGetter;
import au.edu.unimelb.csse.mugle.shared.api.GameTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.server.model.UserGameProfileData;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import javax.jdo.PersistenceManager;

/**
 * The server side implementation of KeyValueService.
 */
@SuppressWarnings("serial")
public class HighscoreServiceImpl extends RemoteServiceServlet implements
        HighscoreService {

    public int getHighScore(String gameToken) throws GameTokenError {
        UserGameProfileData ugp;
        ugp = UserGameProfileGetter.getCurrentUserGameProfile(gameToken);
        return ugp.getHighscore();
    }

    public void saveScore(String gameToken, int score) throws GameTokenError {
        PersistenceManager pm = PMF.getManager();
        UserGameProfileData ugp = null;
        try {
            ugp = UserGameProfileGetter
                    .getCurrentUserGameProfile(pm, gameToken);
            if (score > ugp.getHighscore())
                ugp.setHighscore(score);
        } finally {
            pm.close();
        }
    }

    @Override
    public void saveScore(String gameToken, int score, String instanceToken)
            throws GameTokenError, InstanceTokenError {
        PersistenceManager pm = PMF.getManager();
        UserGameProfileData ugp = null;
        try {
            if (instanceToken == null)
                ugp = UserGameProfileGetter.getCurrentUserGameProfile(pm,
                        gameToken);
            else
                ugp = UserGameProfileGetter.getCurrentUserGameInstanceProfile(
                        pm, instanceToken);
            if (score > ugp.getHighscore())
                ugp.setHighscore(score);
        } finally {
            pm.close();
        }
    }

    @Override
    public int getHighScore(String gameToken, String instanceToken)
            throws GameTokenError, InstanceTokenError {
        UserGameProfileData ugp;
        if (instanceToken == null)
            ugp = UserGameProfileGetter.getCurrentUserGameProfile(gameToken);
        else
            ugp = UserGameProfileGetter
                    .getCurrentUserGameInstanceProfile(instanceToken);
        return ugp.getHighscore();
    }
}
