/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.api;

import java.io.Serializable;

import au.edu.unimelb.csse.mugle.client.api.KeyValueService;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.KeyValuePairGetter;
import au.edu.unimelb.csse.mugle.shared.api.GameTokenError;
import au.edu.unimelb.csse.mugle.shared.api.KeyError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.server.model.KeyValuePairData;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import javax.jdo.PersistenceManager;

/**
 * The server side implementation of KeyValueService.
 */
@SuppressWarnings("serial")
public class KeyValueServiceImpl extends RemoteServiceServlet implements
        KeyValueService {
    public void put(String gameToken, String key, Serializable value)
            throws GameTokenError {
        PersistenceManager pm = PMF.getManager();
        try {
            KeyValuePairData kvp = KeyValuePairGetter.getKeyValuePair(pm, key,
                    gameToken, true);
            kvp.setValue(value);
        } catch (KeyError e) {
            // XXX This should never happen (since we are using
            // createIfNotFound=true)
            throw new Error(e);
        } finally {
            pm.close();
        }
    }

    public Serializable get(String gameToken, String key)
            throws GameTokenError, KeyError {
        KeyValuePairData kvp = KeyValuePairGetter.getKeyValuePair(key,
                gameToken);
        return (Serializable) kvp.getValue();
    }

    public boolean containsKey(String gameToken, String key)
            throws GameTokenError {
        try {
            KeyValuePairGetter.getKeyValuePair(key, gameToken);
        } catch (KeyError e) {
            return false;
        }
        return true;

    }

    @Override
    public void put(String gameToken, String key, Serializable value,
            String instanceToken) throws GameTokenError, InstanceTokenError {
        PersistenceManager pm = PMF.getManager();
        KeyValuePairData kvp = null;
        try {
            if (instanceToken == null)
                kvp = KeyValuePairGetter.getKeyValuePair(pm, key, gameToken,
                        true);
            else
                kvp = KeyValuePairGetter.getInstanceKeyValuePair(pm, key,
                        instanceToken, true);
            kvp.setValue(value);
        } catch (KeyError e) {
            throw new Error(e);
        } finally {
            pm.close();
        }
    }

    @Override
    public Serializable get(String gameToken, String key, String instanceToken)
            throws GameTokenError, KeyError, InstanceTokenError {
        KeyValuePairData kvp = null;
        if (instanceToken == null)
            kvp = KeyValuePairGetter.getKeyValuePair(key, gameToken);
        else
            kvp = KeyValuePairGetter.getInstanceKeyValuePair(key,
                    instanceToken, false);
        return (Serializable) kvp.getValue();
    }

    @Override
    public boolean containsKey(String gameToken, String key,
            String instanceToken) throws GameTokenError, InstanceTokenError {
        try {
            if (instanceToken == null)
                KeyValuePairGetter.getKeyValuePair(key, gameToken);
            else
                KeyValuePairGetter.getInstanceKeyValuePair(key, instanceToken,
                        false);
        } catch (KeyError e) {
            return false;
        }
        return true;
    }

}
