/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.jdo.PersistenceManager;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.annotations.*;

@MugleData
@PersistenceCapable
public class AchievementData extends ModelDataClass<Long, Achievement, AchievementData> {


    // Fields

    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private String name;

    @Persistent
    private String displayName;

    @Persistent
    private String description;

    @Persistent
    private Key game;

    @Persistent
    private Integer maxProgress;

    @Persistent
    private Set<Key> userAchievements;

    @Persistent
    private Key createdUser;

    @Persistent
    private Date createdTime;

    @Persistent
    private Key updatedUser;

    @Persistent
    private Date updatedTime;


    //TODO: Image

    public AchievementData(String name, String displayName, String description,
            int maxProgress, Key game) {
        this.userAchievements = new HashSet<Key>();
        this.name = name;
        this.displayName = displayName;
        this.description = description;
        this.maxProgress = maxProgress;
        this.game = game;
    }


    // Constructors
    public AchievementData() {

        this.userAchievements = new HashSet<Key>();
        this.name = null;
        this.displayName = null;
        this.description = null;
        this.maxProgress = null;
        this.game = null;
        //TODO: Image

    }

    // Getters

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setPrimaryKey")
    public Long getPrimaryKey() {
        if (this.id == null) {
            return null;
        }
        return this.id.getId();
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedTo="setName")
    public String getName() {
        return this.name;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setDisplayName")
    public String getDisplayName() {
        return this.displayName;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setDescription")
    public String getDescription() {
        return this.description;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setMaxProgress")
    public Integer getMaxProgress() {
        return this.maxProgress;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setGameKey", overrideBy="keyToGame")
    public Key getGame() {
        return this.game;
    }

    @GetterUserLevel(privateView=Role.NONE, publicView=Role.NONE, list=true)
    public Set<Key> getUserAchievements() {
        return this.userAchievements;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedUser", overrideBy="keyToCreated")
    public Key getCreatedUser() {
        return this.createdUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedTime")
    public Date getCreatedTime() {
        return this.createdTime;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedUser", overrideBy="keyToUpdated")
    public Key getUpdatedUser() {
        return this.updatedUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedTime")
    public Date getUpdatedTime() {
        return this.updatedTime;
    }


    // Setters

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedUser(Key createdUser) {
        this.createdUser = createdUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedUser(Key updatedUser) {
        this.updatedUser = updatedUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    /* in the case of wanting to change an achievement, better
     * to leave the old ID, so that users dont lose their achievements if they ever need
     * updating
     */
    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getName")
    public void setName(String newName) {
        this.name = newName;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getDisplayName")
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getDescription")
    public void setDescription(String newDescription) {
        this.description = newDescription;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getMaxProgress")
    public void setMaxProgress(Integer newProgress) {
        this.maxProgress = newProgress;
    }

    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedBy="getGameKey", overrideBy="gameFromKey")
    public void setGame(GameData game) {
        if (game == null) {
            this.game = null;
            return;
        }
        this.game = game.getServerKey();
        game.getAchievements().add(this.id);
    }

    // Misc methods

    public void addUserAchievement(UserAchievementData ua)
    {
        ua.setAchievement(this);
    }

    public void removeUserAchievement(UserAchievementData ua)
    {
        this.userAchievements.remove(ua.getServerKey());
    }

    @Override
    public Key getServerKey() {
        return this.id;
    }

    @Override
    public void setServerKey(Key key) {
        this.id = key;
    }

    @Override
    public void ClearFields() {
        this.setDescription(null);
        this.setGame(null);
        this.setMaxProgress(null);
        this.setName(null);
        this.setServerKey(null);
    }

    @Override
    public Class<Achievement> getClientClassType() {
        return Achievement.class;
    }

    public Long keyToGame() {
        if (this.game == null) {
            return null;
        }
        return this.game.getId();
    }

    public GameData gameFromKey(Long key) {
        if (key == null) { return null; }
        PersistenceManager pm = PMF.getManager();
        GameData result = null;
        try {
            result = pm.getObjectById(GameData.class, key);
        } finally {
            pm.close();
        }
        return result;
    }

    public Long keyToCreated() {
        if (this.createdUser == null) { return null; }
        return this.createdUser.getId();
    }

    public Long keyToUpdated() {
        if (this.updatedUser == null) { return null; }
        return this.updatedUser.getId();
    }

}
