package au.edu.unimelb.csse.mugle.server.model;

import java.util.List;

import javax.jdo.JDOObjectNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.DevTeamNotExists;

public class DevTeamGetter {

    /**
     * Gets the DevTeamData by its urlName - READ ONLY
     * @param name The name of the DevTeamData
     * @return The DevTeamData - READ ONLY
     * @throws DevTeamDataNotExists
     */
    public static DevTeamData getDevTeam(String name) throws DevTeamNotExists {
        PersistenceManager pm = PMF.getManager();
        try {
            return getDevTeam(pm, name);
        } finally {
            pm.close();
        }
    }
    
    /**
     * Gets the DevTeamData by its urlName, for editing in the datastore
     * The Persistence Manager must be handled by the caller
     * @param pm The PersistenceManager
     * @param name The Name of the Dev Team
     * @return The DevTeamData
     * @throws DevTeamDataNotExists
     */
    @SuppressWarnings("unchecked")
    public static DevTeamData getDevTeam(PersistenceManager pm, String name) throws DevTeamNotExists {
        Query q = pm.newQuery(DevTeamData.class, "urlName == u");
        q.declareParameters("String u");
        List<DevTeamData> results = (List<DevTeamData>) q.execute(name);
        
        if (results.isEmpty()) {
            throw new DevTeamNotExists(name);
        }
        
        return results.get(0);
    }
    /**
     * Gets the DevTeamDataData by its primary key - READ ONLY
     * @param primaryKey
     * @return The DevTeamDataData - READ ONLY
     * @throws DevTeamDataDataNotExists
     */
    public static DevTeamData getDevTeam(Long primaryKey) throws DevTeamNotExists {
        PersistenceManager pm = PMF.getManager();
        try {
            return getDevTeam(pm, primaryKey);
        } finally {
            pm.close();
        }
    }
    
    /**
     * Gets the DevTeamDataData by its primary key, for editing in the datastore
     * The Persistence Manager must be handled by the caller
     * @param pm The PersistenceManager
     * @param primaryKey
     * @return The DevTeamDataData
     * @throws DevTeamDataDataNotExists
     */
    public static DevTeamData getDevTeam(PersistenceManager pm, Long primaryKey) throws DevTeamNotExists {
        try {
            DevTeamData devTeam = pm.getObjectById(DevTeamData.class, primaryKey);
            return devTeam;
        } catch (JDOObjectNotFoundException e) {
            throw new DevTeamNotExists(primaryKey);
        }
    }

    /**
     * Gets the DevTeamDataData by its primary key - READ ONLY
     * @param primaryKey
     * @return The DevTeamDataData - READ ONLY
     * @throws DevTeamDataDataNotExists
     */
    public static DevTeamData getDevTeam(Key primaryKey) throws DevTeamNotExists {
        PersistenceManager pm = PMF.getManager();
        try {
            return getDevTeam(pm, primaryKey);
        } finally {
            pm.close();
        }
    }

    /**
     * Gets the DevTeam associated with the given DevTeamData primary key.
     * @param pm The PersistenceManager
     * @param key The server-side key (ModelDataClass.getServerKey())
     * @return The DevTeamData object directly off the datastore.
     * @throws DevTeamNotExists 
     */
    public static DevTeamData getDevTeam(PersistenceManager pm, Key key) 
           throws DevTeamNotExists {
        try {
            DevTeamData devTeam = pm.getObjectById(DevTeamData.class, key);
            return devTeam;
        } catch (JDOObjectNotFoundException e) {
            throw new DevTeamNotExists(key.getId());
        }
    }
    
}
