/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;

import javax.jdo.PersistenceManager;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.Blob;

/** The contents of a GameFile.
 * This simple class contains only the contents and no other meta-data. It
 * may be referenced by one or more GameFile objects.
 */
@PersistenceCapable
public class GameFileContents {
    // Fields

    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private Blob data;

    /** Create a new file contents blob, from a given input stream.
     @param size The number of bytes to read from stream.
     */
    public GameFileContents(InputStream stream, long size)
        throws IOException
    {
        this.setContents(stream, size);
    }

    // Getters

    public Key getServerKey() {
        return this.id;
    }

    // Misc methods

    /** Get the contents of the file as a byte array.
     */
    public byte[] getContents()
    {
        return this.data.getBytes();
    }

    /** Write the contents of the file to the OutputStream object.
     * @param writer Stream object to receive the contents of the file.
     */
    public void getContents(OutputStream writer)
        throws IOException
    {
        byte[] bytes = this.data.getBytes();
        writer.write(bytes);
    }

    /** Read the contents of an InputStream and store it in the database as
     * the contents of this file.
     * @param stream Stream object to read contents of file from.
     @param size The number of bytes to read from stream.
     */
    public void setContents(InputStream stream, long size)
        throws IOException
    {
        //TODO: check that the DevTeam's space limit hasn't been exceeded.
        byte[] bytes = new byte[(int)size];
        // Read in a loop
        // (stream.read might read fewer than the requested number of bytes)
        int offset = 0;
        int remaining = (int)size;
        while (remaining > 0)
        {
            int bytes_read = stream.read(bytes, offset, remaining);
            offset += bytes_read;
            remaining -= bytes_read;
        }
        this.data = new Blob(bytes);
    }
}
