/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Blob;
import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.annotations.*;

@MugleData
@PersistenceCapable
public class GameVersionData extends ModelDataClass<Long, GameVersion, GameVersionData> {


    // Fields

    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private Key game;

    //Unique per game, handled by the server
    @Persistent
    private Integer version;

    @Persistent
    private String dispVersion;

    @Persistent
    private String fullName;

    @Persistent
    private String description; //Do not know if there are restrictions on Length for JDO

    //TODO: Implement properly
    @Persistent(serialized = "true")
    private Blob image;

    @Persistent
    private Key promotedGame;

    @Persistent
    private Set<Key> gameFiles;

    @Persistent
    private Key createdUser;

    @Persistent
    private Date createdTime;

    @Persistent
    private Key updatedUser;

    @Persistent
    private Date updatedTime;

    @Persistent
    private String entryPoint;


    // Constructors

    public GameVersionData() {

        this.gameFiles = new HashSet<Key>();
        this.promotedGame = null;
        this.game = null;
        this.image = null;
        this.version = null;
        this.dispVersion = null;
        this.fullName = null;
        this.description = null;
        this.entryPoint = null;
        //TODO: Image
    }

    // Getters

    public GameVersionData(String displayVersion, String fullName, String description) {
        this.gameFiles = new HashSet<Key>();
        this.promotedGame = null;
        this.game = null;
        this.image = null;
        this.version = null;
        this.dispVersion = displayVersion;
        this.fullName = fullName;
        this.description = description;
        this.entryPoint = null;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setPrimaryKey")
    public Long getPrimaryKey() {
        if (this.id == null) {
            return null;
        }
        return this.id.getId();
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setGameKey", overrideBy="keyToGame")
    public Key getGame() {
        return this.game;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setDispVersion")
    public String getDisplayVersion() {
        return this.dispVersion;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setFullName")
    public String getFullName() {
        return this.fullName;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setDescription")
    public String getDescription() {
        return this.description;
    }

    /* TODO:
     *  public <Type> getImage() {
     *      return this.Image;
     *  }
     */

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, list=true)
    public Set<Key> getGameFiles() {
        return this.gameFiles;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setPromotedGameKey", overrideBy = "keyToPromotedGame")
    public Key getPromotedGame() {
        return this.promotedGame;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setVersion")
    public Integer getVersion() {
        return this.version;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedUser", overrideBy="keyToCreated")
    public Key getCreatedUser() {
        return this.createdUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedTime")
    public Date getCreatedTime() {
        return this.createdTime;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedUser", overrideBy="keyToUpdated")
    public Key getUpdatedUser() {
        return this.updatedUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedTime")
    public Date getUpdatedTime() {
        return this.updatedTime;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setEntryPoint")
    public String getEntryPoint() {
        return this.entryPoint;
    }


    // Setters

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedUser(Key createdUser) {
        this.createdUser = createdUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedUser(Key updatedUser) {
        this.updatedUser = updatedUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getDescription")
    public void setDescription(String description) {
        this.description = description;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getDisplayVersion")
    public void setDispVersion(String dispVersion) {
        this.dispVersion = dispVersion;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getFullName")
    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    @SuppressWarnings("unchecked")
    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getGameKey", overrideBy="gameFromKey")
    public void setGame(GameData game) {

        if (game == null) {
            this.game = null;
            return;
        }

        this.game = game.getServerKey();
        game.getGameVersions().add(this.id);

        PersistenceManager pm = PMF.getManager();
        try {
            Query q = pm.newQuery(GameVersionData.class, "game == g");
            q.declareParameters("Long g");
            q.setOrdering("version desc");
            List<GameVersionData> results = (List<GameVersionData>) q.execute(game.getPrimaryKey());
            if (results.isEmpty()) {
                this.version = 0;
            } else {
                this.version = results.get(0).getVersion() + 1;
            }
        } finally {
            pm.close();
        }
    }

    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedBy="getPromotedGameKey", overrideBy="promotedGameFromKey")
    public void setPromotedGame(PromotedGameData promotedGame) {
        if (promotedGame == null) {
            this.promotedGame = null;
            return;
        }
        this.promotedGame = promotedGame.getServerKey();
        if (promotedGame.getGameVersion() == null) {
            promotedGame.setGameVersion(this);
        }
    }

    public void addGameFile(GameFileData gameFile) {
        gameFile.setGameVersion(this);
    }

    public void removeGameFile(GameFileData gameFile) {
        this.gameFiles.remove(gameFile.getServerKey());
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getVersion")
    public void setVersion(Integer version) {
        this.version = version;
    }

    @SetterUserLevel(privateView=Role.GUEST, publicView=Role.ADMIN, mappedBy="getEntryPoint")
    public void setEntryPoint(String entryPoint) {
        this.entryPoint = entryPoint;
    }

    // TODO: setImage(Image image);


    @Override
    public Key getServerKey() {
        return this.id;
    }

    @Override
    public void setServerKey(Key key) {
        this.id = key;
    }

    @Override
    public void ClearFields() {
        this.setDescription(null);
        this.setFullName(null);
        this.setDispVersion(null);
        this.setGame(null);
        this.setServerKey(null);
        this.setPromotedGame(null);
    }

    @Override
    public Class<GameVersion> getClientClassType() {
        return GameVersion.class;
    }

    public Long keyToGame() {
        if (this.game == null) {
            return null;
        }
        return this.game.getId();
    }

    public Long keyToPromotedGame() {
        if (this.promotedGame == null) {
            return null;
        }
        return this.promotedGame.getId();
    }

    public GameData gameFromKey(Long key) {
        if (key == null) { return null; }
        PersistenceManager pm = PMF.getManager();
        GameData result = null;
        try {
            result = pm.getObjectById(GameData.class, key);
        } finally {
            pm.close();
        }
        return result;
    }

    public PromotedGameData promotedGameFromKey(Long key) {
        if (key == null) { return null; }
        PersistenceManager pm = PMF.getManager();
        PromotedGameData result = null;
        try {
            result = pm.getObjectById(PromotedGameData.class, key);
        } finally {
            pm.close();
        }
        return result;
    }

    public Long keyToCreated() {
        if (this.createdUser == null) { return null; }
        return this.createdUser.getId();
    }

    public Long keyToUpdated() {
        if (this.updatedUser == null) { return null; }
        return this.updatedUser.getId();
    }

}
