/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.util.Date;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.shared.model.ModelClientClass;

/**
 * Base model data class that should be extended by all shared model class entities.
 * @param <S> The primary key type
 * @param <T> The client class type (usually client counter part type)
 * @param <U> The server class type (usually self type)
 */
public abstract class ModelDataClass<S, T extends ModelClientClass<S,T>, U extends ModelDataClass<S,T,U>> {


    /**
     * The getter for the Key type primary key stored in the datastore.
     * @return The corresponding key of the object
     */
    public abstract Key getServerKey();

    /**
     * The setter the Key type primary key stored in the datastore.
     * @param key The key to be set
     */
    public abstract void setServerKey(Key key);

    /**
     * The getter for the primary key
     * @return The primary key
     */
    public abstract S getPrimaryKey();

    /**
     * The getter for the primary key
     * @return The primary key
     */
    public abstract Class<T> getClientClassType();

    /**
     * Clears all fields
     */
    protected abstract void ClearFields();

    // Logging purposes

    public abstract Key getCreatedUser();

    public abstract Date getCreatedTime();

    public abstract Key getUpdatedUser();

    public abstract Date getUpdatedTime();

    public abstract void setCreatedUser(Key user);

    public abstract void setCreatedTime(Date time);

    public abstract void setUpdatedUser(Key user);

    public abstract void setUpdatedTime(Date time);

    public void logCreated(Key user, Date time) {
        this.setCreatedUser(user);
        this.setCreatedTime(time);
    }

    public void logUpdated(Key user, Date time) {
        this.setUpdatedUser(user);
        this.setUpdatedTime(time);
    }

}
