/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.util.Date;

import javax.jdo.PersistenceManager;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.annotations.*;

@MugleData
@PersistenceCapable
public class PromotedGameData extends ModelDataClass<Long, PromotedGame, PromotedGameData> {


    // Fields

    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private String urlName;

    // Promoted Game shouldn't exist without a game version
    @Persistent
    private Key gameVersion;

    @Persistent
    private Key createdUser;

    @Persistent
    private Date createdTime;

    @Persistent
    private Key updatedUser;

    @Persistent
    private Date updatedTime;


    // Constructors

    public PromotedGameData() {
        this.urlName = null;
        this.gameVersion = null;
    }

    // Getters

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setPrimaryKey")
    public Long getPrimaryKey() {
        if (this.id == null) {
            return null;
        }
        return this.id.getId();
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUrlName")
    public String getUrlName() {
        return this.urlName;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setGameVersionKey", overrideBy="keyToGameVersion")
    public Key getGameVersion() {
        return this.gameVersion;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedUser", overrideBy="keyToCreated")
    public Key getCreatedUser() {
        return this.createdUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedTime")
    public Date getCreatedTime() {
        return this.createdTime;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedUser", overrideBy="keyToUpdated")
    public Key getUpdatedUser() {
        return this.updatedUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedTime")
    public Date getUpdatedTime() {
        return this.updatedTime;
    }


    // Setters

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedUser(Key createdUser) {
        this.createdUser = createdUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedUser(Key updatedUser) {
        this.updatedUser = updatedUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    /* Can be updated only by admin */
    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.NONE, mappedBy="getGameVersionKey", overrideBy="gameVersionFromKey")
    public void setGameVersion(GameVersionData version) {
        if (version == null) {
            this.gameVersion = null;
            return;
        }
        this.gameVersion = version.getServerKey();
        if (version.getPromotedGame() == null) {
            version.setPromotedGame(this);
        }
    }

    @GetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedTo="getUrlName")
    public void setUrlName(String urlName) {
        this.urlName = urlName;
    }

    @Override
    public Key getServerKey() {
        return this.id;
    }

    @Override
    public void setServerKey(Key key) {
        this.id = key;
    }

    @Override
    public void ClearFields() {
        this.setGameVersion(null);
        this.setUrlName(null);
        this.setServerKey(null);
    }
    @Override
    public Class<PromotedGame> getClientClassType() {
        return PromotedGame.class;
    }

    public Long keyToGameVersion() {
        if (this.gameVersion == null) {
            return null;
        }
        return this.gameVersion.getId();
    }

    public GameVersionData gameVersionFromKey(Long key) {
        if (key == null) { return null; }
        PersistenceManager pm = PMF.getManager();
        GameVersionData result = null;
        try {
            result = pm.getObjectById(GameVersionData.class, key);
        } finally {
            pm.close();
        }
        return result;
    }

    public Long keyToCreated() {
        if (this.createdUser == null) { return null; }
        return this.createdUser.getId();
    }

    public Long keyToUpdated() {
        if (this.updatedUser == null) { return null; }
        return this.updatedUser.getId();
    }

}
