/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.util.List;

import javax.jdo.JDOObjectNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.*;

public class PromotedGameGetter {
    
    // TODO need to recheck this class
    
    /**
     * Gets the PromotedGame by its Primary Key - READ ONLY
     * @param primaryKey
     * @return The PromotedGame - READ ONLY
     * @throws PromotedGameNotExists
     */
    public static PromotedGameData getPromotedGame(Long primaryKey) 
        throws PromotedGameNotExists {
        PersistenceManager pm = PMF.getManager();
        try {
            return getPromotedGame(pm, primaryKey);
        } finally {
            pm.close();
        }
    }
    
    /**
     * Gets the PromotedGame by its Primary Key, for editing the object in the datastore
     * PersistenceManager must be handled by the caller
     * @param pm The PersistenceManager
     * @param primaryKey
     * @return The PromotedGame
     * @throws PromotedGameNotExists
     */
    public static PromotedGameData getPromotedGame(PersistenceManager pm, Long primaryKey) 
        throws PromotedGameNotExists {
        try {
            PromotedGameData ugp = pm.getObjectById(PromotedGameData.class, primaryKey);
            return ugp;
        } catch (JDOObjectNotFoundException e) {
            throw new PromotedGameNotExists(primaryKey);
        }
    }

    /**
     * Gets the PromotedGame by its Primary Key - READ ONLY
     * @param primaryKey
     * @return The PromotedGame - READ ONLY
     * @throws PromotedGameNotExists
     */
    public static PromotedGameData getPromotedGame(Key primaryKey)
        throws PromotedGameNotExists {
        PersistenceManager pm = PMF.getManager();
        try {
            return getPromotedGame(pm, primaryKey);
        } finally {
            pm.close();
        }
    }

    /**
     * Gets the PromotedGame associated with the given PromotedGameData primary key.
     * @param pm The PersistenceManager
     * @param key The server-side key (ModelDataClass.getServerKey())
     * @return The UserData object directly off the datastore.
     * @throws PromotedGameNotExists 
     */
    public static PromotedGameData getPromotedGame(PersistenceManager pm, Key key)
            throws PromotedGameNotExists {
        try {
            PromotedGameData ugp = pm.getObjectById(PromotedGameData.class, key);
            return ugp;
        } catch (JDOObjectNotFoundException e) {
            throw new PromotedGameNotExists(key.getId());
        }
    }
    
    /**
     * Gets the PromotedGame associated with the given PromotedGameData urlName.
     * @param pm The PersistenceManager
     * @param name the Name of the PromotedGame
     * @return The UserData object directly off the datastore.
     * @throws PromotedGameNotExists 
     */
    public static PromotedGameData getPromotedGame(String name)
            throws PromotedGameNotExists {
       
        PersistenceManager pm = PMF.getManager();
        
        try {
            return getPromotedGame(pm, name);
        } finally {
            pm.close();
        }
   
    }

    /**
     * Gets the PromotedGame associated with the given PromotedGameData urlName.
     * @param pm The PersistenceManager
     * @param name The name of the PromotedGame
     * @return The UserData object directly off the datastore.
     * @throws PromotedGameNotExists 
     */
    @SuppressWarnings("unchecked")
    public static PromotedGameData getPromotedGame(PersistenceManager pm, String name)
            throws PromotedGameNotExists {
       Query q = pm.newQuery(PromotedGameData.class, "urlName == n");
       q.declareParameters("String n");
       List<PromotedGameData> results = (List<PromotedGameData>) q.execute(name);
       if (results.isEmpty()) {
           throw new PromotedGameNotExists();
       }
       return results.get(0);
    }
}
