/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model;

import java.util.Date;

import javax.jdo.PersistenceManager;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.annotations.*;

@MugleData
@PersistenceCapable
public class UserAchievementData extends ModelDataClass<Long, UserAchievement, UserAchievementData> {


    // Fields

    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private Key ugp;

    @Persistent
    private Key achievement;

    @Persistent
    private Boolean achieved;

    @Persistent
    private Integer progress;

    @Persistent
    private Key createdUser;

    @Persistent
    private Date createdTime;

    @Persistent
    private Key updatedUser;

    @Persistent
    private Date updatedTime;


    //By default assume a lack of this object means no progress towards achievement

    // Constructors

    public UserAchievementData() {
        this.ugp = null;
        this.achievement = null;
        this.achieved = false;
        this.progress = 0;
    }

    // Getters

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setPrimaryKey")
    public Long getPrimaryKey() {
        if (this.id == null) {
            return null;
        }
        return this.id.getId();
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUserGameProfileKey", overrideBy="keyToUGP")
    public Key getUserGameProfile() {
        return this.ugp;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setAchievementKey", overrideBy="keyToAchievement")
    public Key getAchievement() {
        return this.achievement;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setAchieved")
    public Boolean getAchieved(){
        return this.achieved;
    }

    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setProgress")
    public Integer getProgress(){
        return this.progress;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedUser", overrideBy="keyToCreated")
    public Key getCreatedUser() {
        return this.createdUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setCreatedTime")
    public Date getCreatedTime() {
        return this.createdTime;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedUser", overrideBy="keyToUpdated")
    public Key getUpdatedUser() {
        return this.updatedUser;
    }

    @Override
    @GetterUserLevel(privateView=Role.GUEST, publicView=Role.GUEST, mappedTo="setUpdatedTime")
    public Date getUpdatedTime() {
        return this.updatedTime;
    }


    // Setters

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedUser(Key createdUser) {
        this.createdUser = createdUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedUser(Key updatedUser) {
        this.updatedUser = updatedUser;
    }

    @Override
    @SetterUserLevel(privateView=Role.NONE, publicView=Role.NONE)
    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedBy="setAchieved")
    public void setAchieved(Boolean achieved) {
        this.achieved = achieved;
    }

    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedBy="setAchievementKey", overrideBy="achievementFromKey")
    public void setAchievement(AchievementData achievement) {
        if (achievement == null) {
            this.achievement = null;
            return;
        }
        this.achievement = achievement.getServerKey();
        achievement.getUserAchievements().add(this.id);
    }

    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedBy="setProgress")
    public void setProgress(Integer progress) {
        this.progress = progress;
    }

    @SetterUserLevel(privateView=Role.ADMIN, publicView=Role.ADMIN, mappedBy="setUserGameProfileKey", overrideBy="ugpFromKey")
    public void setUserGameProfile(UserGameProfileData ugp) {
        if (ugp == null) {
            this.ugp = null;
            return;
        }
        this.ugp = ugp.getServerKey();
        ugp.getUserAchievements().add(this.id);
    }

    // Misc methods

    @Override
    public Key getServerKey() {
        return this.id;
    }

    @Override
    public void setServerKey(Key key) {
        this.id = key;
    }

    @Override
    public void ClearFields() {
        this.setAchieved(null);
        this.setAchievement(null);
        this.setProgress(null);
        this.setUserGameProfile(null);
        this.setServerKey(null);
    }

    @Override
    public Class<UserAchievement> getClientClassType() {
        return UserAchievement.class;
    }

    public Long keyToUGP() {
        if (this.ugp == null) {
            return null;
        }
        return this.ugp.getId();
    }

    public Long keyToAchievement() {
        if (this.achievement == null) {
            return null;
        }
        return this.achievement.getId();
    }

    public UserGameProfileData ugpFromKey(Long key) {
        if (key == null) { return null; }
        PersistenceManager pm = PMF.getManager();
        UserGameProfileData result = null;
        try {
            result = pm.getObjectById(UserGameProfileData.class, key);
        } finally {
            pm.close();
        }
        return result;
    }

    public AchievementData achievementFromKey(Long key) {
        if (key == null) { return null; }
        PersistenceManager pm = PMF.getManager();
        AchievementData result = null;
        try {
            result = pm.getObjectById(AchievementData.class, key);
        } finally {
            pm.close();
        }
        return result;
    }

    public Long keyToCreated() {
        if (this.createdUser == null) { return null; }
        return this.createdUser.getId();
    }

    public Long keyToUpdated() {
        if (this.updatedUser == null) { return null; }
        return this.updatedUser.getId();
    }

}
