/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import au.edu.unimelb.csse.mugle.shared.model.Role;

/**
 * Defines the user level for reading fields by client code, for model classes.
 */
@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
public @interface GetterUserLevel {

    /**
     * The minimum user level for a getter method, when the client request is
     * during a PRIVATE view. NOTE: Role.NONE refers to the method be totally
     * NOT accessible.
     * 
     * @return The minimum role required.
     */
    Role privateView() default Role.GUEST;

    /**
     * The minimum user level for a getter method, when the client request is
     * during a PUBLIC view. NOTE: Role.NONE refers to the method be totally NOT
     * accessible.
     * 
     * @return The minimum role required.
     */
    Role publicView() default Role.GUEST;

    /**
     * The setter method in the corresponding wrapper class that the returned
     * value will be stored in. NOTE: The mapped method MUST have a single
     * parameter, that of the same type as the annotated method's return type.
     * 
     * @return The method name that is mapped to (usually a setter).
     */
    String mappedTo() default "";

    /**
     * A method that can be used to override the annotated method. The return
     * value of the overridden method will be used for the mapped setter method
     * in the wrapper class.
     * 
     * @return The method name used to override. Leave as empty string to
     *         ignore.
     */
    String overrideBy() default "";

    /**
     * Status as to whether the annotated method return a Collection<>, by
     * default set to false.
     * 
     * @return True if the method returns a Collection<>.
     */
    boolean list() default false;

}
