/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import au.edu.unimelb.csse.mugle.shared.model.Role;

@Target(ElementType.METHOD)
@Retention(RetentionPolicy.RUNTIME)
public @interface SetterUserLevel {

    /**
     * The minimum user level for a setter method, when the client request is
     * during a PRIVATE view. NOTE: Role.NONE refers to the method be totally
     * NOT accessible.
     * 
     * @return The minimum role required.
     */
    Role privateView() default Role.GUEST;

    /**
     * The minimum user level for a setter method, when the client request is
     * during a PUBLIC view. NOTE: Role.NONE refers to the method be totally NOT
     * accessible.
     * 
     * @return The minimum role required.
     */
    Role publicView() default Role.GUEST;

    /**
     * The getter method in the corresponding wrapper class that the setting
     * value is obtained from. NOTE: The mapped method MUST have a return value,
     * that of the same type as the annotated method's parameter type.
     * 
     * @return The method name that is mapped to (usually a getter).
     */
    String mappedBy() default "";

    /**
     * A method that can be used to override the annotated method. The
     * overridden method will be used for as the setter instead.
     * 
     * @return The method name used to override. Leave as empty string to
     *         ignore.
     */
    String overrideBy() default "";

}
