/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.model.services;

import javax.jdo.PersistenceManager;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.*;
import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.DataObjectNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.ServerException;

/**
 * A static class that contains most commonly used methods (methods used by
 * services).
 */
public class DataServiceHelper {

    // treating as static class
    private DataServiceHelper() { }
    
    public static <S extends ModelClientClass<?,S>, T extends ModelDataClass<?,S,T>>
            S write(S object, Class<T> type, Role role, ClientView cview)
                throws ServerException, DataObjectNotExists {
                
        // gets the server-side objects with its properties updated
        // from the client-side object. However, this only happens
        // using permissions.
        T data = ModelWrapper.getDataCloneSafe(object, type, role, cview);
        
        PersistenceManager pm = PMF.getManager();
        
        // update the database
        try {
            data = pm.makePersistent(data);
        } finally {
            pm.close();
        }       
        
        // Convert the updated object back to
        return ModelWrapper.getClientCloneSafe(data, role, cview);
        
    }
    
    public static <S extends ModelClientClass<?,S>, T extends ModelDataClass<?,S,T>>
           S write(S object, Class<T> type, RoleAndView rv)
               throws ServerException, DataObjectNotExists {
               
       return write(object, type, rv.getRole(), rv.getClientView());
       
    }
    
    public static <S extends ModelClientClass<?,S>, T extends ModelDataClass<?,S,T>>
       void remove(S object, Class<T> type, Role role, ClientView cview)
           throws ServerException, DataObjectNotExists {
                   
       PersistenceManager pm = PMF.getManager();
       
       // remove from the database
       try {
           T data = pm.getObjectById(type, object.getPrimaryKey());
           if (data == null) { throw new DataObjectNotExists(); }
           pm.deletePersistent(data);
       } finally {
           pm.close();
       }       
              
    }
        
    public static <S extends ModelClientClass<?,S>, T extends ModelDataClass<?,S,T>>
          void remove(S object, Class<T> type, RoleAndView rv)
              throws ServerException, DataObjectNotExists {
              
        remove(object, type, rv.getRole(), rv.getClientView());
      
    }
    // TODO add more methods to do modifications/additions of objects
    
}
