/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.api;

import java.io.Serializable;

import javax.jdo.PersistenceManager;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import au.edu.unimelb.csse.mugle.client.multiplayer.api.GameKeyValueService;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.UserData;
import au.edu.unimelb.csse.mugle.server.model.UserGetter;
import au.edu.unimelb.csse.mugle.server.multiplayer.model.*;
import au.edu.unimelb.csse.mugle.shared.api.KeyError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.PlayerNotFoundInGame;

/**
 * The server side implementation of GameKeyValueService
 */
@SuppressWarnings("serial")
public class GameKeyValueServiceImpl extends RemoteServiceServlet implements
        GameKeyValueService {

    @Override
    public boolean putValue(String instanceToken, String key, Serializable value)
            throws InstanceTokenError, PlayerNotFoundInGame {
        PersistenceManager pm = PMF.getManager();
        GameKeyValuePairData kv;
        try {
            UserData player = UserGetter.getCurrentUser();
            GameInstanceData instance = GameInstanceGetter
                    .getInstanceByToken(instanceToken);
            GameInstanceGetter.findPlayerInGame(instance, player);
            kv = GameKeyValuePairGetter.getGameKeyValuePair(pm, key,
                    instanceToken);
            return kv.setValue(value);
        } catch (KeyError e) {
            kv = GameKeyValuePairGetter.getNewGameKeyValuePair(pm,
                    instanceToken);
            kv.setKey(key);
            return kv.setValue(value);
        } finally {
            pm.close();
        }
    }

    @Override
    public Serializable getValue(String instanceToken, String key)
            throws InstanceTokenError, KeyError, PlayerNotFoundInGame {
        UserData player = UserGetter.getCurrentUser();
        GameInstanceData instance = GameInstanceGetter
                .getInstanceByToken(instanceToken);
        GameInstanceGetter.findPlayerInGame(instance, player);
        GameKeyValuePairData kv = GameKeyValuePairGetter.getGameKeyValuePair(
                key, instanceToken);
        return kv.getValue();
    }

}
