/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.api;

import java.util.Date;
import java.util.List;

import javax.jdo.PersistenceManager;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

import au.edu.unimelb.csse.mugle.client.multiplayer.api.LoggingService;
import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.UserData;
import au.edu.unimelb.csse.mugle.server.model.UserGetter;
import au.edu.unimelb.csse.mugle.server.multiplayer.model.*;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameState;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.PlayerNotFoundInGame;

/**
 * The server side implementation of LoggingService
 */
@SuppressWarnings("serial")
public class LoggingServiceImpl extends RemoteServiceServlet implements
        LoggingService {

    @Override
    public void log(String instanceToken, GameState state,
            boolean hasLogMessages) throws InstanceTokenError,
            PlayerNotFoundInGame {
        PersistenceManager pm = PMF.getManager();
        try {
            UserData player = UserGetter.getCurrentUser(pm);
            GameInstanceData instance = GameInstanceGetter.getInstanceByToken(
                    pm, instanceToken);
            GameInstanceGetter.findPlayerInGame(instance, player);
            GameLogData gameLog = new GameLogData(state);
            gameLog.setCreatedTime(new Date());
            gameLog.setHasLogMessages(hasLogMessages);
            pm.makePersistent(gameLog);
            instance.addGameLog(gameLog.getServerKey());
            List<PlayerMessageData> messageData = PlayerMessageGetter
                    .getAllMessageData(pm, instance);
            for (PlayerMessageData playerMessage : messageData) {
                if (hasLogMessages)
                    playerMessage.addPendingLogMessage(gameLog);
                playerMessage.clearPendingLogMessage();
                playerMessage.setPendingIndex();
            }
        } finally {
            pm.close();
        }

    }

}
