/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.io.IOException;
import java.util.List;

import javax.jdo.PersistenceManager;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.UserData;
import au.edu.unimelb.csse.mugle.server.model.UserGetter;
import au.edu.unimelb.csse.mugle.server.multiplayer.api.ChannelServiceImpl;
import au.edu.unimelb.csse.mugle.server.multiplayer.api.MultiPlayerGameServiceImpl;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.PlayerNotFoundInGame;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.ServerCommand;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.UserNotExists;

import com.google.appengine.api.channel.ChannelMessage;
import com.google.appengine.api.channel.ChannelPresence;
import com.google.appengine.api.channel.ChannelServiceFactory;

/**
 * Handler for channel presence when a client connects to or disconnects from a
 * channel
 */
public class ChannelPresenceServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp)
            throws ServletException, IOException {
        if (req.getRequestURI().endsWith("/connected/")
                || req.getRequestURI().endsWith("/disconnected/")) {
            ChannelPresence parsePresence = ChannelServiceFactory
                    .getChannelService().parsePresence(req);
            MessageData message = new MessageData();
            if (parsePresence.clientId() == null)
                return;
            PersistenceManager pm = PMF.getManager();
            try {
                UserData player = UserGetter.getUser(pm,
                        Long.valueOf(parsePresence.clientId()));
                String playerName = player.getUrlName();
                message.setSender(playerName);

                if (parsePresence.isConnected()) {
                    message.setServerCommand(ServerCommand.PLAYER_CONNECTED);
                    for (String id : ChannelServiceImpl.channelIds) {
                        // TODO This is just for testing, needs to be removed
                        // later.
                        ChannelServiceFactory.getChannelService().sendMessage(
                                new ChannelMessage(id, message.toString()));
                    }
                } else {
                    message.setServerCommand(ServerCommand.PLAYER_DISCONNECTED);

                    // First to find out whether this player is in pending
                    // status
                    List<PendingPlayerData> pendings = PendingPlayerGetter
                            .getPendingsByPlayer(pm, player);
                    if (!pendings.isEmpty()) {
                        for (PendingPlayerData pending : pendings) {
                            pending.removePlayer(player.getServerKey());
                        }
                    } else {
                        MultiPlayerGameServiceImpl gameService = new MultiPlayerGameServiceImpl();
                        PendingPlayerData pending;
                        List<GameInstanceData> instances = GameInstanceGetter
                                .getActiveInstanceByPlayer(pm, player);
                        if (!instances.isEmpty()) {
                            for (GameInstanceData instance : instances) {
                                pending = PendingPlayerGetter
                                        .getPendingByGameKey(pm,
                                                instance.getGame());
                                gameService.disconnectGame(pm, pending,
                                        instance, player);
                            }
                        }
                    }
                }
            } catch (UserNotExists e) {
                e.printStackTrace();
            } catch (PlayerNotFoundInGame e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } finally {
                pm.close();
            }
        }
    }
}
