/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.util.List;

import javax.jdo.JDOObjectNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.UserData;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameInstanceNotExists;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.PlayerNotFoundInGame;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;

public class GameInstanceGetter {
    public static GameInstanceData getInstanceByToken(String instanceToken)
            throws InstanceTokenError {
        PersistenceManager pm = PMF.getManager();
        try {
            return getInstanceByToken(pm, instanceToken);
        } finally {
            pm.close();
        }
    }

    public static GameInstanceData getInstanceByToken(PersistenceManager pm,
            String instanceToken) throws InstanceTokenError {
        Key key = KeyFactory.stringToKey(instanceToken);
        GameInstanceData gameInstance;
        try {
            gameInstance = pm.getObjectById(GameInstanceData.class, key);
        } catch (JDOObjectNotFoundException e) {
            throw new InstanceTokenError(instanceToken);
        }

        return gameInstance;
    }

    public static GameInstanceData getInstance(Long primaryKey)
            throws GameInstanceNotExists {
        PersistenceManager pm = PMF.getManager();
        try {
            return getInstance(pm, primaryKey);
        } finally {
            pm.close();
        }
    }

    public static GameInstanceData getInstance(PersistenceManager pm,
            Long primaryKey) throws GameInstanceNotExists {
        GameInstanceData gameInstance;
        try {
            gameInstance = pm.getObjectById(GameInstanceData.class, primaryKey);
        } catch (JDOObjectNotFoundException e) {
            throw new GameInstanceNotExists(primaryKey);
        }

        return gameInstance;
    }

    public static List<GameInstanceData> getActiveInstanceByPlayer(
            UserData player) {
        PersistenceManager pm = PMF.getManager();
        try {
            return getActiveInstanceByPlayer(pm, player);
        } finally {
            pm.close();
        }
    }

    @SuppressWarnings("unchecked")
    public static List<GameInstanceData> getActiveInstanceByPlayer(
            PersistenceManager pm, UserData player) {
        Query q = pm.newQuery(GameInstanceData.class,
                "active == true && activePlayers == p");
        q.declareParameters(Key.class.getName() + " p");
        return (List<GameInstanceData>) q.execute(player.getServerKey());
    }

    public static List<GameInstanceData> getActiveInstanceByGameKey(Key gameKey) {
        PersistenceManager pm = PMF.getManager();
        try {
            return getActiveInstancesByGameKey(pm, gameKey);
        } finally {
            pm.close();
        }
    }

    @SuppressWarnings("unchecked")
    public static List<GameInstanceData> getActiveInstancesByGameKey(
            PersistenceManager pm, Key gameKey) {
        Query q = pm.newQuery(GameInstanceData.class,
                "active == true && game == g");
        q.declareParameters(Key.class.getName() + " g");
        return (List<GameInstanceData>) q.execute(gameKey);
    }

    public static void findPlayerInGame(GameInstanceData instance,
            UserData player) throws PlayerNotFoundInGame {
        if (!instance.containPlayer(player.getServerKey()))
            throw new PlayerNotFoundInGame(player.getUrlName());
    }
}
