/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.util.List;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.shared.api.KeyError;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;

public class GameKeyValuePairGetter {

    public static GameKeyValuePairData getGameKeyValuePair(String key,
            String instanceToken) throws KeyError {
        PersistenceManager pm = PMF.getManager();
        try {
            return getGameKeyValuePair(pm, key, instanceToken);
        } finally {
            pm.close();
        }
    }

    @SuppressWarnings("unchecked")
    public static GameKeyValuePairData getGameKeyValuePair(
            PersistenceManager pm, String key, String instanceToken)
            throws KeyError {
        Query q = pm.newQuery(GameKeyValuePairData.class,
                "gameInstance == gi && key == k");
        q.declareParameters(Key.class.getName() + " gi, " + "String k");
        Key gi = KeyFactory.stringToKey(instanceToken);
        List<GameKeyValuePairData> results = (List<GameKeyValuePairData>) q
                .execute(gi, key);
        if (!results.isEmpty())
            return results.get(0);
        else
            throw new KeyError(key);
    }

    public static GameKeyValuePairData getNewGameKeyValuePair(
            PersistenceManager pm, String instanceToken) {
        GameKeyValuePairData kv = new GameKeyValuePairData();
        pm.makePersistent(kv);
        Key gi = KeyFactory.stringToKey(instanceToken);
        kv.setGameInstance(gi);
        return kv;
    }
}
