/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;

import au.edu.unimelb.csse.mugle.server.model.ModelDataClass;
import au.edu.unimelb.csse.mugle.server.model.annotations.GetterUserLevel;
import au.edu.unimelb.csse.mugle.server.model.annotations.MugleData;
import au.edu.unimelb.csse.mugle.shared.model.Role;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameState;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.GameLog;

@MugleData
@PersistenceCapable
public class GameLogData extends ModelDataClass<Long, GameLog, GameLogData> {
    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent(serialized = "true", defaultFetchGroup = "true")
    private Map<String, String> keyStringPairs = new HashMap<String, String>();

    @Persistent(serialized = "true", defaultFetchGroup = "true")
    private Map<String, List<String>> keyListPairs = new HashMap<String, List<String>>();

    @Persistent
    private boolean hasLogMessages;

    @Persistent
    private Date createdTime;

    public GameLogData(GameState gameState) {
        this.keyStringPairs = gameState.getAttributeValuePairs();
        this.keyListPairs = gameState.getAttributeListPairs();
    }

    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setKeyStringPairs")
    public Map<String, String> getKeyStringPairs() {
        return keyStringPairs;
    }

    public void setKeyStringPairs(Map<String, String> keyStringPairs) {
        this.keyStringPairs = keyStringPairs;
    }

    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setKeyListPairs")
    public Map<String, List<String>> getKeyListPairs() {
        return keyListPairs;
    }

    public void setKeyListPairs(Map<String, List<String>> keyListPairs) {
        this.keyListPairs = keyListPairs;
    }

    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setHasLogMessages")
    public boolean hasLogMessages() {
        return hasLogMessages;
    }

    public void setHasLogMessages(boolean hasLogMessages) {
        this.hasLogMessages = hasLogMessages;
    }

    @Override
    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setToken", overrideBy = "keyToToken")
    public Key getServerKey() {
        return this.id;
    }

    @Override
    public void setServerKey(Key key) {
        this.id = key;
    }

    @Override
    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setPrimaryKey")
    public Long getPrimaryKey() {
        if (this.id == null) {
            return null;
        }
        return this.id.getId();
    }

    @Override
    public Class<GameLog> getClientClassType() {
        return GameLog.class;
    }

    @Override
    protected void ClearFields() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Key getCreatedUser() {
        throw new UnsupportedOperationException();
    }

    @Override
    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setCreatedTime")
    public Date getCreatedTime() {
        return this.createdTime;
    }

    @Override
    public Key getUpdatedUser() {
        throw new UnsupportedOperationException();
    }

    @Override
    public Date getUpdatedTime() {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setCreatedUser(Key user) {
        throw new UnsupportedOperationException();

    }

    @Override
    public void setCreatedTime(Date time) {
        this.createdTime = time;
    }

    @Override
    public void setUpdatedUser(Key user) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setUpdatedTime(Date time) {
        throw new UnsupportedOperationException();
    }
    
    public String keyToToken() {
        return KeyFactory.keyToString(this.id);
    }

}
