/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import au.edu.unimelb.csse.mugle.server.model.ModelDataClass;
import au.edu.unimelb.csse.mugle.server.model.annotations.GetterUserLevel;
import au.edu.unimelb.csse.mugle.shared.model.Role;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.MessageBean;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.MessageBeanFactory;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.ServerCommand;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.LogMessage;

import com.google.appengine.api.datastore.Key;
import com.google.web.bindery.autobean.shared.AutoBean;
import com.google.web.bindery.autobean.shared.AutoBeanCodex;
import com.google.web.bindery.autobean.shared.AutoBeanUtils;
import com.google.web.bindery.autobean.vm.AutoBeanFactorySource;

public class MessageData extends ModelDataClass<Long, LogMessage, MessageData> {
    private MessageBean messageBean;

    private static MessageBeanFactory factory = AutoBeanFactorySource
            .create(MessageBeanFactory.class);

    public MessageData() {
        messageBean = makeMessageBean();
    }

    public MessageData(String message) {
        messageBean = deserialiseFromJson(message);
    }

    public MessageData(MessageBean message) {
        messageBean = message;
    }

    public MessageBean messageBean() {
        return messageBean;
    }

    public void setMessageBean(MessageBean messageBean) {
        this.messageBean = messageBean;
    }

    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setBody")
    public String body() {
        return messageBean.getBody();
    }

    public String command() {
        return messageBean.getCommand();
    }

    public String getAttribute(String name) {
        return messageBean.getAttributeValuePairs().get(name);
    }

    public Map<String, String> getAttributeValuePairs() {
        return messageBean.getAttributeValuePairs();
    }

    public List<String> getList(String key) {
        return messageBean.getAttributeListPairs().get(key);
    }

    public Map<String, List<String>> getKeyListPairs() {
        return messageBean.getAttributeListPairs();
    }

    public ServerCommand serverCommand() {
        return messageBean.getServerCommand();
    }

    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setSender")
    public String sender() {
        return messageBean.getSender();
    }

    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setReceivers")
    public Set<String> receivers() {
        return messageBean.getReceivers();
    }

    public String instanceToken() {
        return messageBean.getInstanceToken();
    }

    @Override
    @GetterUserLevel(privateView = Role.GUEST, publicView = Role.GUEST, mappedTo = "setCreatedTime")
    public Date getCreatedTime() {
        return messageBean.getTimestamp();
    }

    public void setBody(String body) {
        messageBean.setBody(body);
    }

    public void setCommand(String command) {
        messageBean.setCommand(command);
    }

    public void setAttributeValuePairs(Map<String, String> parameters) {
        messageBean.setAttributeValuePairs(parameters);
    }

    public void addAttribute(String name, String value) {
        if (messageBean.getAttributeValuePairs() == null)
            messageBean.setAttributeValuePairs(new HashMap<String, String>());
        messageBean.getAttributeValuePairs().put(name, value);
    }

    public void setKeyListPairs(Map<String, List<String>> keyListPairs) {
        messageBean.setAttributeListPairs(keyListPairs);
    }

    public void addKeyList(String key, List<String> list) {
        if (messageBean.getAttributeListPairs() == null)
            messageBean.setAttributeListPairs(new HashMap<String, List<String>>());
        messageBean.getAttributeListPairs().put(key, list);
    }

    public void setServerCommand(ServerCommand command) {
        messageBean.setServerCommand(command);
    }

    public void setSender(String sender) {
        messageBean.setSender(sender);
    }

    public void setReceivers(Set<String> receivers) {
        messageBean.setReceivers(receivers);
    }

    public void addReceiver(String receiver) {
        if (this.receivers() == null)
            this.setReceivers(new HashSet<String>());
        messageBean.getReceivers().add(receiver);
    }

    public void setInstanceToken(String gameToken) {
        messageBean.setInstanceToken(gameToken);
    }

    @Override
    public void setCreatedTime(Date timestamp) {
        messageBean.setTimestamp(timestamp);
    }

    // reset all the message fields
    public void clear() {
        this.setBody(null);
        this.setCommand(null);
        this.setAttributeValuePairs(null);
        this.setReceivers(null);
        this.setCreatedTime(null);
    }

    public MessageBean makeMessageBean() {
        // Construct the AutoBean
        AutoBean<MessageBean> message = factory.message();

        return message.as();
    }

    // Serialise the message to JSON
    @Override
    public String toString() {
        AutoBean<MessageBean> bean = AutoBeanUtils
                .getAutoBean(this.messageBean);

        return AutoBeanCodex.encode(bean).getPayload();
    }

    public static MessageBean deserialiseFromJson(String json) {
        AutoBean<MessageBean> bean = AutoBeanCodex.decode(factory,
                MessageBean.class, json);
        return bean.as();
    }

    public int counter() {
        return this.messageBean.getCounter();
    }

    public static int getCounter(String message) {
        return AutoBeanCodex.decode(factory, MessageBean.class, message).as()
                .getCounter();
    }

    public void setCounter(int counter) {
        this.messageBean.setCounter(counter);
    }

    @Override
    public Key getServerKey() {
        return null;
    }

    @Override
    public void setServerKey(Key key) {
    }

    @Override
    public Long getPrimaryKey() {
        return null;
    }

    @Override
    public Class<LogMessage> getClientClassType() {
        return LogMessage.class;
    }

    @Override
    protected void ClearFields() {
    }

    @Override
    public Key getCreatedUser() {
        return null;
    }

    @Override
    public Key getUpdatedUser() {
        return null;
    }

    @Override
    public Date getUpdatedTime() {
        return null;
    }

    @Override
    public void setCreatedUser(Key user) {
    }

    @Override
    public void setUpdatedUser(Key user) {
    }

    @Override
    public void setUpdatedTime(Date time) {
    }
}
