/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.util.List;

import javax.jdo.JDOObjectNotFoundException;
import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import au.edu.unimelb.csse.mugle.server.PMF;
import au.edu.unimelb.csse.mugle.server.model.UserData;
import au.edu.unimelb.csse.mugle.shared.api.GameTokenError;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.KeyFactory;

public class PendingPlayerGetter {
    public static PendingPlayerData getPendingByGameKey(Key gameKey)
            throws GameTokenError {
        PersistenceManager pm = PMF.getManager();
        try {
            return getPendingByGameKey(pm, gameKey);
        } finally {
            pm.close();
        }
    }

    public static PendingPlayerData getPendingByGameKey(PersistenceManager pm,
            Key gameKey) {
        // To make sure each game has only one pending player data.
        Key key = KeyFactory.createKey(PendingPlayerData.class.getSimpleName(),
                gameKey.getId());
        PendingPlayerData pending;
        try {
            pending = pm.getObjectById(PendingPlayerData.class, key);

        } catch (JDOObjectNotFoundException e) {
            pending = new PendingPlayerData();
            pending.setServerKey(key);
            pending.setGame(gameKey);
//            pm.makePersistent(pending);
            System.out.println("New pending data");
            return pending;
        }

        return pending;
    }

    public static List<PendingPlayerData> getPendingsByPlayer(UserData player) {
        PersistenceManager pm = PMF.getManager();
        try {
            return getPendingsByPlayer(pm, player);
        } finally {
            pm.close();
        }
    }

    @SuppressWarnings("unchecked")
    public static List<PendingPlayerData> getPendingsByPlayer(
            PersistenceManager pm, UserData player) {
        Query q = pm.newQuery(PendingPlayerData.class, "players == p");
        q.declareParameters(Key.class.getName() + " p");
        return (List<PendingPlayerData>) q.execute(player.getServerKey());
    }
}
