/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jdo.annotations.Extension;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;

@PersistenceCapable
public class PlayerMessageData {
    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private Key gameInstance;

    @Persistent
    private Key sender;

    @Persistent
    private int counter;

    @Persistent
    private List<Integer> sequence;

    @Persistent
    private int pendingIndex;

    @Persistent
    private boolean hasLogMessages;

    @Persistent
    @Extension(vendorName = "datanucleus", key = "gae.unindexed", value = "true")
    private List<String> pendingLogmessages;

    @Persistent
    // @Persistent(mappedBy = "playerMessage")
    private Set<LogMessageData> logMessages;

    public PlayerMessageData() {
        this.logMessages = new HashSet<LogMessageData>();
        this.pendingLogmessages = new ArrayList<String>();
        this.sequence = new ArrayList<Integer>();
        this.hasLogMessages = false;
        pendingIndex = 0;
    }

    public Key getServerKey() {
        return id;
    }

    public void setServerKey(Key key) {
        this.id = key;
    }

    public Key getGameInstance() {
        return gameInstance;
    }

    public void setGameInstance(Key gameInstance) {
        this.gameInstance = gameInstance;
    }

    public Key getSender() {
        return sender;
    }

    public void setSender(Key sender) {
        this.sender = sender;
    }

    public List<String> getMessages() {
        ArrayList<String> messages = new ArrayList<String>();
        for (LogMessageData logMessage : logMessages)
            messages.addAll(logMessage.getMessages());
        messages.addAll(this.pendingLogmessages);

        return messages;
    }

    public Set<LogMessageData> getLogMessages() {
        return logMessages;
    }

    public void setPendingIndex() {
        pendingIndex = Math.max(0, sequence.size() - 1);
    }

    public void addPendingLogMessage(GameLogData log) {
        this.hasLogMessages = true;
        LogMessageData logMessage = new LogMessageData();
        logMessage.setGameLog(log.getServerKey());
        logMessage.setMessages(pendingLogmessages);
        this.logMessages.add(logMessage);
    }

    public void clearPendingLogMessage() {
        pendingLogmessages.clear();
        this.sequence = this.sequence.subList(0, pendingIndex);
    }

    public void setLogMessages(Set<LogMessageData> logMessages) {
        this.logMessages = logMessages;
    }

    public void addMessage(String message) {
        this.pendingLogmessages.add(message);
        addCounter();
        addSequence(MessageData.getCounter(message));
    }

    public void addSequence(int counter) {
        this.sequence.add(counter);
    }

    public int getCounter() {
        return counter;
    }

    public void setCounter(int counter) {
        this.counter = counter;
    }

    public boolean hasLogMessages() {
        return hasLogMessages;
    }

    public void setHasLogMessages(boolean hasLogMessages) {
        this.hasLogMessages = hasLogMessages;
    }

    public void addCounter() {
        this.counter += 1;
    }

}
