package au.edu.unimelb.csse.mugle.server.multiplayer.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.HashSet;
import java.util.Set;

import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import au.edu.unimelb.csse.mugle.shared.multiplayer.api.Transaction;

import com.google.appengine.api.datastore.Blob;
import com.google.appengine.api.datastore.Key;

@PersistenceCapable
public class TransactionData {
    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key id;

    @Persistent
    private Set<Key> users;

    @Persistent
    private Set<Key> confirmedUsers;

    @Persistent(serialized = "true")
    private Blob transaction;

    public TransactionData() {
        this.users = new HashSet<Key>();
        this.confirmedUsers = new HashSet<Key>();
    }

    public Key getServerKey() {
        return this.id;
    }

    public void setServerKey(Key id) {
        this.id = id;
    }

    public Set<Key> getUsers() {
        return this.users;
    }

    public void addUser(Key user) {
        users.add(user);
    }

    public Set<Key> getConfirmedUsers() {
        return confirmedUsers;
    }

    public void addConfirmedUsers(Key user) {
        if (users.contains(user))
            this.confirmedUsers.add(user);
    }

    public boolean isAllConfirmed() {
        return this.confirmedUsers.containsAll(users);
    }

    public Transaction getTransaction() {
        ByteArrayInputStream byteStream = new ByteArrayInputStream(
                this.transaction.getBytes());
        try {
            Object obj = (new ObjectInputStream(byteStream)).readObject();
            return (Transaction) obj;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return null;
    }

    public boolean setTransaction(Transaction transaction) {
        ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
        ObjectOutput out;
        try {
            out = new ObjectOutputStream(byteStream);
            out.writeObject(transaction);
            this.transaction = new Blob(byteStream.toByteArray());
            return true;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }
}
