/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class Achievement extends ModelClientClass<Long, Achievement> {

    private static final long serialVersionUID = 3247031914722631109L;

    // Fields
        
    private String name;

    private String displayName;
    
    private String description;

    private Long game;

    private Integer maxProgress;

    //TODO: Image


    // Constructors
    
    public Achievement(String name, String displayName, String description,
            int maxProgress, Long game) {
        this.name = name;
        this.setDisplayName(displayName);
        this.description = description;
        this.maxProgress = maxProgress;
        this.game = game;
        //TODO: Image    
    }    
    
    public Achievement() {
        this.name = null;
        this.setDisplayName(null);
        this.description = null;
        this.maxProgress = null;
        this.game = null;
        //TODO: Image    
    }

    // Getters
    
    public String getName() {
        return this.name;
    }

    public String getDisplayName() {
        return displayName;
    }
    
    public String getDescription() {
        return this.description;
    }

    public Integer getMaxProgress() {
        return this.maxProgress;
    }

    public Long getGameKey() {
        return this.game;
    }
    
    public void getUserAchievements(AsyncCallback<ModelCollection<UserAchievement>> callback) {
        DataProvider.getAchievementService().getUserAchievements(this, callback);
    }
    
    public void addUserAchievement(UserAchievement userAchievement,
            AsyncCallback<UpdatedPair<Achievement, UserAchievement>> callback) {
        DataProvider.getAchievementService().addUserAchievement(this, userAchievement, callback);
    }
    
    public void removeUserAchievement(UserAchievement userAchievement,
            AsyncCallback<UpdatedPair<Achievement, UserAchievement>> callback) {
        DataProvider.getAchievementService().removeUserAchievement(this, userAchievement, callback);
    }
    
    
    // Setters
    
    public void setName(String newName) {
        this.name = newName;
    }

    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }
    
    public void setDescription(String newDescription) {
        this.description = newDescription;
    }

    public void setMaxProgress(Integer newProgress) {
        this.maxProgress = newProgress;
    }

    public void setGameKey(Long game) {
        this.game = game;
    }
    
    // ModelClass - common methods

    @Override
    public  Class<Achievement> getClassType() {
        return Achievement.class;
    }

    public void create(AsyncCallback<Achievement> callback) {
        DataProvider.getAchievementService().create(this, callback);
    }
    
    public void remove(AsyncCallback<Void> callback) {
        DataProvider.getAchievementService().remove(this, callback);
    }
    
    @Override
    public void write(AsyncCallback<Achievement> callback) {
        DataProvider.getAchievementService().write(this, callback);
    }
    
    // Static methods
    
    public static void getByKey(Long key, AsyncCallback<Achievement> callback) {
        DataProvider.getAchievementService().getByKey(key, callback);
    }
    
    public static void getByName(String name, Game game, AsyncCallback<Achievement> callback) {
        DataProvider.getAchievementService().getByName(name, game, callback);
    }
    
    public static void getByName(String name, String gameToken, AsyncCallback<Achievement> callback) {
        DataProvider.getAchievementService().getByName(name, gameToken, callback);
    }
    
}
