/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import au.edu.unimelb.csse.mugle.shared.model.services.*;

import com.google.gwt.core.client.GWT;

/**
 * A class that is treated as a static holder of the DataService.
 */
public class DataProvider {
    
    // treating class as static
    private DataProvider() { }
    
    // singleton objects
    
    private static AchievementServiceAsync achievementService = null;
    private static DevTeamServiceAsync devTeamService = null;
    private static GameServiceAsync gameService = null;
    private static GameFileServiceAsync gameFileService = null;
    private static GameVersionServiceAsync gameVersionService = null;
    private static KeyValuePairServiceAsync keyValuePairService = null;
    private static PromotedGameServiceAsync promotedGameService = null;
    private static UserServiceAsync userService = null;
    private static UserAchievementServiceAsync userAchievementService = null;
    private static UserGameProfileServiceAsync userGameProfileService = null;
    
    /**
     * This method needs to called by the client before the services can be used.
     */
    public static void initialiseDataServices() {
        
        achievementService = GWT.create(AchievementService.class);
        devTeamService = GWT.create(DevTeamService.class);
        gameService = GWT.create(GameService.class);
        gameFileService = GWT.create(GameFileService.class);
        gameVersionService = GWT.create(GameVersionService.class);
        keyValuePairService = GWT.create(KeyValuePairService.class);
        promotedGameService = GWT.create(PromotedGameService.class);
        userService = GWT.create(UserService.class);
        userAchievementService = GWT.create(UserAchievementService.class);
        userGameProfileService = GWT.create(UserGameProfileService.class);
        
    }
    
    // get the service objects
    
    public static AchievementServiceAsync getAchievementService() {
        return achievementService;
    }
    
    public static DevTeamServiceAsync getDevTeamService() {
        return devTeamService;
    }
    
    public static GameServiceAsync getGameService() {
        return gameService;
    }
    
    public static GameFileServiceAsync getGameFileService() {
        return gameFileService;
    }
    
    public static GameVersionServiceAsync getGameVersionService() {
        return gameVersionService;
    }
    
    public static KeyValuePairServiceAsync getKeyValuePairService() {
        return keyValuePairService;
    }
    
    public static PromotedGameServiceAsync getPromotedGameService() {
        return promotedGameService;
    }
    
    public static UserServiceAsync getUserService() {
        return userService;
    }
    
    public static UserAchievementServiceAsync getUserAchievementService() {
        return userAchievementService;
    }
    
    public static UserGameProfileServiceAsync getUserGameProfileService() {
        return userGameProfileService;
    }

}
