/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class DevTeam extends ModelClientClass<Long, DevTeam> {

    private static final long serialVersionUID = 2888414240862340393L;

    //Fields

    private String urlName; //Must be unique

    private String fullName;
    

    // Constructors
    
    public DevTeam() {        
        this.urlName = null;
        this.fullName = null;
    }

    // Getters

    public String getFullName() {
        return this.fullName;
    }

    public String getUrlName() {
        return this.urlName;
    }
    
    public void getUsers(AsyncCallback<ModelCollection<User>> callback) {
        DataProvider.getDevTeamService().getUsers(this, callback);
    }
    
    public void addUser(User user,
            AsyncCallback<UpdatedPair<DevTeam, User>> callback)
    {
        DataProvider.getDevTeamService().addUser(this, user, callback);
    }
    
    public void removeUser(User user,
            AsyncCallback<UpdatedPair<DevTeam, User>> callback)
    {
        DataProvider.getDevTeamService().removeUser(this, user, callback);
    }

    public void getGames(boolean includePrivate,
        AsyncCallback<ModelCollection<Game>> callback) {
        DataProvider.getDevTeamService().getGames(this, includePrivate,
                                                  callback);
    }
    
    public void addGame(Game game,
            AsyncCallback<UpdatedPair<DevTeam, Game>> callback) {
        DataProvider.getDevTeamService().addGame(this, game, callback);
    }
    
    public void removeGame(Game game,
            AsyncCallback<UpdatedPair<DevTeam, Game>> callback) {
        DataProvider.getDevTeamService().removeGame(this, game, callback);
    }

    
    // Setters

    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    public void setUrlName(String urlName) {
        this.urlName = urlName;
    }

    
    // ModelClass - common methods

    @Override
    public  Class<DevTeam> getClassType() {
        return DevTeam.class;
    }
    
    public void create(AsyncCallback<DevTeam> callback) {
        DataProvider.getDevTeamService().create(this, callback);
    }
    
    public void remove(AsyncCallback<Void> callback) {
        DataProvider.getDevTeamService().remove(this, callback);
    }
    
    @Override
    public void write(AsyncCallback<DevTeam> callback) {
        DataProvider.getDevTeamService().write(this, callback);
    }
    
    
    // Static methods
    
    public static void getByKey(Long key, AsyncCallback<DevTeam> callback) {
        DataProvider.getDevTeamService().getByKey(key, callback);
    }
    
    public static void getByName(String name, AsyncCallback<DevTeam> callback) {
        DataProvider.getDevTeamService().getByName(name, callback);
    }
    
    public static void getDevTeams(int startIndex, int count, 
            AsyncCallback<ModelCollection<DevTeam>> callback) {
        DataProvider.getDevTeamService().getDevTeams(startIndex, count, callback);
    }
        
}
