/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class Game extends ModelClientClass<Long, Game> {

    private static final long serialVersionUID = 3286071539553738327L;

    /** The maximum possible game rating. */
    public static final int MAX_RATING = 100;

    // Fields

    private String urlName; //Must be unique

    private Integer userRating;

    private Integer totalRatings; // number of users who have rated this game

    private Long devTeam;

    private String gameToken;

    private Boolean isPublic;

    /** Only set when the Game was created by GameService.getGames. */
    private GameVersion activeVersion;


    // Constructors

    public Game() {
        this.gameToken = null;
        this.urlName = null;
        this.isPublic = false;
        this.userRating = null;
        this.totalRatings = null;
        this.devTeam = null;
        this.activeVersion = null;
    }

    // Getters

    public String getUrlName() {
        return this.urlName;
    }

    public Boolean getIsPublic() {
        return this.isPublic;
    }

    public Integer getUserRating() {
        return this.userRating;
    }

    public Integer getTotalRatings() {
        return this.totalRatings;
    }

    public Long getDevTeamKey() {
        return this.devTeam;
    }

    public String getGameToken() {
        return this.gameToken;
    }

    public GameVersion getActiveVersion() {
        return this.activeVersion;
    }

    public void getAchievements(AsyncCallback<ModelCollection<Achievement>> callback) {
        DataProvider.getGameService().getAchievements(this, callback);
    }

    public void addAchievement(Achievement achievement,
            AsyncCallback<UpdatedPair<Game, Achievement>> callback) {
        DataProvider.getGameService().addAchievement(this, achievement, callback);
    }

    public void removeAchievement(Achievement achievement,
            AsyncCallback<UpdatedPair<Game, Achievement>> callback) {
        DataProvider.getGameService().removeAchievement(this, achievement, callback);
    }

    public void getGameVersions(AsyncCallback<ModelCollection<GameVersion>> callback) {
        DataProvider.getGameService().getGameVersions(this, callback);
    }

    public void addGameVersion(GameVersion version,
            AsyncCallback<UpdatedPair<Game, GameVersion>> callback) {
        DataProvider.getGameService().addGameVersion(this, version, callback);
    }

    public void removeGameVersion(GameVersion version,
            AsyncCallback<UpdatedPair<Game, GameVersion>> callback) {
        DataProvider.getGameService().removeGameVersion(this, version, callback);
    }

    public void getUserGameProfiles(AsyncCallback<ModelCollection<UserGameProfile>> callback) {
        DataProvider.getGameService().getUserGameProfiles(this, callback);
    }

    public void addUserGameProfile(UserGameProfile ugp,
            AsyncCallback<UpdatedPair<Game, UserGameProfile>> callback) {
        DataProvider.getGameService().addUserGameProfile(this, ugp, callback);
    }

    public void removeUserGameProfile(UserGameProfile ugp,
            AsyncCallback<UpdatedPair<Game, UserGameProfile>> callback) {
        DataProvider.getGameService().removeUserGameProfile(this, ugp, callback);
    }

    public void addToDevTeam(DevTeam devTeam,
            AsyncCallback<UpdatedPair<Game, DevTeam>> callback) {
        DataProvider.getGameService().addToDevTeam(this, devTeam, callback);
    }

    public void removeFromDevTeam(DevTeam devTeam,
            AsyncCallback<UpdatedPair<Game, DevTeam>> callback) {
        DataProvider.getGameService().removeFromDevTeam(this, devTeam, callback);
    }


    // Setters

    public void setUrlName(String newName) {
        this.urlName = newName;
    }

    public void setIsPublic(Boolean newIsPublic) {
        this.isPublic = newIsPublic;
    }

    public void setDevTeamKey(Long devTeam) {
        this.devTeam = devTeam;
    }

    public void setTotalRatings(Integer totalRatings) {
        this.totalRatings = totalRatings;
    }

    public void setUserRating(Integer userRating) {
        this.userRating = userRating;
    }

    public void setGameToken(String gameToken) {
        this.gameToken = gameToken;
    }

    public void setActiveVersion(GameVersion activeVersion) {
        this.activeVersion = activeVersion;
    }

    // ModelClass - common methods

    @Override
    public Class<Game> getClassType() {
        return Game.class;
    }

    public void create(AsyncCallback<Game> callback) {
        DataProvider.getGameService().create(this, callback);
    }

    public void remove(AsyncCallback<Void> callback) {
        DataProvider.getGameService().remove(this, callback);
    }

    @Override
    public void write(AsyncCallback<Game> callback) {
        DataProvider.getGameService().write(this, callback);
    }


    // Static methods

    public static void getByKey(Long key, AsyncCallback<Game> callback) {
        DataProvider.getGameService().getByKey(key, callback);
    }

    public static void getByName(String devteamName, String name,
        AsyncCallback<Game> callback) {
        DataProvider.getGameService().getByName(devteamName, name, callback);
    }

    public static void getByGameToken(String gameToken, AsyncCallback<Game> callback) {
        DataProvider.getGameService().getByGameToken(gameToken, callback);
    }

    public static void getAllGames(int startIndex, int count,
            AsyncCallback<ModelCollection<Game>> callback) {
        DataProvider.getGameService().getGames(startIndex, count, callback);
    }

    public static void getActiveGames(int startIndex, int count,
            AsyncCallback<ModelCollection<Game>> callback) {
        DataProvider.getGameService().getActiveGames(startIndex, count, callback);
    }

    public static void getMyGames(int startIndex, int count,
            AsyncCallback<ModelCollection<Game>> callback) {
        DataProvider.getGameService().getMyGames(startIndex, count, callback);
    }

}
