/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class GameFile extends ModelClientClass<Long, GameFile> {

    private static final long serialVersionUID = 6382996609407684393L;

    // Fields   
    
    private String path; //path to the file relative to the Game URL
    
    private String mimeType; //mime type of the file

    private String blobKey; //key to the actual blob
    
    private Long version; //Game versions this file belongs to

    // Constructors
    public GameFile() {
        this.version = null;
        this.mimeType = null;
        this.blobKey = null;
        this.path = null;
    }

    // Getters
    
    public String getPath() {
        return this.path;
    }

    public Long getGameVersionKey() {
        return this.version;
    }
    
    public String getMimeType() {
        return mimeType;
    }

    public String getBlobKey() {
        return blobKey;
    }
    

    // Setters
    
    public void setPath(String path) {
        this.path = path;
    }

    public void setGameVersionKey(Long version) {
        this.version = version;
    }

    public void setMimeType(String mimeType) {
        this.mimeType = mimeType;
    }

    public void setBlobKey(String blobKey) {
        this.blobKey = blobKey;
    }
    
    
    // ModelClass - common methods

    @Override
    public Class<GameFile> getClassType() {
        return GameFile.class;
    }
    
    public void create(AsyncCallback<GameFile> callback) {
        DataProvider.getGameFileService().create(this, callback);
    }
    
    public void remove(AsyncCallback<Void> callback) {
        DataProvider.getGameFileService().remove(this, callback);
    }
    
    @Override
    public void write(AsyncCallback<GameFile> callback) {
        DataProvider.getGameFileService().write(this, callback);
    }
    
    
    // Static methods
    
    public static void getByKey(Long key, AsyncCallback<GameFile> callback) {
        DataProvider.getGameFileService().getByKey(key, callback);
    }
    
    public static void getByPath(GameVersion gameVersion, String path, AsyncCallback<GameFile> callback) {
        DataProvider.getGameFileService().getByPath(gameVersion, path, callback);
    }
    
    public static void getByPath(Game game, int gameVersion, String path, AsyncCallback<GameFile> callback) {
        DataProvider.getGameFileService().getByPath(game, gameVersion, path, callback);
    }
    
    public static void getByPath(String devteamName, String gameName,
            int gameVersion, String path, AsyncCallback<GameFile> callback) {
        DataProvider.getGameFileService().getByPath(devteamName, gameName,
            gameVersion, path, callback);
    }
        
}
