/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import au.edu.unimelb.csse.mugle.shared.model.annotations.MugleDataWrapper;

import com.google.gwt.user.client.rpc.AsyncCallback;

@MugleDataWrapper
public class GameVersion extends ModelClientClass<Long, GameVersion>  {

    private static final long serialVersionUID = -8748810181728458783L;

    // Fields

    private Long game;

    private String dispVersion;

    private Integer version;

    private String fullName;

    private String description; //Do not know if there are restrictions on Length for JDO

    //TODO: Decide on package to use for Image handling

    private Long promotedGame;

    private String entryPoint;


    // Constructors

    public GameVersion() {

        this.promotedGame = null;
        this.game = null;
        this.version = null;
        this.dispVersion = null;
        this.fullName = null;
        this.description = null;
        //TODO: Image

    }


    // Getters

    public Long getGameKey() {
        return this.game;
    }

    public String getDisplayVersion() {
        return this.dispVersion;
    }

    public String getFullName() {
        return this.fullName;
    }

    public String getDescription() {
        return this.description;
    }

    public Integer getVersion() {
        return version;
    }

    /* TODO:
     *  public <Type> getImage() {
     *      return this.Image;
     *  }
     */

    public Long getPromotedGameKey() {
        return this.promotedGame;
    }

    public String getEntryPoint() {
        return this.entryPoint;
    }

    public void getGameFiles(AsyncCallback<ModelCollection<GameFile>> callback) {
        DataProvider.getGameVersionService().getGameFiles(this, callback);
    }

    public void addGameFile(GameFile gameFile,
            AsyncCallback<UpdatedPair<GameVersion, GameFile>> callback) {
        DataProvider.getGameVersionService().addGameFile(this, gameFile, callback);
    }

    public void removeGameFile(GameFile gameFile,
            AsyncCallback<UpdatedPair<GameVersion, GameFile>> callback) {
        DataProvider.getGameVersionService().removeGameFile(this, gameFile, callback);
    }


    // Setters

    public void setDescription(String description) {
        this.description = description;
    }

    public void setDispVersion(String dispVersion) {
        this.dispVersion = dispVersion;
    }

    public void setFullName(String fullName) {
        this.fullName = fullName;
    }

    public void setGameKey(Long game) {
        this.game = game;
    }

    public void setPromotedGameKey(Long promotedGame) {
        this.promotedGame = promotedGame;
    }

    public void setVersion(Integer version) {
        this.version = version;
    }

    public void setEntryPoint(String entryPoint) {
        this.entryPoint = entryPoint;
    }


    // ModelClass - common methods

    @Override
    public Class<GameVersion> getClassType() {
        return GameVersion.class;
    }

    public void create(AsyncCallback<GameVersion> callback) {
        DataProvider.getGameVersionService().create(this, callback);
    }

    public void remove(AsyncCallback<Void> callback) {
        DataProvider.getGameVersionService().remove(this, callback);
    }

    @Override
    public void write(AsyncCallback<GameVersion> callback) {
        DataProvider.getGameVersionService().write(this, callback);
    }


    // Static methods

    public static void getByKey(Long key, AsyncCallback<GameVersion> callback) {
        DataProvider.getGameVersionService().getByKey(key, callback);
    }

    public static void getByVersion(Game game, int version, AsyncCallback<GameVersion> callback) {
        DataProvider.getGameVersionService().getByVersion(game, version, callback);
    }

}
