/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import java.io.Serializable;
import java.util.Date;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.IsSerializable;

/**
 * Base model class that should be extended by all shared model class entities.
 * @param <S> The primary key type
 * @param <T> The client class type (usually self type)
 * @param <U> The server class type (usually the data counter part type)
 */

public abstract class ModelClientClass<S,T extends ModelClientClass<S,?>> implements Serializable, IsSerializable {

    private static final long serialVersionUID = -2271296896173152105L;

    private Long createdUser = null;

    private Date createdTime = null;

    private Long updatedUser = null;

    private Date updatedTime = null;

    private S id = null;

    private boolean userCanEdit;

    /**
     * The getter for the primary key
     * @return The primary key
     */
    public S getPrimaryKey() {
        return this.id;
    }

    /**
     * The setter for the primary key
     */
    public void setPrimaryKey(S key) {
        this.id = key;
    }

    public abstract Class<T> getClassType();

    public abstract void write(AsyncCallback<T> callback);

    public Long getCreatedUser() {
        return createdUser;
    }

    public void setCreatedUser(Long createdUser) {
        this.createdUser = createdUser;
    }

    public Date getCreatedTime() {
        return createdTime;
    }

    public void setCreatedTime(Date createdTime) {
        this.createdTime = createdTime;
    }

    public Long getUpdatedUser() {
        return this.updatedUser;
    }

    public void setUpdatedUser(Long updatedUser) {
        this.updatedUser = updatedUser;
    }

    public Date getUpdatedTime() {
        return this.updatedTime;
    }

    public void setUpdatedTime(Date updatedTime) {
        this.updatedTime = updatedTime;
    }

    /** True if the current user can edit this object.
     */
    public boolean userCanEdit() {
        return this.userCanEdit;
    }

    public void setUserCanEdit() {
        this.userCanEdit = true;
    }
}
