/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import java.io.Serializable;

import com.google.gwt.user.client.rpc.IsSerializable;

/**
 * Holder of two ModelClass objects that is used for AsyncCallbacks after 
 * doing updates to a particular object.
 * @param <S> The first object.
 * @param <T> The second object.
 */
public class UpdatedPair<S extends ModelClientClass<?,?>, T extends ModelClientClass<?,?>> 
    implements Serializable, IsSerializable {

    private static final long serialVersionUID = 4807655751603914825L;

    private S object1 = null;
    
    private T object2 = null;
    
    /**
     * Default constructor
     */
    public UpdatedPair() {
        this.object1 = null;
        this.object2 = null;
    }
    
    public UpdatedPair(S object1, T object2) {
        this.object1 = object1;
        this.object2 = object2;
    }

    public S getObject1() {
        return this.object1;
    }

    public void setObject1(S object1) {
        this.object1 = object1;
    }

    public T getObject2() {
        return this.object2;
    }

    public void setObject2(T object2) {
        this.object2 = object2;
    }
    
}
