/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class User extends ModelClientClass<Long, User> {

    private static final long serialVersionUID = -1044895784157484073L;

    //Fields
    
    private String googleID; //Unique id provided by google
    
    private String urlName; //Must be unique

    private String fullName;

    private String email;

    private Role role;

    private Boolean active;


    // Constructors

    public User() {
        this.googleID = null;
        this.urlName = null;
        this.fullName = null;
        this.email = null;
        this.role = null;
        this.active = null;
        
    }
    
    
    // Getters

    public String getUrlName() {
        return this.urlName;
    }

    public String getFullName() {
        return this.fullName;
    }

    public String getEmail() {
        return this.email;
    }

    public Boolean isActive() {
        return active;
    }

    public Role getRole() {
        return this.role;
    }
    
    public String getGoogleID() {
        return googleID;
    }
    
    public void getDevTeams(AsyncCallback<ModelCollection<DevTeam>> callback) {
        DataProvider.getUserService().getDevTeams(this, callback);
    }
    
    public void joinDevTeam(DevTeam devTeam,
            AsyncCallback<UpdatedPair<User, DevTeam>> callback) {
        DataProvider.getUserService().joinDevTeam(this, devTeam, callback);
    }
    
    public void leaveDevTeam(DevTeam devTeam,
            AsyncCallback<UpdatedPair<User, DevTeam>> callback) {
        DataProvider.getUserService().leaveDevTeam(this, devTeam, callback);
    }
        
    public void getUserGameProfiles(AsyncCallback<ModelCollection<UserGameProfile>> callback) {
        DataProvider.getUserService().getUserGameProfiles(this, callback);
    }
    
    public void addUserGameProfile(UserGameProfile ugp,
            AsyncCallback<UpdatedPair<User, UserGameProfile>> callback) {
        DataProvider.getUserService().addUserGameProfile(this, ugp, callback);
    }
    
    public void removeUserGameProfile(UserGameProfile ugp,
            AsyncCallback<UpdatedPair<User, UserGameProfile>> callback) {
        DataProvider.getUserService().removeUserGameProfile(this, ugp, callback);
    }


    // Setters
    
    /* Ideally url names should be static, so this should be discouraged, and can
     * only be changed by an admin - need an Admin Control panel class perhaps?
     */
    public void setUrlName(String newName) {
        //TODO: check that newName is unique.
        this.urlName = newName;
    }
    
    /* This can only be changed by an Admin, again we need an Admin control panel */
    public void setRole(Role newRole) {
        this.role = newRole;
    }
        
    public void setFullName(String newName) {
        this.fullName = newName;
    }
    
    public void setEmail(String newEmail) {
        //TODO: check that email is unique and valid
        this.email = newEmail;
    }

    public void setActive(Boolean active) {
        this.active = active;
    }

    public void setGoogleID(String googleID) {
        this.googleID = googleID;
    }
        
    
    // ModelClass - common methods
    
    @Override
    public Class<User> getClassType() {
        return User.class;
    }
    
    public void create(AsyncCallback<User> callback) {
        DataProvider.getUserService().create(this, callback);
    }
    
    @Override
    public void write(AsyncCallback<User> callback) {
        DataProvider.getUserService().write(this, callback);
    }
    
    
    // Static methods
    
    public static void getByKey(Long key, AsyncCallback<User> callback) {
        DataProvider.getUserService().getByKey(key, callback);
    }
    
    public static void getByName(String name, AsyncCallback<User> callback) {
        DataProvider.getUserService().getByName(name, callback);
    }
    
    public static void getLoggedInUser(AsyncCallback<User> callback) {
        DataProvider.getUserService().getLoggedInUser(callback);
    }
    
    public static void getUsers(int startIndex, int count, 
                            AsyncCallback<ModelCollection<User>> callback) {
        DataProvider.getUserService().getUsers(startIndex, count, callback);
    }
    
    public static void getUsersByRole(Role role, int startIndex, int count,
                            AsyncCallback<ModelCollection<User>> callback) {
        DataProvider.getUserService().getUsersByRole(role, startIndex, count, callback);
    }
    
}
