/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class UserAchievement extends ModelClientClass<Long, UserAchievement> {

    private static final long serialVersionUID = -6880774872825946912L;

    // Fields
    
    private Long ugp;
    
    private Long achievement;
    
    private Boolean achieved;
    
    private Integer progress;
    
    // Constructors

    public UserAchievement() {
        this.ugp = null;
        this.achievement = null;
        this.achieved = null;
        this.progress = null;
    }
    
    // Getters
    
    public Long getUserGameProfileKey() {
        return this.ugp;
    }
    
    public Long getAchievementKey() {
        return this.achievement;
    }
    
    public Boolean getAchieved(){
        return this.achieved;
    }
    
    public Integer getProgress(){
        return this.progress;
    }
    
    public void addToUserGameProfile(UserGameProfile ugp,
            AsyncCallback<UpdatedPair<UserAchievement, UserGameProfile>> callback) {
        DataProvider.getUserAchievementService().addToUserGameProfile(this, ugp, callback);
    }
    
    public void removeFromUserGameProfile(UserGameProfile ugp,
            AsyncCallback<UpdatedPair<UserAchievement, UserGameProfile>> callback) {
        DataProvider.getUserAchievementService().removeFromUserGameProfile(this, ugp, callback);
    }
    
    public void addToAchievement(Achievement achievement,
            AsyncCallback<UpdatedPair<UserAchievement, Achievement>> callback) {
        DataProvider.getUserAchievementService().addToAchievement(this, achievement, callback);
    }
    
    public void removeFromAchievement(Achievement achievement,
            AsyncCallback<UpdatedPair<UserAchievement, Achievement>> callback) {
        DataProvider.getUserAchievementService().removeFromAchievement(this, achievement, callback);
    }
    

    // Setters

    public void setAchieved(Boolean achieved) {
        this.achieved = achieved;
    }

    public void setAchievementKey(Long achievement) {
        this.achievement = achievement;
    }

    public void setProgress(Integer progress) {
        this.progress = progress;
    }

    public void setUserGameProfileKey(Long ugp) {
        this.ugp = ugp;
    }
    
    
    // ModelClass - common methods

    @Override
    public Class<UserAchievement> getClassType() {
        return UserAchievement.class;
    }
    
    @Override
    public void write(AsyncCallback<UserAchievement> callback) {
        DataProvider.getUserAchievementService().write(this, callback);
    }
    
    
    // Static methods
    
    public static void getByKey(Long key, AsyncCallback<UserAchievement> callback) {
        DataProvider.getUserAchievementService().getByKey(key, callback);
    }
    
    public static void getByUserGameProfile(UserGameProfile ugp, Achievement achievement,
                           AsyncCallback<UserAchievement> callback) {
        DataProvider.getUserAchievementService().getByUserGameProfile(ugp, achievement, callback);
    }
 
}
