/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.model;

import com.google.gwt.user.client.rpc.AsyncCallback;

import au.edu.unimelb.csse.mugle.shared.model.annotations.*;

@MugleDataWrapper
public class UserGameProfile extends ModelClientClass<Long, UserGameProfile> {
    
    private static final long serialVersionUID = 6851639153971063347L;
    
    // Fields
    
    private Integer highscore;

    private Long user;
 
    private Long game;


    // Constructors

    public UserGameProfile() {
        this.highscore = null;
        this.user = null;
        this.game = null;
    }
    
    // Getters
    
    public Long getUserKey() {
        return this.user;
    }
    
    public Long getGameKey() {
        return this.game;
    }
    
    public Integer getHighscore() {
        return this.highscore;
    }
    
    public void getUserAchievements(AsyncCallback<ModelCollection<UserAchievement>> callback) {
        DataProvider.getUserGameProfileService().getUserAchievements(this, callback);
    }
    
    public void addUserAchievement(UserAchievement userAhievement,
            AsyncCallback<UpdatedPair<UserGameProfile, UserAchievement>> callback) {
        DataProvider.getUserGameProfileService().addUserAchievement(this, userAhievement, callback);
    }
    
    public void removeUserAchievement(UserAchievement userAhievement,
            AsyncCallback<UpdatedPair<UserGameProfile, UserAchievement>> callback) {
        DataProvider.getUserGameProfileService().removeUserAchievement(this, userAhievement, callback);
    }
      
    public void getKeyValuePairs(AsyncCallback<ModelCollection<KeyValuePair>> callback) {
        DataProvider.getUserGameProfileService().getKeyValuePairs(this, callback);
    }
    
    public void addKeyValuePair(KeyValuePair kvp,
            AsyncCallback<UpdatedPair<UserGameProfile, KeyValuePair>> callback) {
        DataProvider.getUserGameProfileService().addKeyValuePair(this, kvp, callback);
    }
    
    public void removeKeyValuePair(KeyValuePair kvp,
            AsyncCallback<UpdatedPair<UserGameProfile, KeyValuePair>> callback) {
        DataProvider.getUserGameProfileService().removeKeyValuePair(this, kvp, callback);
    }
    
    public void addToUser(User user,
            AsyncCallback<UpdatedPair<UserGameProfile, User>> callback) {
        DataProvider.getUserGameProfileService().addToUser(this, user, callback);
    }
    
    public void removeFromUser(User user,
            AsyncCallback<UpdatedPair<UserGameProfile, User>> callback) {
        DataProvider.getUserGameProfileService().removeFromUser(this, user, callback);
    }
    
    public void addToGame(Game game,
            AsyncCallback<UpdatedPair<UserGameProfile, Game>> callback) {
        DataProvider.getUserGameProfileService().addToGame(this, game, callback);
    }
    
    public void removeFromGame(Game game,
            AsyncCallback<UpdatedPair<UserGameProfile, Game>> callback) {
        DataProvider.getUserGameProfileService().removeFromGame(this, game, callback);
    }
    
    
    // Setters

    public void setGameKey(Long game) {
        this.game = game;
    }

    public void setUserKey(Long user) {
        this.user = user;
    }

    /* can only be set by the current Game */
    public void setHighscore(Integer newScore) {
        this.highscore = newScore;
    }
    
    
    // ModelClass - common methods

    @Override
    public Class<UserGameProfile> getClassType() {
        return UserGameProfile.class;
    }
    
    @Override
    public void write(AsyncCallback<UserGameProfile> callback) {
        DataProvider.getUserGameProfileService().write(this, callback);
    }
    
    
    // Static methods
    
    public static void getByKey(Long key, AsyncCallback<UserGameProfile> callback) {
        DataProvider.getUserGameProfileService().getByKey(key, callback);
    }
    
    public static void getByUserAndGame(User user, Game game, AsyncCallback<UserGameProfile> callback) {
        DataProvider.getUserGameProfileService().getByUserAndGame(user, game, callback);
    }

}
