/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.shared.model.services;

import au.edu.unimelb.csse.mugle.shared.api.GameTokenError;
import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.*;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("data-achievement")
public interface AchievementService extends RemoteService {
    
    Achievement getByKey(Long key)
        throws ServerException, AchievementNotExists;

    Achievement getByName(String name, Game game)
        throws ServerException, AchievementNotExists, GameTokenError;

    Achievement getByName(String name, String gameToken)
            throws ServerException, AchievementNotExists, GameTokenError;

    Achievement create(Achievement object)
        throws ServerException, UserPrivilegeException, AchievementExists, GameNotExists;

    Achievement write(Achievement object)
        throws ServerException, UserPrivilegeException, AchievementNotExists, GameNotExists;

    ModelCollection<UserAchievement> getUserAchievements(Achievement achievement)
        throws ServerException, AchievementNotExists;
    
    UpdatedPair<Achievement, UserAchievement>
        addUserAchievement(Achievement achievement, UserAchievement userAchievement)
            throws ServerException, AchievementNotExists, UserAchievementNotExists, UserPrivilegeException, GameNotExists;
    
    UpdatedPair<Achievement, UserAchievement>
        removeUserAchievement(Achievement achievement, UserAchievement userAchievement)
            throws ServerException, AchievementNotExists, UserAchievementNotExists, UserPrivilegeException, GameNotExists;
    
    void remove(Achievement object)
        throws ServerException, UserPrivilegeException, AchievementNotExists, GameNotExists;

    void remove(ModelCollection<Achievement> achievements)
        throws ServerException, UserPrivilegeException, AchievementNotExists, GameNotExists;
}
