/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.shared.model.services;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.*;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("data-dev-team")
public interface DevTeamService extends RemoteService {

    DevTeam getByKey(Long key)
        throws ServerException, DevTeamNotExists, UserPrivilegeException;

    public DevTeam getByName(String name)
        throws ServerException, DevTeamNotExists, UserPrivilegeException;

    public DevTeam create(DevTeam object)
        throws ServerException, DevTeamExists, UserPrivilegeException;

    DevTeam write(DevTeam object)
        throws ServerException, DevTeamNotExists, UserPrivilegeException;

    ModelCollection<Game> getGames(DevTeam devTeam, Boolean includePrivate)
        throws ServerException, DevTeamNotExists, UserPrivilegeException;

    UpdatedPair<DevTeam, Game>
        addGame(DevTeam devTeam, Game game)
            throws ServerException, DevTeamNotExists, GameNotExists, UserPrivilegeException;

    UpdatedPair<DevTeam, Game>
        removeGame(DevTeam devTeam, Game game)
            throws ServerException, DevTeamNotExists, GameNotExists, UserPrivilegeException;

    ModelCollection<User> getUsers(DevTeam devTeam)
        throws ServerException, DevTeamNotExists, UserPrivilegeException;

    UpdatedPair<DevTeam, User>
        addUser(DevTeam devTeam, User user)
            throws ServerException, DevTeamNotExists, UserNotExists, UserPrivilegeException;

    UpdatedPair<DevTeam, User>
        removeUser(DevTeam devTeam, User user)
            throws ServerException, DevTeamNotExists, UserNotExists, UserPrivilegeException;

    ModelCollection<DevTeam> getDevTeams(int startIndex, int count)
        throws ServerException, UserPrivilegeException;

    void remove(DevTeam object)
        throws ServerException, UserPrivilegeException, DevTeamNotExists;

    void remove(ModelCollection<DevTeam> teams)
        throws ServerException, UserPrivilegeException, DevTeamNotExists;
}
