/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.shared.model.services;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.*;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("data-game")
public interface GameService extends RemoteService {

    Game getByKey(Long key)
        throws ServerException, GameNotExists;

    Game getByName(String devteamName, String name)
        throws ServerException, GameNotExists;

    Game getByGameToken(String token)
        throws ServerException, GameNotExists;

    Game create(Game object)
        throws ServerException, UserPrivilegeException, DevTeamNotExists,
               GameExists;

    Game write(Game object)
        throws ServerException, UserPrivilegeException, GameNotExists;

    ModelCollection<Achievement> getAchievements(Game game)
        throws ServerException, GameNotExists;

    UpdatedPair<Game, Achievement>
        addAchievement(Game game, Achievement achievement)
            throws ServerException, GameNotExists, AchievementNotExists, UserPrivilegeException;

    UpdatedPair<Game, Achievement>
        removeAchievement(Game game, Achievement achievement)
            throws ServerException, GameNotExists, AchievementNotExists, UserPrivilegeException;

    ModelCollection<GameVersion> getGameVersions(Game game)
        throws ServerException, GameNotExists;

    UpdatedPair<Game, GameVersion>
        addGameVersion(Game game, GameVersion version)
            throws ServerException, GameNotExists, GameVersionNotExists, UserPrivilegeException;

    UpdatedPair<Game, GameVersion>
        removeGameVersion(Game game, GameVersion version)
            throws ServerException, GameNotExists, GameVersionNotExists, UserPrivilegeException;

    ModelCollection<UserGameProfile> getUserGameProfiles(Game game)
        throws ServerException, GameNotExists;

    UpdatedPair<Game, UserGameProfile>
        addUserGameProfile(Game game, UserGameProfile ugp)
            throws ServerException, GameNotExists, UserGameProfileNotExists, UserPrivilegeException;

    UpdatedPair<Game, UserGameProfile>
        removeUserGameProfile(Game game, UserGameProfile ugp)
            throws ServerException, GameNotExists, UserGameProfileNotExists, UserPrivilegeException;

    UpdatedPair<Game, DevTeam>
        addToDevTeam(Game game, DevTeam devTeam)
            throws ServerException, GameNotExists, DevTeamNotExists, UserPrivilegeException;

    UpdatedPair<Game, DevTeam>
        removeFromDevTeam(Game game, DevTeam devTeam)
            throws ServerException, GameNotExists, DevTeamNotExists, UserPrivilegeException;

    void remove(Game object)
        throws ServerException, UserPrivilegeException, GameNotExists;

    ModelCollection<Game> getGames(int startIndex, int count)
        throws ServerException, UserPrivilegeException;

    ModelCollection<Game> getActiveGames(int startIndex, int count)
        throws ServerException, UserPrivilegeException;

    ModelCollection<Game> getMyGames(int startIndex, int count)
        throws ServerException, UserPrivilegeException;


}
