/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.shared.model.services;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.*;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("data-ugp")
public interface UserGameProfileService extends RemoteService {

    UserGameProfile getByKey(Long key) 
        throws ServerException, UserGameProfileNotExists;
    
    UserGameProfile getByUserAndGame(User user, Game game) 
        throws ServerException, UserGameProfileNotExists, UserNotExists, GameNotExists;

    UserGameProfile write(UserGameProfile object)
        throws ServerException, UserPrivilegeException, UserGameProfileNotExists;

    ModelCollection<KeyValuePair> getKeyValuePairs(UserGameProfile ugp)
        throws ServerException, UserGameProfileNotExists;
    
    UpdatedPair<UserGameProfile, KeyValuePair>
        addKeyValuePair(UserGameProfile ugp, KeyValuePair kvp)
            throws ServerException, UserGameProfileNotExists, KeyValuePairNotExists, UserPrivilegeException;
    
    UpdatedPair<UserGameProfile, KeyValuePair>
        removeKeyValuePair(UserGameProfile ugp, KeyValuePair kvp)
            throws ServerException, UserGameProfileNotExists, KeyValuePairNotExists, UserPrivilegeException;
    
    ModelCollection<UserAchievement> getUserAchievements(UserGameProfile ugp)
        throws ServerException, UserGameProfileNotExists;
    
    UpdatedPair<UserGameProfile, UserAchievement>
        addUserAchievement(UserGameProfile ugp, UserAchievement userAchievement)
            throws ServerException, UserGameProfileNotExists, UserAchievementNotExists, UserPrivilegeException;
    
    UpdatedPair<UserGameProfile, UserAchievement>
        removeUserAchievement(UserGameProfile ugp, UserAchievement userAchievement)
            throws ServerException, UserGameProfileNotExists, UserAchievementNotExists, UserPrivilegeException;
    
    UpdatedPair<UserGameProfile, User>
        addToUser(UserGameProfile ugp, User user)
            throws ServerException, UserGameProfileNotExists, UserNotExists, UserPrivilegeException;

    UpdatedPair<UserGameProfile, User>
        removeFromUser(UserGameProfile ugp, User user)
            throws ServerException, UserGameProfileNotExists, UserNotExists, UserPrivilegeException;
    
    UpdatedPair<UserGameProfile, Game>
        addToGame(UserGameProfile ugp, Game game)
            throws ServerException, UserGameProfileNotExists, GameNotExists, UserPrivilegeException;
    
    UpdatedPair<UserGameProfile, Game>
        removeFromGame(UserGameProfile ugp, Game game)
            throws ServerException, UserGameProfileNotExists, GameNotExists, UserPrivilegeException;
    
}
