/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package au.edu.unimelb.csse.mugle.shared.model.services;

import au.edu.unimelb.csse.mugle.shared.model.*;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.*;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("data-user")
public interface UserService extends RemoteService {

    User getByKey(Long key)
        throws ServerException, UserNotExists;
    
    User getByName(String urlName)
        throws ServerException, UserNotExists;

    User getLoggedInUser()
        throws ServerException;

    User create(User object)
        throws ServerException, UserPrivilegeException, UserExists;
    
    User write(User object)
        throws ServerException, UserPrivilegeException, UserExists,
               UserNotExists;
    
    ModelCollection<DevTeam> getDevTeams(User user)
        throws ServerException, UserNotExists;
    
    UpdatedPair<User, DevTeam>
        joinDevTeam(User user, DevTeam devTeam)
            throws ServerException, UserNotExists, DevTeamNotExists, UserPrivilegeException;
    
    UpdatedPair<User, DevTeam>
        leaveDevTeam(User user, DevTeam devTeam)
            throws ServerException, UserNotExists, DevTeamNotExists, UserPrivilegeException;
    
    ModelCollection<UserGameProfile> getUserGameProfiles(User user)
        throws ServerException, UserNotExists;
    
    UpdatedPair<User, UserGameProfile>
        addUserGameProfile(User user, UserGameProfile ugp)
            throws ServerException, UserNotExists, UserGameProfileNotExists, UserPrivilegeException;

    UpdatedPair<User, UserGameProfile>
        removeUserGameProfile(User user, UserGameProfile ugp)
            throws ServerException, UserNotExists, UserGameProfileNotExists, UserPrivilegeException;
    
    ModelCollection<User> getUsers(int startIndex, int count)
        throws UserPrivilegeException, ServerException;

    ModelCollection<User> getUsersByRole(Role role, int startIndex, int count)
        throws UserPrivilegeException, ServerException;

}
