/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.multiplayer.api;

import java.io.Serializable;

import com.google.gwt.user.client.rpc.IsSerializable;

/** Information about a game instance */
public class GameInfo implements Serializable, IsSerializable {
    /**
	 * 
	 */
    private static final long serialVersionUID = -7004700355133481177L;
    private String name;
    private int playerCount;
    private int size;
    private String token;
    private boolean canContinue = false;

    public GameInfo(String name, int numOfPlayer, int size, String token) {
        this.setName(name);
        this.setSize(size);
        this.setPlayerCount(numOfPlayer);
        this.setToken(token);
    }

    public GameInfo() {
    }

    /**
     * Return the game instance name
     * 
     * @return the game instance name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the game instance name
     * 
     * @param name
     *            the game instance name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Return the number of player
     * 
     * @return the number of player in this game instance
     */
    public int getPlayerCount() {
        return playerCount;
    }

    /**
     * Set the number of player
     * 
     * @param playerCount
     *            the number of player in this game instance
     */
    public void setPlayerCount(int playerCount) {
        this.playerCount = playerCount;
    }

    /**
     * Return the maximum number of player this game instance allows
     * 
     * @return the maximum number of player this game instance allows
     */
    public int getSize() {
        return size;
    }

    /**
     * Return true if this game has room to join
     * 
     * @return true if this game has room to join
     */
    public boolean hasRoom() {
        return playerCount <= size;
    }

    /**
     * Set the maximum number of player this game instance allows
     * 
     * @param size
     *            the maximum number of player this game instance allows
     */
    public void setSize(int size) {
        this.size = size;
    }

    /**
     * Return the game instance token
     * 
     * @return the game instance token
     */
    public String getToken() {
        return token;
    }

    /**
     * Set the game instance token
     * 
     * @param token
     *            the game instance token
     */
    public void setToken(String token) {
        this.token = token;
    }

    /**
     * Return true if this player once joined this game and can continue
     * 
     * @return true if this player once joined this game and can continue
     */
    public boolean canContinue() {
        return canContinue;
    }

    /**
     * Set whether the player can continue playing in this game
     */
    public void setCanContinue() {
        this.canContinue = true;
    }
}
