package au.edu.unimelb.csse.mugle.shared.multiplayer.api;

import java.io.Serializable;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import com.google.gwt.user.client.rpc.IsSerializable;

/** A transaction represents a series of actions that are run atomically. */

public class Transaction implements Serializable, IsSerializable {

    /**
     * 
     */
    private static final long serialVersionUID = -5640362233846171616L;
    private LinkedList<Action> actions = new LinkedList<Action>();
    private boolean isInstanceGameProfile = true;
    private String gameToken;
    private String instanceToken;

    public Transaction add(Action action) {
        actions.add(action);
        return this;
    }

    public LinkedList<Action> getActions() {
        return actions;
    }

    /**
     * To set whether the transaction is operating on game instance user game
     * profile.
     * 
     * @param bool
     *            True if the transaction is operating on game instance user
     *            game profile. Otherwise it is on game user game profile which
     *            is shared by all the game instances of a particular game.
     */
    public void setIsInstanceGameProfile(Boolean bool) {
        this.isInstanceGameProfile = bool;
    }

    public boolean isInstanceGameProfile() {
        return this.isInstanceGameProfile;
    }

    public Set<String> getUsers() {
        Set<String> users = new HashSet<String>();
        for (Action action : actions) {
            for (String name : action.getUsers()) {
                users.add(name);
            }
        }
        return users;
    }

    /**
     * To tell whether two transactions are equivalent.
     * 
     * @param transaction
     *            The transaction to be compared.
     * @return
     */
    public boolean compareTo(Transaction transaction) {
        return this.toString().equals(transaction.toString());
    }

    @Override
    public String toString() {
        String actionStrings = "";
        for (Action action : actions) {
            actionStrings = actionStrings + action.actionString();
        }
        actionStrings += " isInstanceGameProfile: " + isInstanceGameProfile;
        return actionStrings;
    }

    public String getGameToken() {
        return gameToken;
    }

    public void setGameToken(String gameToken) {
        this.gameToken = gameToken;
    }

    public String getInstanceToken() {
        return instanceToken;
    }

    public void setInstanceToken(String instanceToken) {
        this.instanceToken = instanceToken;
    }
}
