package au.edu.unimelb.csse.mugle.shared.multiplayer.api;

import java.util.ArrayList;
import java.util.List;

/** An action that transfers an item from a user profile to the other. */
public class Transfer implements Action {

    /**
     * 
     */
    private static final long serialVersionUID = -7557405166090521260L;

    private String transferFrom;

    private String transferTo;

    private String key;

    public Transfer() {

    }

    /**
     * Constructor for the transfer action.
     * 
     * @param transferFrom
     *            The user name which the item transfered from.
     * @param transferTo
     *            The user name which the item transfered to.
     * @param key
     *            The key of the item in the key-value store.
     */
    public Transfer(String transferFrom, String transferTo, String key) {
        this.transferFrom = transferFrom;
        this.transferTo = transferTo;
        this.key = key;
    }

    /**
     * Return the transferrer.
     * 
     * @return The name of the transferrer.
     */
    public String getTransferFrom() {
        return this.transferFrom;
    }

    /**
     * Return the transferee.
     * 
     * @return The name of the transferee.
     */
    public String getTransferTo() {
        return this.transferTo;
    }

    /**
     * The key name of the item to be transfered.
     * 
     * @return The key of the item in the key-value store.
     */
    public String getKey() {
        return this.key;
    }

    @Override
    public List<String> getUsers() {
        ArrayList<String> users = new ArrayList<String>();
        users.add(transferFrom);
        users.add(transferTo);
        return users;
    }

    @Override
    public String actionString() {
        return "transfer " + key + " from: " + transferFrom + " transfer to: "
                + transferTo;
    }

}
