/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.multiplayer.model;

import java.util.Date;

import au.edu.unimelb.csse.mugle.shared.model.Game;
import au.edu.unimelb.csse.mugle.shared.model.ModelClientClass;
import au.edu.unimelb.csse.mugle.shared.model.ModelCollection;
import au.edu.unimelb.csse.mugle.shared.model.ModelList;
import au.edu.unimelb.csse.mugle.shared.model.User;
import au.edu.unimelb.csse.mugle.shared.model.annotations.MugleDataWrapper;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.services.MultiPlayerPlatformServices;

import com.google.gwt.user.client.rpc.AsyncCallback;

@MugleDataWrapper
public class GameInstance extends ModelClientClass<Long, GameInstance> {

    /**
	 * 
	 */
    private static final long serialVersionUID = 4557542841025807642L;

    private String token;

    private String name; // the same as the creator's url name

    private boolean active;

    private boolean hasLog;

    private int playerCount;

    private Date endedTime;

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public boolean hasLog() {
        return hasLog;
    }

    public void setHasLog(boolean hasLog) {
        this.hasLog = hasLog;
    }

    public int getPlayerCount() {
        return playerCount;
    }

    public void setPlayerCount(int playerCount) {
        this.playerCount = playerCount;
    }

    public Date getEndedTime() {
        return endedTime;
    }

    public void setEndedTime(Date endedTime) {
        this.endedTime = endedTime;
    }

    @Override
    public Class<GameInstance> getClassType() {
        return GameInstance.class;
    }

    @Override
    public void write(AsyncCallback<GameInstance> callback) {
        throw new UnsupportedOperationException();
    }

    public static void getPlayers(GameInstance instance,
            AsyncCallback<ModelCollection<User>> callback) {
        MultiPlayerPlatformServices.gameMonitor.getPlayers(instance, callback);
    }

    public static void getGameInstances(Game game, int startIndex, int count,
            AsyncCallback<ModelList<GameInstance>> callback) {
        MultiPlayerPlatformServices.gameMonitor.getGameInstances(game, startIndex,
                count, callback);
    }

    public static void getCount(Game game, AsyncCallback<Integer> callback) {
        MultiPlayerPlatformServices.gameMonitor.getInstanceCount(game, callback);
    }

    public enum InstanceField {
        NAME, LOG, CREATE_TIME, END_TIME, PLAYER_COUNT
    }

    public enum Order {
        ASC, DESC
    }

}
