/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published
 *  by the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.multiplayer.model.services;

import au.edu.unimelb.csse.mugle.shared.model.Game;
import au.edu.unimelb.csse.mugle.shared.model.ModelCollection;
import au.edu.unimelb.csse.mugle.shared.model.ModelList;
import au.edu.unimelb.csse.mugle.shared.model.User;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.GameInstanceNotExists;
import au.edu.unimelb.csse.mugle.shared.multiplayer.api.InstanceTokenError;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.GameInstance;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.GameLog;
import au.edu.unimelb.csse.mugle.shared.multiplayer.model.LogMessage;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.GameNotExists;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.ServerException;
import au.edu.unimelb.csse.mugle.shared.platform.exceptions.UserPrivilegeException;

import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

@RemoteServiceRelativePath("data-game-monitor")
public interface GameMonitorService extends RemoteService {
    ModelList<GameInstance> getGameInstances(Game game, int startIndex,
            int count) throws GameNotExists, ServerException,
            UserPrivilegeException;

    ModelList<GameInstance> getSortedGameInstances(Game game, int startIndex,
            int count, String sortBy) throws GameNotExists, ServerException,
            UserPrivilegeException;

    int getInstanceCount(Game game) throws GameNotExists, ServerException,
            UserPrivilegeException;

    ModelCollection<User> getPlayers(GameInstance instance)
            throws GameInstanceNotExists, ServerException,
            UserPrivilegeException;

    ModelList<GameLog> getGameLogs(String instanceToken)
            throws InstanceTokenError, ServerException, UserPrivilegeException;

    int getLogMessageCount(String logToken) throws ServerException;

    ModelList<LogMessage> getLogMessages(String logToken, int startIndex,
            int count) throws ServerException, UserPrivilegeException;
}
