/*  Melbourne University Game-based Learning Environment
 *  Copyright (C) 2011 The University of Melbourne
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package au.edu.unimelb.csse.mugle.shared.platform.exceptions;

import au.edu.unimelb.csse.mugle.shared.model.GameVersion;

/**
 * Exception used when a given game version does not exist.
 */
public class GameVersionNotExists extends MugleException {

    private static final long serialVersionUID = -7145089113784018080L;

    public GameVersionNotExists() {
        super();
    }
    
    public GameVersionNotExists(GameVersion gameVersion) {
        super("GameVersion '" + gameVersion.getPrimaryKey().toString() + "' does not exist.");
    }
    
    public GameVersionNotExists(Long pkey) {
        super("GameVersion with primary key '" + pkey.toString() + "' does not exist.");
    }

    public GameVersionNotExists(String dispVersion, String gameName) {
        super("Game " + gameName + " does not have version " + dispVersion + ".");
    }

    public GameVersionNotExists(int version, String gameName) {
        super("Game " + gameName + " does not have version " + version + ".");
    }

}